/*
 * @brineFrame.java Version 1.1 10/29/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;             // Common Data Structure

import brine.brineListStruct;     // Brine List Structure
import brine.gui.brinePanel;      // Brine Panel

import pc.pcDataStruct;           // Principal Components Data Structure
import pc.gui.pcBrineFrame;       // Brine Frame
import pc.gui.pcBrineExpandFrame; // Expanded Brine Frame

/** Class brineFrame
 *  <p> This Class will create the Stiff Frame.
 *
 *  @version 1.1 10/29/2015
 *  @author  John R. Victorine
 */

public class brineFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private cmnStruct          stCMN    = null;
  private brineListStruct    st       = null;

  // Global Variables

  private Observable         notifier = null; // Observables Object
  private brinePanel         pnl      = null; // Brine Panel

  private pcDataStruct       pcd      = null; // Principal Components Data Structure
  private pcBrineFrame       pcframe  = null; // Frame
  private pcBrineExpandFrame pcExpand = null; // Expanded Frame

  private double             dLimit   = 2.0;  // The Normalization value

  // Global Widgets

  private JTextField txtPercent = new JTextField();

  private JButton btnDelete    = new JButton();
  private JButton btnProcess   = new JButton();
  private JButton btnNormalize = new JButton();
  private JButton btnExit      = new JButton();

  /** Constructor brineFrame()
   *  <p> This is the Constructor for this class.
   *  @param st  = Brine Plot Data Structure
   */

  public brineFrame( cmnStruct stCMN, brineListStruct st )
  {
    try
    {
	  this.stCMN = stCMN;
      this.st    = st;

      jbInit();

      addWindowListener( new brineFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int    iWidth    = 650;
	JPanel pnlBase   = new JPanel();
	JPanel pnlLeft   = new JPanel();
	JPanel pnlCenter = new JPanel();
	JPanel pnlRight  = new JPanel();

    JLabel lblPercent = new JLabel();

    // Create an Observable

    notifier = new brineFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.setTitle("Brine Data");
    this.getContentPane().setLayout(new BorderLayout());

    pnl = new brinePanel( notifier, st );

    // Base Panel

    pnlBase.setLayout(new BorderLayout());

    txtPercent.setText( "2.0" );
    txtPercent.setColumns(3);
    txtPercent.setFont(new java.awt.Font("Dialog", 1, 12));
    txtPercent.addFocusListener(new brineFrameFocusAdapter(this));

    lblPercent.setText(" % from Anions/Cations equal to 1.0" );
    lblPercent.setFont(new java.awt.Font("Dialog", 1, 12));

    btnDelete.setFont(new java.awt.Font("Dialog", 1, 11));
    btnDelete.setText("Delete Well(s)");
    btnDelete.addActionListener(this);

    btnProcess.setFont(new java.awt.Font("Dialog", 1, 11));
    btnProcess.setText("Compute Correlation Matrix");
    btnProcess.addActionListener(this);

    btnNormalize.setFont(new java.awt.Font("Dialog", 1, 11));
    btnNormalize.setText("Begin Normalization Process");
    btnNormalize.addActionListener(this);

    btnExit.setFont(new java.awt.Font("Dialog", 1, 11));
    btnExit.setText("Cancel");
    btnExit.addActionListener(this);

    // Attach Widgets

    this.getContentPane().add(pnl,     BorderLayout.CENTER);

    this.getContentPane().add(pnlBase, BorderLayout.SOUTH);
    pnlBase.add(pnlLeft,               BorderLayout.WEST);
    pnlLeft.add(btnDelete,             null);

    pnlBase.add(pnlCenter,             BorderLayout.CENTER);
    pnlCenter.add(btnProcess,          null);
    pnlCenter.add(btnNormalize,        null);
    pnlCenter.add(txtPercent,          null);
    pnlCenter.add(lblPercent,          null);

    pnlBase.add(pnlRight,              BorderLayout.EAST);
    pnlRight.add(btnExit,              null);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    iWidth = pnl.getWidth();
    if (iWidth == 0)
      iWidth = 650;

    this.setSize(new Dimension(iWidth, d.height-60));

//    this.setLocation((d.width  - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);
    this.setLocation( (d.width  - this.getSize().width) / 2, 10 );

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
	stCMN    = null;
    st       = null;
    notifier = null;

    if (pnl != null)
      pnl.close();
    pnl = null;

    if (pcd != null)
      pcd.delete();
    pcd          = null;  // Principal Components Data Structure

    if (pcframe != null)
      pcframe.close();
    pcframe = null;

    if (pcExpand != null)
      pcExpand.close();
    pcExpand = null;

    txtPercent   = null;

    btnDelete    = null;
    btnProcess   = null;
    btnNormalize = null;
    btnExit      = null;

    dispose();
  }

  /** METHOD brineFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class brineFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */


  /* ======================================================================= *
   * ----------------------------- SET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method setData()
   *  <p> This method will set the principal component Data Structure
   *  @param st = principal component Data Structure
   */

  public void setData()
  {
	int iBrine[] = null;

	if ((pnl != null) && (st != null))
	{
	  iBrine = pnl.getMnemonics();
	  pcd    = pc.pcUtility.getBrineData_meq( iBrine, st );
	}
  }

  /** Method deleteWells()
   *  <p> This method will delete the selected wells.
   */

  private void deleteWells()
  {
	brineListStruct stOut = null;

	if ((pnl != null) && (st != null))
	{
	  if (pnl.getSelected() != null)
	  {
		if (pnl.getSelected().length > 0)
		{
		  stOut = pnl.getData();

		  if (stOut != null)
		  {
			for (int i=0; i<stOut.iCount; i++)
			{
			  st = brine.brineUtility.remove( stOut.stItem[i].sKEY, st );
			}
		  }
		}
	  }

	  pnl.setData(st);
	}
  }

  /** Method setData()
   */

  /* ======================================================================= *
   * --------------------------- DISPLAY FRAMES ---------------------------- *
   * ======================================================================= */

  /** Method process()
   *  <p> This method will begin the analysis
   */

  private void process()
  {
	setData();
//    pc.pcUtility.print(pcd);

    if (pcframe != null)
      pcframe.close();
    pcframe = null;

    pcframe = new pcBrineFrame( pcd );
    pcframe.setData( st );
  }

  /** Method processExpand()
   *  <p> This method will begin the analysis
   */

  private void processExpand()
  {
	setData();

    if (pcExpand != null)
      pcExpand.close();
    pcExpand = null;

    pcExpand = new pcBrineExpandFrame( pcd, dLimit );
    pcExpand.setData( st );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnDelete)    { deleteWells(); }
    if (event.getSource() == btnProcess)   { process(); }
    if (event.getSource() == btnNormalize) { processExpand(); }

    if (event.getSource() == btnExit)      { close(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");

    if (e.getSource() == txtPercent)
    {
      iNumeric = 1;
      sTemp    = txtPercent.getText();
      sMessage = new String("The Percent Value is a Numeric Field");
	}

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
		txtPercent.setText("" + dLimit);
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);
      }
      else
      {
		dLimit = cmn.cmnString.stringToDouble(sTemp);
	  }
	}
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("Data Entry - Add Brine Data")) {  }
  }
}

/** CLASS brineFrameFocusAdapter()
  *  <p> This Method will handle Actions when focus is lost on a textfield
  */

class brineFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  brineFrame adaptee;

  brineFrameFocusAdapter(brineFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS brineFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class brineFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}


/*
 *  @version 1.1 10/29/2015
 *  @author  John Victorine
 */
