/*
 * @brinePanel.java Version 1.1 10/28/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.gui;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;         // Common Data Structure

import brine.gui.brineTable;  // Brine Data Table
import brine.brineListStruct; // Brine List Structure

/** Class brinePanel
 *  <p> This Class will create the Table Panel.
 *
 *  @version 1.1 10/28/2015
 *  @author  John R. Victorine
 */

public class brinePanel extends JPanel implements ActionListener
{
  public static final int _SPGR   = brine.brineStandardTools._SPGR;   // Specific Gravity
  public static final int _SIGMA  = brine.brineStandardTools._SIGMA;  // Specific Conductivity
  public static final int _PH     = brine.brineStandardTools._PH;     // PH
  public static final int _DEG    = brine.brineStandardTools._DEG;    // Degree F
  public static final int _OHM    = brine.brineStandardTools._OHM;    // Resistivity (Rw)
  public static final int _OHM75  = brine.brineStandardTools._OHM75;  // Resistivity at 75 deg
  public static final int _OHME   = brine.brineStandardTools._OHME;   // Estimated Rw

  public static final int _ALKNTY = brine.brineStandardTools._ALKNTY; // Alkalinity
  public static final int _NTU    = brine.brineStandardTools._NTU;    // Turbidity

  // .. Alkali Metal
  public static final int _LI     = brine.brineStandardTools._LI;     // Lithium            1+
  public static final int _NA     = brine.brineStandardTools._NA;     // Sodium             1+
  public static final int _K      = brine.brineStandardTools._K;      // Potassium          1+
  public static final int _RB     = brine.brineStandardTools._RB;     // Rubidium           1+
  public static final int _CS     = brine.brineStandardTools._CS;     // Cesium             1+
  public static final int _NA_K   = brine.brineStandardTools._NA_K;   // Sodium/Potasium    1+

  // ..  Alkali Earth Metal
  public static final int _BE     = brine.brineStandardTools._BE;     // Beryllium          2+
  public static final int _MG     = brine.brineStandardTools._MG;     // Magnesium          2+
  public static final int _CA     = brine.brineStandardTools._CA;     // Calcium            2+
  public static final int _SR     = brine.brineStandardTools._SR;     // Strontium          2+
  public static final int _BA     = brine.brineStandardTools._BA;     // Barium             2+

  // ..  VIB
  public static final int _CR_II  = brine.brineStandardTools._CR_II;  // Chromium (II)   chromous  2+
  public static final int _CR_III = brine.brineStandardTools._CR_III; // Chromium (III)  chromic   3+

  // ..  VIIB
  public static final int _MN_II  = brine.brineStandardTools._MN_II;  // Manganese (II)  manganous 2+
  public static final int _MN_III = brine.brineStandardTools._MN_III; // Manganese (III) manganic  3+

  // ..  VIIIB
  public static final int _FE_II  = brine.brineStandardTools._FE_II;  // Iron (II)    ferrous      2+
  public static final int _FE_III = brine.brineStandardTools._FE_III; // Iron (III)   ferric		 3+
  public static final int _CO_II  = brine.brineStandardTools._CO_II;  // Cobalt (II)  cobaltous    2+
  public static final int _CO_III = brine.brineStandardTools._CO_III; // Cobalt (III) cobaltic     3+
  public static final int _NI_II  = brine.brineStandardTools._NI_II;  // Nickel (II)  nickelous    2+
  public static final int _NI_III = brine.brineStandardTools._NI_III; // Nickel (III) nickelic     3+

  // ..  IB
  public static final int _CU_I   = brine.brineStandardTools._CU_I;   // Copper (I)   cuprous      1+
  public static final int _CU_II  = brine.brineStandardTools._CU_II;  // Copper (II)  cupric       2+
  public static final int _AG     = brine.brineStandardTools._AG;     // Silver                    1+
  public static final int _AU     = brine.brineStandardTools._AU;     // Gold (I)     aurous       1+
  public static final int _AU_III = brine.brineStandardTools._AU_III; // Gold (III)   auric        3+

    // ..  IIB
  public static final int _ZN     = brine.brineStandardTools._ZN;     // Zinc                      2+
  public static final int _CD     = brine.brineStandardTools._CD;     // Cadmium                   2+
  public static final int _HG     = brine.brineStandardTools._HG;     // Mercury (I)  mercurous Hg2 2+
  public static final int _HG_II  = brine.brineStandardTools._HG_II;  // Mercury (II) mercuric     2+

  // ..  IIIA
  public static final int _AL     = brine.brineStandardTools._AL;     // Aluminium                 3+

  //.. VA
  public static final int _SB_III = brine.brineStandardTools._SB_III; // Antimony (III)  antimonous  3+
  public static final int _SB_V   = brine.brineStandardTools._SB_V;   // Antimony (V)    antimonic   5+
  public static final int _BI_III = brine.brineStandardTools._BI_III; // Bismuthous(III) bismuthous  3+
  public static final int _BI_V   = brine.brineStandardTools._BI_V;   // Bismuth(V)      bismuthic   5+

  // ..  IVA
  public static final int _SN_II  = brine.brineStandardTools._SN_II;  // Tin (II)  stannous          2+
  public static final int _SN_IV  = brine.brineStandardTools._SN_IV;  // Tin (IV)  stannic           4+
  public static final int _PB_II  = brine.brineStandardTools._PB_II;  // Lead (II) plumbous          2+
  public static final int _PB_IV  = brine.brineStandardTools._PB_IV;  // Lead (IV) plumbic           4+

  // .. Other
  public static final int _NH4    = brine.brineStandardTools._NH4;    // Ammonium                    1+

  // Common Anions

  public static final int _F      = brine.brineStandardTools._F;      // Flouride        1-
  public static final int _CL     = brine.brineStandardTools._CL;     // Chloride        1-
  public static final int _BR     = brine.brineStandardTools._BR;     // Bromide         1-
  public static final int _I      = brine.brineStandardTools._I;      // Iodide          1-

  public static final int _OH     = brine.brineStandardTools._OH;     // Hydroxide       1-

  public static final int _BO3    = brine.brineStandardTools._BO3;    // Borate
  public static final int _CO3    = brine.brineStandardTools._CO3;    // Carbonate       2-
  public static final int _HCO3   = brine.brineStandardTools._HCO3;   // Bicarbonate     1-

  public static final int _CLO    = brine.brineStandardTools._CLO;    // Hypochlorite    1-
  public static final int _CLO2   = brine.brineStandardTools._CLO2;   // Chlorite        1-
  public static final int _CLO3   = brine.brineStandardTools._CLO3;   // Chlorate        1-
  public static final int _CLO4   = brine.brineStandardTools._CLO4;   // Per chlorate    1-

  public static final int _CN     = brine.brineStandardTools._CN;     // Cyanide         1-
  public static final int _NCO    = brine.brineStandardTools._NCO;    // Cyanate         1-
  public static final int _OCN    = brine.brineStandardTools._OCN;    // Isocyanate      1-
  public static final int _SCN    = brine.brineStandardTools._SCN;    // Thiocyanite     1-

  public static final int _N      = brine.brineStandardTools._N;      // Nitride         3-
  public static final int _N3     = brine.brineStandardTools._N3;     // Azide           1-
  public static final int _NO2    = brine.brineStandardTools._NO2;    // Nitrite         1-
  public static final int _NO3    = brine.brineStandardTools._NO3;    // Nitrate         1-

  public static final int _CRO4   = brine.brineStandardTools._CRO4;   // Chromium (II)   Chromate    2-
  public static final int _CR2O7  = brine.brineStandardTools._CR2O7;  // Chromium (III)  Dichormate  2-

  public static final int _MNO4   = brine.brineStandardTools._MNO4;   // Permanganate    1-

  public static final int _P      = brine.brineStandardTools._P;      // Phosphide       3-
  public static final int _PO3    = brine.brineStandardTools._PO3;    // Phosphite       3-
  public static final int _PO4    = brine.brineStandardTools._PO4;    // Phosphate       3-
  public static final int _HPO4   = brine.brineStandardTools._HPO4;   // Monohydrogen Phosphate 2-
  public static final int _H2PO4  = brine.brineStandardTools._H2PO4;  // Dihydrogen Phosphate   1-

  public static final int _AS     = brine.brineStandardTools._AS;     // Arsinide        3-
  public static final int _SE     = brine.brineStandardTools._SE;     // Selenide        2-

  public static final int _S      = brine.brineStandardTools._S;      // Sulfide            2-
  public static final int _HS     = brine.brineStandardTools._HS;     // Hydrogen Sulfide   1-
  public static final int _SO3    = brine.brineStandardTools._SO3;    // Sulphite           2-
  public static final int _HSO3   = brine.brineStandardTools._HSO3;   // Hydrogen Sulphite  1-
  public static final int _S2O3   = brine.brineStandardTools._S2O3;   // Thiosulphate       2-
  public static final int _SO4    = brine.brineStandardTools._SO4;    // Sulfate            2-
  public static final int _HSO4   = brine.brineStandardTools._HSO4;   // Bisulfate          1-

  public static final int _SOLID  = brine.brineStandardTools._SOLID;  // Total Solids
  public static final int _TDS    = brine.brineStandardTools._TDS;    // Total Solids Computed

  public static final int _NAME   = brine.brineStandardTools._NAME;   // Well Name
  public static final int _API    = brine.brineStandardTools._API;    // API-Number
  public static final int _SRC    = brine.brineStandardTools._SRC;    // Source

  public static final String CURVES[][] = brine.brineStandardTools.CURVES;

  // Input Variables

  private Observable      notifier  = null;  // Observables Object
  private brineListStruct stData    = null; // Brine List Structure

  // Global Variables

  private brineTable      pTable    = null; // Brine Data Table
  private int             iColumns  = 0;
  private int             iMnem[]   = null;
  private String          sColumn[] = null;
  private int             iLength[] = null;

  // Global Frame Widgets

  private JCheckBox cb[] = null;

  /** Constructor brinePanel()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = Global IQSTRAT Data Structure
   */

  public brinePanel( Observable notifier, brineListStruct stData )
  {
    try
    {
      this.notifier = notifier;
      this.stData   = stData;

      getColumns( stData );

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop      = new JPanel();
    JLabel lbl         = new JLabel();
    JScrollPane scroll = new JScrollPane();

    this.setLayout(new BorderLayout());

    // Top Panel

//    pnlTop.setLayout(new BorderLayout());

    lbl.setText("p=compression; s=shear; sf=fast shear; ss=slow shear;    " +
                "[Edit] - Editable Column");
    lbl.setForeground(Color.blue);
    lbl.setFont(new java.awt.Font("Monospaced", 2, 11));

    cb = new JCheckBox[iColumns];
    for (int i=0; i<iColumns; i++)
    {
      cb[i] = new JCheckBox();
      cb[i].setText( sColumn[i] );
      cb[i].setSelected(true);
      cb[i].addActionListener(this);

      if (i!=0) { pnlTop.add( cb[i] ); }
	}

    // Center Panel

    pTable = new brineTable(stData, iColumns, iMnem, sColumn, iLength );
    scroll = pTable.getScrollPane();

    this.add(pnlTop, BorderLayout.NORTH);
//    pnlTop.add(lbl,  BorderLayout.WEST);

    this.add(scroll, BorderLayout.CENTER);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

	notifier  = null; // Observable
    stData    = null; // Brine List Structure

    // Global Variables

    pTable    = null; // Brine Data Table

    iColumns  = 0;
    iMnem     = null;
    sColumn   = null;
    iLength   = null;

    // Global Frame Widgets

    cb        = null;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getWidth()
   *  <p> This method will determine the width of the Frame from the width of
   *      the table.
   */

  public int getWidth()
  {
	int iWidth = 0;

	if (iLength != null)
	{
	  if (iLength.length > 0)
	  {
		for (int i=0; i<iLength.length; i++)
		{
		  iWidth = iWidth + 8 * iLength[i];
		}
	  }
	}

	return (iWidth + 25);
  }

  /** Method getMnemonics()
   *  <p> This method will return the list of the mnemonic IDs of the measured
   *      brine data.
   */

  public int[] getMnemonics()
  {
	int i=0;
	int iBrine[] = null;
	int iCount   = 0;
	int iTotal   = 0;

    for (i=1; i<iColumns; i++)
    {
	  if (cb[i].isSelected())
	  {
		iCount++;
	  }
	}

    iTotal = iCount;

	if (iCount > 0)
	{
	  iBrine = new int[iCount];
	  iCount = 0;

      for (i=1; i<iColumns; i++)
      {
	    if (cb[i].isSelected())
	    {
		  if (iCount < iTotal)
		  {
	  	    iBrine[iCount] = iMnem[i];
	  	    iCount++;
		  }
	    }
	  }
	}

	return ( iBrine );
  }

  /** Method getSelected()
   *  <p> This method will return the selected rows
   *  @return return selected rows.
   */

  public int[] getSelected() { return (pTable.getSelected()); }

  /** Method getData()
   *  <p> This will return the Brine Data List Structure
   *  @return stData = Brine Data List Structure of Selected Data
   */

  public brineListStruct getData() { return (pTable.getData()); }

  /** Method getCount()
   *  <p> This method will count the number anions and cations that can be used
   *  @param  st     = The brine List Structure
   *  @return iCount = The total number of Cations & Anions
   */

  private static int getCount( brineListStruct st )
  {
	int iCount = 1; // Add API-Number to List

	if (st != null)
	{
	  for (int i=_LI; i<_SOLID; i++)
	  {
        if (st.checkData(i))
   	    {
		  iCount++;
		}
	  }

	  if (st.checkData(_PH))
	    iCount++;
	}

	return (iCount);
  }

  /** Method getColumns()
   *  <p> This method will order the number anions and cations that can be used
   *  @param  st     = The brine List Structure
   */

  private void getColumns( brineListStruct st )
  {
	int iCount   = 0;

	if (st != null)
	{
	  iColumns = getCount( st );
	  iMnem    = new int[iColumns];
      sColumn  = new String[iColumns];
      iLength  = new int[iColumns];

      iMnem[iCount]   = _API;
      sColumn[iCount] = "API-Number";
      iLength[iCount] = 16;
      iCount++;

	  for (int i=_LI; i<_SOLID; i++)
	  {
        if (st.checkData(i))
	    {
		  if (iCount<iColumns)
		  {
            iMnem[iCount]   = i;
            sColumn[iCount] = CURVES[i][1];
            iLength[iCount] = 8;
            iCount++;
		  }
		}
	  }

	  if (st.checkData(_PH))
      {
	    if (iCount<iColumns)
	    {
          iMnem[iCount]   = _PH;
          sColumn[iCount] = CURVES[_PH][1];
          iLength[iCount] = 8;
          iCount++;
		}
	  }
	}
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   *  <p> This method will set the new Brine Data List Structure
   */

  public void setData( brineListStruct stData )
  {
	this.stData = stData;
	pTable.setData(stData);
  }

//      if (notifier != null)
//        notifier.notifyObservers(new String("LAS - WELL Information"));

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
  }
}


/*
 *  @version 1.1 10/28/2015
 *  @author  John Victorine
 */

