/*
 * @brineStartFrame.java Version 1.1 05/14/2014
 *
 * Copyright (c) 2014 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.gui;

import java.util.Observer;
import java.util.Observable;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import cmn.cmnStruct;			         // Global Common Data Structure

import iqstrat.iqstratHeadersStruct;     // Headers Data Structure
import iqstrat.iqstratHeadersListStruct; // Headers Data List Structure
import iqstrat.io.ReadHeadersXMLFile;    // Read Well Header Information as XML Stream

import kgs.gui.kgsBrineFrame;            // Brine Data KGS Frame
import kgs.gui.kgsBrineTable;            // KGS Brine Data Table

import grid.gui.gridSearchPanel;         // Gridding & Mapping Search Panel

import brine.brineListStruct;            // Brine Data List Structure
import brine.brineStruct;                // Brine Data Structure
import brine.gui.brineMapCSVFrame;       // Import Brine Data CSV File
import brine.gui.brineFrame;             // Brine Data Frame to show raw data.
import brine.io.ReadBrineDataXMLFile;    // Read Brine Data XML File

/** Class brineStartFrame
 *  <p> This Class Starting Panel for this web application
 *
 *  @version 1.1 05/14/2014
 *  @author  John R. Victorine
 */

public class brineStartFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private cmnStruct     stCMN    = new cmnStruct();

  // Global Variables

  private Observable     notifier  = null;  // Observables Object

  public static final int _BY_XML  = 1;
  public static final int _BY_CSV  = 2;

  public int iByASCII = _BY_XML;

  // -- Load Profile Data Frames

  public static final int _PC           = 0;
  public static final int _KGS          = 1;
  public static final int _BRINE_DATA   = 14;

  private kgsBrineTable        pTable        = null; // Display Measured Sections
  private brineMapCSVFrame     pMapBrineData = null; // Import Brine Data CSV File
  private kgsBrineFrame        pKGSBrine     = null; // Search KGS Brine Database
  private gridSearchPanel      pGridding     = null; // Gridding & Mapping Search Panel
  private brineFrame           frame         = null;

  // -- Type of data

  public static final int _HEADERS = 0;  // Headers Data

  // -- Brine Data

  public static final int _BRINE   = 17; // Brine Data

  // .. Header Information Data Structure

  private iqstratHeadersStruct stHeader  = null; // Header Data Structure

  // .. Brine Data

  private brineListStruct      stBrine  = null; // Brine Data

  // Global Frame Widgets

  private JFrame  frameCO2     = null;

  // -- Top Panel Widgets

  private JButton btnKGS       = null; // KGS Icon
  private JButton btnCO2       = null; // CO2 Icon
  private JButton btnPC        = null; // PC Icon

  private JRadioButton rbByXML = new JRadioButton(); // By XML ASCII File
  private JRadioButton rbByCSV = new JRadioButton(); // By CSV ASCII File

  // -- Base Panel Widgets

  private JButton btnContinue = new JButton();
  private JButton btnClear    = new JButton();
  private JButton btnExit     = new JButton();

  /** Constructor brineStartFrame( iqstratStruct stStruct )
   *  <p> This is the Constructor for this class.
   *  @param stCMN = Common Data Structure
   */

  public brineStartFrame( cmnStruct stCMN )
  {
    try
    {
      this.stCMN = stCMN;

      jbInit();

      setButtons();
      addWindowListener( new brineStartFrame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop         = new JPanel();
    JPanel pnlKGS         = new JPanel();
    JPanel pnlKGSicon     = new JPanel();
    JPanel pnlKGSCenter   = new JPanel();
    JPanel pnlPC          = new JPanel();
    JPanel pnlPCicon      = new JPanel();
    JPanel pnlPCCenter    = new JPanel();

    JPanel pnlCenter      = new JPanel();

    JPanel pnlBase        = new JPanel();
    JPanel pnlWait        = new JPanel();
    JPanel pnlBase1       = new JPanel();
    JPanel pnlBase2       = new JPanel();

    JScrollPane scroll    = new JScrollPane();

    ButtonGroup groupASCII = new ButtonGroup();

    TitledBorder titledBorderTop = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Data Source");
    titledBorderTop.setTitleFont(new java.awt.Font("Dialog", 1, 14));

    TitledBorder titledBorderKGS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "KGS Database");
    titledBorderKGS.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    TitledBorder titledBorderKGSIcon = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Load Brine Data");
    titledBorderKGSIcon.setTitleJustification(TitledBorder.CENTER);
    titledBorderKGSIcon.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderPC = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "PC: Read Brine Data By");
    titledBorderPC.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    TitledBorder titledBorderPCIcon = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Load PC Data");
    titledBorderPCIcon.setTitleJustification(TitledBorder.CENTER);
    titledBorderPCIcon.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Create an Observable

    notifier = new brineStartFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.setTitle( "Load Data" );
    this.getContentPane().setLayout( new BorderLayout() );

    // Top Panel

    pnlTop.setBorder( titledBorderTop );
    pnlTop.setLayout( new GridLayout(2,1) );
//    pnlTop.setPreferredSize( new Dimension(10, 280) );

    // -- KGS Data Source

    pnlKGS.setLayout( new BorderLayout() );
    pnlKGS.setBorder( titledBorderKGS );

    // .. KGS ICON Panel

    pnlKGSicon.setBorder( titledBorderKGSIcon );
    pnlKGSicon.setPreferredSize( new Dimension(100, 10) );

    btnKGS = new JButton( createImageIcon( cmn.cmnStruct.KGS_DATA ) );
    btnKGS.setBackground(Color.white);
    btnKGS.setPreferredSize( new Dimension(64, 64) );
    btnKGS.addActionListener( this );

    btnCO2 = new JButton( createImageIcon( cmn.cmnStruct.CO2_ICON ) );
    btnCO2.setBackground(Color.white);
    btnCO2.setPreferredSize( new Dimension(64, 64) );
    btnCO2.addActionListener( this );

    // .. KGS Center Panel

    pnlKGSCenter.setLayout( new GridLayout() );

    // -- PC Data Source

    pnlPC.setBorder( titledBorderPC );
    pnlPC.setLayout( new BorderLayout() );

    // .. PC ICON Panel

    pnlPCicon.setBorder( titledBorderPCIcon );
    pnlPCicon.setPreferredSize( new Dimension(100, 100) );

    btnPC = new JButton( createImageIcon( cmn.cmnStruct.BRINE_DATA ) );
    btnPC.setBackground(Color.white);
    btnPC.setPreferredSize( new Dimension(64, 64) );
    btnPC.addActionListener( this );

    // .. PC Center Panel

    pnlPCCenter.setLayout( new GridLayout(2,1) );

    // .... PC Buttons Panel

    rbByXML.setFont(new java.awt.Font("Dialog", 1, 13));
    rbByXML.setText( "XML (Extensible Markup Language) File" );
    rbByXML.setSelected(true);
    rbByXML.addActionListener(this);

    rbByCSV.setFont(new java.awt.Font("Dialog", 1, 13));
    rbByCSV.setText( "CSV (Comma Delimited) File" );
    rbByCSV.addActionListener(this);

    // Center Panel

    pnlCenter.setLayout( new BorderLayout() );

    // .. Brine Data Table

    pTable = new kgsBrineTable( notifier, stBrine, gui.guiTable._SINGLE_SELECTION );
    scroll = pTable.getScrollPane();

    // Base Panel

    pnlBase.setLayout( new BorderLayout() );

    btnContinue.setFont(new java.awt.Font("Dialog", 1, 11));
    btnContinue.setPreferredSize( new Dimension(120, 25) );
    btnContinue.setText("Continue");
    btnContinue.addActionListener(this);

    btnClear.setFont(new java.awt.Font("Dialog", 1, 11));
    btnClear.setPreferredSize( new Dimension(120, 25) );
    btnClear.setText("Clear");
    btnClear.addActionListener(this);

    btnExit.setFont(new java.awt.Font("Dialog", 1, 11));
    btnExit.setPreferredSize( new Dimension(120, 25) );
    btnExit.setText("Exit");
    btnExit.addActionListener(this);

    // Add Widgets to Panel

    this.getContentPane().add( pnlTop,    BorderLayout.NORTH );
    pnlTop.add( pnlKGS,                   null  );
    pnlKGS.add( pnlKGSicon,               BorderLayout.CENTER );
    pnlKGSicon.add( btnKGS,               null );
    pnlKGSicon.add( btnCO2,               null );

    pnlTop.add( pnlPC,                    null );
    pnlPC.add( pnlPCicon,                 BorderLayout.EAST );
    pnlPCicon.add( btnPC,                 null );

    pnlPC.add( pnlPCCenter,               BorderLayout.CENTER );
    pnlPCCenter.add( rbByXML,             null );
    pnlPCCenter.add( rbByCSV,             null );

    groupASCII.add( rbByXML );
    groupASCII.add( rbByCSV );

    this.getContentPane().add( pnlCenter, BorderLayout.CENTER );
    pnlCenter.add(scroll,                 BorderLayout.CENTER);

    this.getContentPane().add( pnlBase,   BorderLayout.SOUTH );
    pnlBase.add( pnlBase2,                BorderLayout.SOUTH );
    pnlBase2.add( btnContinue,            null );
    pnlBase2.add( btnClear,               null );
    pnlBase2.add( btnExit,                null );

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int iScreenHigh  = d.height-50;

    this.setSize(new Dimension(550, iScreenHigh)); //675));
    this.setLocation(10, 5);
//    this.setLocation((d.width - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * ------------------------- CLOSE METHODS ----------------------------- *
   * ===================================================================== */

  /** METHOD brineFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class brineStartFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN    = null; // Common Data Structure

    // Global Variables

    notifier = null; // Observables Object

    if (pMapBrineData != null)
      pMapBrineData.close();
    pMapBrineData = null;

    if (pKGSBrine != null)
      pKGSBrine.close();
    pKGSBrine = null;

    if (pGridding != null)
      pGridding.close();
    pGridding = null;

	if (frameCO2 != null)
	  frameCO2.dispose();
	frameCO2     = null;

    if (frame != null)
      frame.close();
    frame = null;

    // Global Panel Widgets

    // -- Top Panel Widgets

    btnKGS      = null; // KGS Icon - No Action
    btnCO2      = null;
    btnPC       = null; // PC Icon  - No Action

    rbByXML     = null; // By XML ASCII File
    rbByCSV     = null; // By CSV ASCII File

    // -- Base Panel Widgets

    btnContinue = null; // Continue
    btnClear    = null; // Clear
    btnExit     = null; // Exit

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getData()
   * <p> This method will retrieve data
   * @param iLocation = Location of Data being retrieved
   * @param iData     = Data Type being retrieved
   */

  private void getData( int iLocation, int iData)
  {
	int i=0;
    double diff   = 0.0;
    double depth1 = 0.0;
    double depth2 = 0.0;

    switch (iData)
    {
      case _HEADERS:    // Headers Data
        stHeader = iqstrat.iqstratHeadersUtility.copy(
            getHeader(iLocation, pKGSBrine ) );
        break;

      case _BRINE:
        stBrine = brine.brineUtility.copyList(
			        getBrine( iLocation, pMapBrineData, pKGSBrine ));

        if (stHeader != null)
          stBrine = brine.brineUtility.addHeaderData( stHeader, stBrine );

	    if (pTable != null) { pTable.repopulateList(stBrine); }
        break;
    }
  }

  /** Method getHeader()
   * <p> This method will retrieve the Header Data Structure
   * @param  iLocation  = Location identifier
   * @param  pKGS       = Search KGS Database for Data
   * @return st         = the Header Data Structure
   */

  public static iqstratHeadersStruct getHeader(
      int                  iLocation,  // Location identifier
      kgsBrineFrame        pBrine )    // KGS Brine Data Frame
  {
    iqstratHeadersStruct st = null;

    switch (iLocation)
    {
      case _BRINE_DATA:
        st = pBrine.getHeader();
        break;
    }

    return (st);
  }

  /** Method getBrine()
   * <p> This method will retrieve the Brine Data List Structure
   * @param  iLocation  = Location identifier
   * @param  pKGS       = Search KGS Database for Data
   * @return st         = Brine Data List Structure
   */

  public static brineListStruct getBrine(
      int                   iLocation,  // Location identifier
      brineMapCSVFrame      pc,        // Brine Map CSV Frame
      kgsBrineFrame         pKGS )     // Search KGS Database for Data
  {
    brineListStruct st     = null;
    brineListStruct stTemp = null;
 	String          sUKey  = new String(cmn.cmnString.UniqueName());
 	double          dTotal = 0.0;
 	double          data   = 0.0;
 	int             iTotal = 0;

    switch (iLocation)
    {
      case _PC:
        st = pc.getData();
        pc.close();
        pc = null;
        break;
      case _KGS:
        stTemp = pKGS.getData();
        if (stTemp != null)
        {
		  for (int i=0; i<stTemp.iCount; i++)
		  {
		    if (stTemp.stItem[i] != null)
		    {
			  if (stTemp.stItem[i].dNA > 0.0)
			  {
			    st = brine.brineUtility.add( stTemp.stItem[i], st );
			  }
			}
		  }
		}

		if (st != null)
		{
		  for (int k=0; k<st.iCount; k++)
		  {
	        for (int i=0; i<brine.brineStandardTools.TOTAL_A; i++)
	        {
			  st.setID(i);
			}
		  }
		}
        break;
    }

    if (st != null)
    {
      for (int i=0; i<st.iCount; i++)
      {
        dTotal = 0.0;
		if (st.stItem[i] != null)
		{
		  if (st.stItem[i].sKID.length() < 2)
		    st.stItem[i].sKID = new String(sUKey + "_" + i);

		  if (st.stItem[i].sKEY.length() < 2)
		    st.stItem[i].sKEY = new String(sUKey + "_" + i);

          for (int j=brine.brineStandardTools._LI; j<brine.brineStandardTools._SOLID; j++)
          {
			if (st.checkData(j))
			{
			  data = st.getData(j, i);
   	          if (data > 0.0)
			    dTotal = dTotal + data;
		    }
		  }

          iTotal = (int) (dTotal * 10.0);
          dTotal = (double) iTotal / 10.0;

  	      st.setData(brine.brineStandardTools._SOLID, i, dTotal);
		  st.setData(brine.brineStandardTools._TDS,   i, dTotal);
		}
	  }
	}

    return (st);
  }

  /** Method getGridHeader()
   *  <p> This method will retrieve the Header information for the Brine data
   *      using the API-Number from the Gridding & Mapping Brine Data Structure
   *  @param  st = Brine Data List Structure
   *  @return st = Brine Data List Structure
   */

  private brineListStruct getGridHeader( brineListStruct st )
  {
	String sAPI = "";
	String sDir = "";
	iqstratHeadersListStruct stList = null;
	iqstratHeadersStruct     stWell = null;
	ReadHeadersXMLFile       pWell  = null;

    if (st != null)
    {
	  for (int i=0; i<st.iCount; i++)
	  {
	    if (st.stItem[i].sAPI.length() > 0)
	    {
		  sDir   = cmn.cmnStruct.KGS_WELL + st.stItem[i].sAPI;
          pWell  = new ReadHeadersXMLFile( iqstrat.io.ReadHeadersXMLFile.URL );
          stList = pWell.Process( sDir );

          if (stList != null)
          {
			if (stList.stItem[0] != null)
			{
			  st.stItem[i] = brine.brineUtility.addHeaderData(
				               i, stList.stItem[0], st.stItem[i] );
			}
		  }
		}
	  }
	}

	return (st);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will set the Profile Plot buttons depending on the
   *      selected data.
   */

  private void setButtons()
  {
    btnContinue.setEnabled(false);

	if (stBrine != null)
	{
	  if (stBrine.iCount > 0)
	  {
		btnContinue.setEnabled(true);
	  }
	}
  }

  /** Method clear()
   * <p> This method will clear the data structures
   */

  private void clear()
  {
	if (stBrine != null)
	  stBrine.delete();
	stBrine = null;

    if (pTable != null) { pTable.repopulateList(stBrine); }
  }

  /* ===================================================================== *
   * ------------------------- SEARCH DIALOGS ---------------------------- *
   * ===================================================================== */

  /** Method searchPC()
   * <p> This method will open the Search KGS Dialog to allow the user to
   *     search for production data.
   */

  private void searchPC()
  {
	switch (iByASCII)
	{
      case _BY_XML:
        readXMLFile();
        break;
      case _BY_CSV:
  	    readCSVFile();
        break;
    }
  }

  /** Method readCSVFile()
   * <p> This method will search for a Brine Data File that is on the user's PC
   */

  private void readCSVFile()
  {
    FileDialog dialog      = null;
    Frame      frameTemp   = new Frame();
    String     sReadMethod = null;
    String     sDirectory  = null;
    String     sFilename   = null;

    dialog = new FileDialog(frameTemp,
             "Select Brine Data Comma Delimited File from your PC.");
    dialog.setVisible(true);
    frameTemp.setVisible(true);

    // Retrieve LAS File Location

    if ((dialog.getDirectory() != null) && (dialog.getFile() != null))
    {
      sReadMethod = new String("FILE");
      sDirectory  = new String(dialog.getDirectory());
      sFilename   = new String(dialog.getFile());

      if (pMapBrineData != null)
        pMapBrineData.close();
      pMapBrineData = null;

      pMapBrineData = new brineMapCSVFrame( notifier,
                                            sReadMethod,
                                            sDirectory,
                                            sFilename );
    }

    // Dispose of Temporary Dialog & Frame

    frameTemp.dispose();
    frameTemp = null;

    dialog.dispose();
    dialog = null;
  }

  /** Method readXMLFile()
   * <p> This method will search for a Brine Data File that is on the user's PC
   */

  private void readXMLFile()
  {
    FileDialog dialog      = null;
    Frame      frameTemp   = new Frame();
    String     sReadMethod = null;
    String     sDirectory  = null;
    String     sFilename   = null;
    String     sPath       = "";  // Build Path to Data XML
  	String     sUKey       = new String(cmn.cmnString.UniqueName());

    dialog = new FileDialog(frameTemp,
             "Select Brine Data Comma Delimited File from your PC.");
    dialog.setVisible(true);
    frameTemp.setVisible(true);

    // Retrieve LAS File Location

    if ((dialog.getDirectory() != null) && (dialog.getFile() != null))
    {
      sReadMethod = new String("FILE");
      sDirectory  = new String(dialog.getDirectory());
      sFilename   = new String(dialog.getFile());

      sPath       = sDirectory + "\\" + sFilename;

      if (exists( sDirectory, sFilename ))
      {
  	    ReadBrineDataXMLFile pRead =
	      new ReadBrineDataXMLFile(brine.io.ReadBrineDataXMLFile.FILE);
        stBrine = pRead.Process( sPath );

        if (stBrine != null)
        {
          for (int i=0; i<stBrine.iCount; i++)
          {
   		    if (stBrine.stItem[i] != null)
		    {
		      if (stBrine.stItem[i].sKID.length() < 2)
		        stBrine.stItem[i].sKID = new String(sUKey + "_" + i);

		      if (stBrine.stItem[i].sKEY.length() < 2)
		        stBrine.stItem[i].sKEY = new String(sUKey + "_" + i);
		    }
	      }
		}

	    if (pTable != null) { pTable.repopulateList(stBrine); }
	  }
    }

    // Dispose of Temporary Dialog & Frame

    frameTemp.dispose();
    frameTemp = null;

    dialog.dispose();
    dialog = null;
  }

  /** Method exists( String sDir, String sFile )
   * <p> This method will check to see if a specific file exists in directory
   * @param  sDir   = The directory path to the cross section data
   * @param  sFile  = The filename exists in the directory path
   * @return bValue = true if file exists and false if it does not
   */

  public boolean exists( String sDir, String sFile )
  {
    boolean bValue = false;

    try
    {
      File p = new File( sDir );

      if (p.exists())
      {
        try
        {
          File pNew = new File( p, sFile );
          if (pNew.exists())
          {
            bValue = true;
          }
        }
        catch (Exception e)
        {
          e.printStackTrace();
          String sError = new String(
              "xsectionIOControl.exists( directory, file )\n " + e.toString());
          JOptionPane.showMessageDialog(
              (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE );
        }
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      String sError = new String(
          "xsectionIOControl.exists( directory, file )\n " + e.toString());
      JOptionPane.showMessageDialog(
          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE );
    }

    return ( bValue );
  }

  /** Method searchKGSBrine ()
   * <p> This method will open the Search PC Dialog to allow the user to
   *     retrieve Digital LAS File & Formation Tops Comma Delimited File
   *     from their PC.
   */

  private void searchKGSBrine()
  {
    if (pKGSBrine != null)
      pKGSBrine.close();
    pKGSBrine = null;

    pKGSBrine = new kgsBrineFrame( notifier );
  }

  /** Method searchCO2 ()
   * <p> This method will open the Search PC Dialog to allow the user to
   *     retrieve Digital LAS File & Formation Tops Comma Delimited File
   *     from their PC.
   */

  private void searchCO2()
  {
    if (pGridding != null)
      pGridding.close();
    pGridding = null;

	if (frameCO2 != null)
	  frameCO2.dispose();
	frameCO2     = null;

    frameCO2 = new JFrame();
    frameCO2.setTitle("SELECT");

    pGridding = new gridSearchPanel( notifier );
    frameCO2.getContentPane().add(pGridding, null);

    // Set the Size and Display the Frame

    frameCO2.setSize(450, 350);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    frameCO2.setLocation((d.width - frameCO2.getSize().width)/2,
                         (d.height - frameCO2.getSize().height)/2);

    frameCO2.setResizable(false);
    frameCO2.setVisible(true);

    frameCO2.addWindowListener( new frameCO2_WindowListener() );
  }

  /** METHOD frameCO2_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class frameCO2_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      if (pGridding != null)
        pGridding.close();
      pGridding = null;

	  if (frameCO2 != null)
        frameCO2.dispose();
      frameCO2 = null;
    }
  }

  /* ===================================================================== *
   * --------------------------- LOAD DATA ------------------------------- *
   * ===================================================================== */

  /** Method loadData()
   * <p> This method will load the data
   */

  private void loadData()
  {
    if (frame != null)
      frame.close();
    frame = null;

    frame = new brineFrame( stCMN, stBrine );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == rbByXML) { iByASCII = _BY_XML; } // By XML ASCII File
    if (event.getSource() == rbByCSV) { iByASCII = _BY_CSV; } // By CSV ASCII File

    if (event.getSource() == btnPC)  { searchPC(); } // Load PC Data
    if (event.getSource() == btnKGS) { searchKGSBrine(); }
    if (event.getSource() == btnCO2) { searchCO2(); }

    if (event.getSource() == btnContinue) { loadData(); } // Load Data
    if (event.getSource() == btnClear)    { clear(); }    // Clear Data

    setButtons();

    if (event.getSource() == btnExit)     { close(); }    // Exit
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg  = new String( (String) arg);
    String sPath = "";

    /* ------------------------------------------------------------------ *
     * ----------------------- Brine Data Actions ----------------------- *
     * ------------------------------------------------------------------ */

    if (sArg.equals("KGS Brine Loaded"))
    {
      getData( _KGS,        _BRINE );
      getData( _BRINE_DATA, _HEADERS );

      pKGSBrine.close();
	}

    if (sArg.equals("Get Brine Data"))
    {
	  sPath   = new String( pGridding.getSQL() );
      stBrine = grid.io.gridIORead.readBrineData( sPath );
      stBrine = getGridHeader( stBrine );
//System.out.println(sPath);

      if (pTable != null) { pTable.repopulateList(stBrine); }

      if (pGridding != null)
        pGridding.close();
      pGridding = null;

  	  if (frameCO2 != null)
 	    frameCO2.dispose();
	  frameCO2     = null;
	}

    if (sArg.equals("Brine CSV File Loaded"))
    {
      getData( _PC,  _BRINE );
	}

    setButtons();
  }
}

/** CLASS brineStartFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class brineStartFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 05/14/2014
 *  @author  John Victorine
 */
