/*
 * @brineTable.java Version 1.1 10/28/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.Observable;

import gui.guiTable;

import brine.brineListStruct;
import brine.brineStruct;

/** Class brineTable
 *  <p> This Class will display the Formation Tops.
 *
 *  @version 1.1 10/28/2015
 *  @author  John R. Victorine
 */

public class brineTable
{
  public static final int _SINGLE_SELECTION = gui.guiTable._SINGLE_SELECTION;
  public static final int _MULTI_SELECTION  = gui.guiTable._MULTI_SELECTION;

  private int             iRows     = 0;
  private int             iColumns  = 0;
  private int             iMnem[]   = null;
  private String          sColumn[] = null;
  private int             iLength[] = null;

  private Object          oData[][] = null;
  private guiTable        pTable    = null;

  private Observable      notifier  = null; // Observable Object
  private brineListStruct st        = null; // Brine Data List Structure
  private int             iChange   = 0;

  /** Constructor brineTable()
   *  <p> This is the Constructor for this class.
   * @param st     = The Headers List Data Structure
   */

  public brineTable( brineListStruct st, int iColumns,
                     int iMnem[], String sColumn[], int iLength[] )
  {
    this.st       = st;
    this.iColumns = iColumns;
    this.iMnem    = iMnem;
    this.sColumn  = sColumn;
    this.iLength  = iLength;

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData, _MULTI_SELECTION);
    else
      pTable = new guiTable(0, iColumns, sColumn, _MULTI_SELECTION);

    resetColumnWidths();
    pTable.showHorScroll(true);

    if (iChange == 0)
      iRows = 0;

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /** Constructor brineTable()
   *  <p> This is the Constructor for this class.
   * @param st     = The Headers List Data Structure
   */

  public brineTable( Observable notifier, brineListStruct st,
                     int iColumns, int iMnem[], String sColumn[], int iLength[] )
  {
	this.notifier = notifier;
    this.st       = st;
    this.iColumns = iColumns;
    this.iMnem    = iMnem;
    this.sColumn  = sColumn;
    this.iLength  = iLength;

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData,
                            _MULTI_SELECTION, notifier);
    else
      pTable = new guiTable(0, iColumns, sColumn,
                            _MULTI_SELECTION, notifier);

    resetColumnWidths();
    pTable.showHorScroll(true);

    if (iChange == 0)
      iRows = 0;

//    if (iRows > 0)
//      pTable.setSelectedRow(0);
  }

  /** Constructor brineTable()
   *  <p> This is the Constructor for this class.
   * @param st     = The Headers List Data Structure
   */

  public brineTable( Observable notifier, brineListStruct st, int iSelection,
                     int iColumns, int iMnem[], String sColumn[], int iLength[] )
  {
	this.notifier = notifier;
    this.st       = st;
    this.iColumns = iColumns;
    this.iMnem    = iMnem;
    this.sColumn  = sColumn;
    this.iLength  = iLength;

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData, iSelection, notifier);
    else
      pTable = new guiTable(0, iColumns, sColumn, iSelection, notifier);

    resetColumnWidths();
    pTable.showHorScroll(true);

    if (iChange == 0)
      iRows = 0;

//    if (iRows > 0)
//      pTable.setSelectedRow(0);
  }

  /** Method resetColumnWidths()
   *  <p> This method will reset the column widths to a specific size.
   */

  private void resetColumnWidths()
  {
    for (int i=0; i<iColumns; i++)
    {
      pTable.resizeColumn(i, iLength[i]);
    }
  }


  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
	iMnem    = null;
    sColumn  = null;
    iLength  = null;
    oData    = null;
    pTable   = null;

    notifier = null;
    st       = null;
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method populateList()
   *  <p> This method will populate the Region List data.
   */

  private void populateList()
  {
    int i,j;
    int iCount = 0;

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if (st != null)
    {
      if (st.iCount > 0)
      {
        iChange = 1;
        iCount  = 0;
        oData   = new Object[st.iCount][iColumns];

        for (i=0; i<st.iCount; i++)
        {
          if (st.stItem[i] != null)
          {
            oData[iCount][0]  = new String(st.stItem[i].sAPI);

            for (j=1; j<iColumns; j++)
            {
			  oData[iCount][j] = new String(""+st.getData(iMnem[j], i));
			}

            iCount++;
          }
        }

        iRows = iCount;
      }
    }
  }

  /** Method setData()
   *  <p> This method will reload the Region List after adding, deleting from
   *      the formation name list.
   * @param st     = The Regions List Data Structure
   */

  public void setData(brineListStruct st)
  {
    this.st = st;

    populateList();

    pTable.reload(iRows, oData);

    if (iChange == 0)
      iRows = 0;

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTotalRows()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getRowData()
   *  <p> This method will return the Region Data Structure of the
   *      selected row.
   *  @return stData = Region Data Structure.
   */

  public brineStruct getRowData()
  {
    brineStruct stData = null;

    stData = st.stItem[pTable.getSelectedRow()];

    return (stData);
  }

  /** Method getSelected()
   *  <p> This method will return the selected rows
   *  @return return selected rows.
   */

  public int[] getSelected()
  {
	return (pTable.getSelectedRows());
  }

  /** Method getData()
   *  <p> This will return the Brine Data List Structure
   *  @return stData = Brine Data List Structure of Selected Data
   */

  public brineListStruct getData()
  {
    brineListStruct stList      = null;
    int             i           = 0;
    int             j           = 0;
    int             iCount      = 1;
    int             icnt        = 0;
    int             iSelected[] = null;

    iSelected = pTable.getSelectedRows();

    if (iSelected != null)
    {
      for (i=0; i<iSelected.length; i++)
      {
        if (iSelected[i] > 0)
          iCount++;
      }
    }

    if (iCount > 0)
    {
      stList        = new brineListStruct();
      stList.stItem = new brineStruct[iCount];

      stList = brine.brineUtility.copyBrineHeader( st );

      iCount = 0;
      icnt   = 0;
      for (i=0; i<st.iCount; i++)
      {
        for (j=0; j<iSelected.length; j++)
        {
          if (icnt == iSelected[j])
          {
            stList.stItem[iCount] =
              brine.brineUtility.copy(st.stItem[i]);
            iCount++;
          }
        }
        icnt++;
      }
      stList.iCount = iCount;
    }

    return (stList);
  }

  /** Method getAllData()
   *  <p> This method will returns the Region List Data Structure.
   *  @return st = Region List Data Structure.
   */

  public brineListStruct getAllData()
  {
    brineListStruct stList      = null;
    int             i           = 0;
    int             j           = 0;
    int             iCount      = 1;

    if (st != null)
    {
      stList        = new brineListStruct();
      stList.stItem = new brineStruct[st.iCount];

      stList = brine.brineUtility.copyBrineHeader( st );

      iCount = 0;
      for (i=0; i<st.iCount; i++)
      {
        stList.stItem[iCount] =
            brine.brineUtility.copy(st.stItem[i]);
        iCount++;
      }
      stList.iCount = iCount;
    }

	return (stList);
  }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 10/28/2015
 *  @author  John Victorine
 */
