/*
 * @WriteBrineCSVFile.java Version 1.1 05/20/2014
 *
 * Copyright (c) 2014 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.io;

import java.io.*;
import java.awt.*;

import util.utilFileIO;
import brine.brineListStruct;
import cmn.cmnString;

/** Class WriteBrineCSVFile
 *  <p> This class will parse the Brine List Structure and
 *      write the information to the Brine CSV File.
 *
 *  @version 1.1 05/20/2014
 *  @author  John R. Victorine
 */

public class WriteBrineCSVFile
{
  /** Method write()
   *  <p> This is the method will write the CSV String to the user's PC
   *  @param sPath     = The Directory Path for the CSV File
   *  @param sFilename = The Filename of the CSV File.
   *  @param sCSV      = The Brine Data CSV File Contents String
   */

  public static void write(String sPath, String sFilename, String sCSV)
  {
    utilFileIO pIO  = new utilFileIO();

    // Open the File for write

    pIO.Open( util.utilFileIO.DIRECTORY,
              util.utilFileIO.WRITE,
              sPath,
              sFilename );

    // Write the CSV Block to the File

    pIO.Write( sCSV );

    // Close the File

    pIO.Close();
  }

  /** Method buildCSV()
   *  <p> This method will build the CSV File
   *  @param  st   = brine list data structure
   *  @return sCSV = Brine ASCII CSV String
   */

  public static String buildCSV(brineListStruct st)
  {
    int i,j;
    String  sCSV     = "";
    int     iAdded   = 0;
    double  data     = 0.0;

    if (st != null)
    {
      iAdded = 1;
      sCSV   = new String(sCSV + brine.io.ReadBrineDataXMLFile.NAMEA);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.API);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.FIELD);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.COUNTY);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.STATE);

      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.LOC);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.LAT);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.LONG);

      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.KB);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.GL);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.DF);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.TGT);

      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.SRC);
      sCSV   = new String(sCSV + ","+brine.io.ReadBrineDataXMLFile.DATE);

	  for(j=1; j<brine.brineStandardTools.TOTAL; j++)
	  {
		if (st.checkData(j))
		{
		  if (iAdded == 0)
		  {
		    sCSV   = new String(sCSV + brine.brineStandardTools.CURVES[j][1]);
		    iAdded = 1;
		  }
		  else
		    sCSV = new String(sCSV + ","+brine.brineStandardTools.CURVES[j][1]);
		}
	  }
	  sCSV = new String(sCSV + "\n");

      iAdded = 1;
      sCSV   = new String(sCSV + " ");
      sCSV   = new String(sCSV + ", ");
      sCSV   = new String(sCSV + ", ");
      sCSV   = new String(sCSV + ", ");
      sCSV   = new String(sCSV + ", ");

      sCSV   = new String(sCSV + ", ");
      sCSV   = new String(sCSV + ",deg");
      sCSV   = new String(sCSV + ",deg");

      sCSV   = new String(sCSV + ",ft");
      sCSV   = new String(sCSV + ",ft");
      sCSV   = new String(sCSV + ",ft");
      sCSV   = new String(sCSV + ",ft");

      sCSV   = new String(sCSV + ", ");
      sCSV   = new String(sCSV + ", ");

	  for(j=1; j<brine.brineStandardTools.TOTAL; j++)
	  {
	    if (st.checkData(j))
	    {
	  	  if (iAdded == 0)
		  {
		    sCSV   = new String(sCSV + brine.brineStandardTools.CURVES[j][3]);
		    iAdded = 1;
		  }
		  else
		    sCSV = new String(sCSV + ","+brine.brineStandardTools.CURVES[j][3]);
		}
	  }
	  sCSV = new String(sCSV + "\n");

      for (i=0; i<st.iCount; i++)
      {
        iAdded = 1;
        sCSV   = new String(sCSV + cmn.cmnString.replaceChar(st.stItem[i].sName, ',', ' '));
        sCSV   = new String(sCSV + ", "+st.stItem[i].sAPI);
        sCSV   = new String(sCSV + ", "+cmn.cmnString.replaceChar(st.stItem[i].sField, ',', ' '));
        sCSV   = new String(sCSV + ", "+st.stItem[i].sCounty);
        sCSV   = new String(sCSV + ", "+st.stItem[i].state);

        if (st.stItem[i].sLocation.length() > 8)
          sCSV = new String(sCSV + ", "+cmn.cmnString.replaceChar(st.stItem[i].sLocation, ',', ' '));
        else
		  sCSV = new String(sCSV + ", ");

        if (st.stItem[i].dLatitude == 0.0)
          sCSV = new String(sCSV + ", ");
        else
          sCSV = new String(sCSV + ", "+st.stItem[i].dLatitude);

        if (st.stItem[i].dLongitude == 0.0)
		  sCSV = new String(sCSV + ", ");
		else
		  sCSV = new String(sCSV + ", "+st.stItem[i].dLongitude);

        if (st.stItem[i].dKB == 0.0)
		  sCSV = new String(sCSV + ", ");
		else
		  sCSV = new String(sCSV + ", "+st.stItem[i].dKB);

        if (st.stItem[i].dGL == 0.0)
		  sCSV = new String(sCSV + ", ");
		else
		  sCSV = new String(sCSV + ", "+st.stItem[i].dGL);

        if (st.stItem[i].dDF == 0.0)
		  sCSV = new String(sCSV + ", ");
		else
		  sCSV = new String(sCSV + ", "+st.stItem[i].dDF);

        if (st.stItem[i].dTGT == 0.0)
		  sCSV = new String(sCSV + ", ");
		else
		  sCSV = new String(sCSV + ", "+st.stItem[i].dTGT);

        sCSV   = new String(sCSV + ", "+cmn.cmnString.replaceChar(st.stItem[i].source, ',', ' '));
        sCSV   = new String(sCSV + ", "+cmn.cmnString.replaceChar(st.stItem[i].sRecovery, ',', ' '));

        if (st.checkData(brine.brineStandardTools._FORM))
          sCSV = new String(sCSV + ", "+st.stItem[i].sFORM);

        if (st.checkData(brine.brineStandardTools._AGE))
     	  sCSV = new String(sCSV + ", "+st.stItem[i].sAGE);

		for(j=brine.brineStandardTools._TOP; j<brine.brineStandardTools.TOTAL; j++)
		{
		  if (st.checkData(j))
		  {
			data = st.getData(j, i);
			if (j != brine.brineStandardTools._LONG)
			{
			  if (data < 0.0)
			    data = 0.0;
			}

			if (iAdded == 0)
			{
			  sCSV   = new String(sCSV + data);
			  iAdded = 1;
			}
			else
			  sCSV = new String(sCSV + "," + data);
		  }
		}
		sCSV = new String(sCSV + "\n");
      }
    }

    return (sCSV);
  }
}

/*
 *  @version 1.1 05/20/2014
 *  @author  John Victorine
 */
