/*
 * @gridSearchPanel.java Version 1.1 06/25/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.gui;

import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import grid.gui.gridStartTable;
import grid.gridTypeStruct;
import grid.io.ReadGridDataXMLFile;

/** Class gridSearchPanel
 *  <p> This Class will display the KHAN Control Frame.
 *
 *  @version 1.1 06/25/2015
 *  @author  John R. Victorine
 */

public class gridSearchPanel extends JPanel implements ActionListener
{
  private static final int _SAMPLED  = 0;
  private static final int _RECIEVED = 1;
  private static final int _REPORTED = 2;
  private static final int _TOPS     = 3;

  private static final String SEARCH_BY[] = { "Sampled Date: ",  "Received Date: ",
                                              "Reported Date: ", "Formation: " };

  private static final int _WELLINGTON = 0;
  private static final int _BEMIS      = 1;

  private static final String FIELD[]  = { "Wellington", "Bemis" };

  // Input Variables

  private Observable     notifier = null;

  // Global Variables

  private int            iDate  = _SAMPLED;
  private int            iField = _WELLINGTON;

  private gridStartTable pTable = null; // Data Type List Table
  private gridTypeStruct stType = null; // Data Type Lists
  private String         plsql  = "";
  private String         search = "";
  private String         sField = FIELD[iField];

  // Global Panel Widgets

  private JRadioButton rbWellington = new JRadioButton(); // Wellington Field
  private JRadioButton rbBemis      = new JRadioButton(); // Bemus-Shutts Field

  private JButton btnSampled  = new JButton();
  private JButton btnReceived = new JButton();
  private JButton btnReported = new JButton();
  private JButton btnTops     = new JButton();
  private JButton btnSelect   = new JButton();

  /** Constructor gridSearchPanel()
   *  <p> This is the Constructor for this class.
   *  @param parent   = The Parent Frame calling this Panel
   *  @param stStruct = The Gridding & Mapping Module Data Structure
   */

  public gridSearchPanel(Observable notifier)
  {
    try
    {
	  this.notifier = notifier;

      jbInit();

      setButtons();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      String sMessage = new String("gridSearchPanel: " +  ex.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                     sMessage,
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Panel.
   * @throws Exception
   */

  void jbInit() throws Exception
  {
    JPanel      pnlTop     = new JPanel();
    JPanel      pnlButtons = new JPanel();
    JPanel      pnlWest    = new JPanel();
    JPanel      pnlEast    = new JPanel();
    JPanel      pnlCenter  = new JPanel();
    JPanel      pnlSouth   = new JPanel();
    JScrollPane scroll     = new JScrollPane();
    ButtonGroup group      = new ButtonGroup();

    TitledBorder titledBorderWest = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Search By");
    titledBorderWest.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Build Panel

    this.setLayout(new BorderLayout());

    pnlTop.setLayout(new GridLayout());

    pnlWest.setLayout(new GridLayout(3,1));
    pnlWest.setBorder(titledBorderWest);

    btnSampled.setFont(new java.awt.Font("Monospaced", 1, 11));
    btnSampled.setText("Sampled Date ");
    btnSampled.addActionListener(this);

    btnReceived.setFont(new java.awt.Font("Monospaced", 1, 11));
    btnReceived.setText("Received Date");
    btnReceived.addActionListener(this);

    btnReported.setFont(new java.awt.Font("Monospaced", 1, 11));
    btnReported.setText("Reported Date");
    btnReported.addActionListener(this);

//    pnlEast.setLayout(new BorderLayout());
    pnlEast.setLayout(new GridLayout(2,1));
    pnlEast.setBorder(titledBorderWest);

    btnTops.setFont(new java.awt.Font("Monospaced", 1, 11));
    btnTops.setText("Formations");
    btnTops.addActionListener(this);

    rbWellington.setFont(new java.awt.Font("Dialog", 1, 13));
    rbWellington.setText( FIELD[_WELLINGTON] );
    rbWellington.setSelected(true);
    rbWellington.addActionListener(this);

    rbBemis.setFont(new java.awt.Font("Dialog", 1, 13));
    rbBemis.setText( FIELD[_BEMIS] );
    rbBemis.addActionListener(this);

    pnlCenter.setLayout(new BorderLayout());

    pTable = new gridStartTable();
    scroll = pTable.getScrollPane();

    pnlSouth.setLayout(new BorderLayout());

    btnSelect.setFont(new java.awt.Font("Monospaced", 1, 11));
    btnSelect.setText("Select");
    btnSelect.addActionListener(this);

    // Attach widgets to panel

    this.add(pnlTop,          BorderLayout.NORTH);
    pnlTop.add(pnlWest,       null);
    pnlWest.add(btnSampled,   null);
    pnlWest.add(btnReceived,  null);
    pnlWest.add(btnReported,  null);

    pnlTop.add(pnlEast,       null);
    pnlEast.add(rbWellington, null);
    pnlEast.add(rbBemis,      null);

    group.add(rbWellington);
    group.add(rbBemis);

  //  pnlEast.add(btnTops,     BorderLayout.NORTH);

    this.add(pnlCenter,       BorderLayout.CENTER);
    pnlCenter.add(scroll,     BorderLayout.CENTER);

    this.add(pnlSouth,        BorderLayout.SOUTH);
    pnlSouth.add(btnSelect,   BorderLayout.CENTER);
  }

  /** Method close()
   * <p> This method will deallocate memory.
   */

  public void close()
  {
	// Input Variables

	notifier = null;

    // Global Variables

    if (stType != null)
      stType.delete();
    stType = null;

    if (pTable != null)
      pTable.close();
    pTable = null;

    plsql  = null;
    search = null;
    sField = null;

    // Global Panel Widgets

    rbWellington = null; // Wellington Field
    rbBemis      = null; // Bemus-Shutts Field

    btnSampled  = null;
    btnReceived = null;
    btnReported = null;
    btnTops     = null;
    btnSelect   = null;
  }

  /* ===================================================================== *
   * --------------------------- GET METHODS ----------------------------- *
   * ===================================================================== */

  /** Method getSQL()
   *  <p> This method will retrieve the PLSQL to retrieve the Brine data from
   *      the KGS database.
   */

  public String getSQL() { return (plsql); }

  /** Method select()
   *  <p> This method will create the PLSQL call to retrieve Brine Data
   */

  public void select()
  {
	String str = new String(pTable.getData());

	switch (iDate)
	{
      case _SAMPLED:
      case _RECIEVED:
      case _REPORTED:
        plsql = new String( grid.gridStruct.BY_DATE + iDate +
                            "&sTime=" + str +
                            "&field=" + sField);
        break;
      case _TOPS:
        plsql = new String(grid.gridStruct.BY_TOPS + str);
        break;
    }

    search = new String( SEARCH_BY[iDate] + str );

    if (notifier != null)
      notifier.notifyObservers( new String("Get Brine Data") );

//    System.out.println(plsql);
  }

  /** Method getSearchData()
   *  <p> This method will get the search parameter data
   */

  public String getSearchData() { return (search); }

  /* ===================================================================== *
   * -------------------------- READ ACTIONS ----------------------------- *
   * ===================================================================== */

  /** Method readTimesData()
   * <p> This method will read the Times Data from the KGS DB xml stream
   * @param  sPath = The URL Path to retrieve Brine Data
   */

  public void readTimesData()
  {
  	ReadGridDataXMLFile pRead =
	  new ReadGridDataXMLFile(grid.io.ReadGridDataXMLFile.URL);
    stType = pRead.Process( grid.gridStruct.DATE_LIST + iDate + "&field=" + sField);

    pTable.setData(stType);
  }

  /** Method readFormationData()
   * <p> This method will read the Times Data from the KGS DB xml stream
   * @param  sPath = The URL Path to retrieve Brine Data
   */

  public void readFormationData()
  {
  	ReadGridDataXMLFile pRead =
	  new ReadGridDataXMLFile(grid.io.ReadGridDataXMLFile.URL);
    stType = pRead.Process( grid.gridStruct.TOPS_LIST );

    pTable.setData(stType);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will enable or disable buttons depending on data
   */

  private void setButtons()
  {
	if (btnSelect != null)
	{
	  btnSelect.setEnabled(false);

	  if (pTable.getTotalRows() > 0)
  	    btnSelect.setEnabled(true);
	}
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions for this Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == rbWellington) // Wellington Field
    {
      if (stType != null)
        stType.delete();
      stType = null;

      pTable.setData(stType);

	  iField = _WELLINGTON;
	  sField = new String(FIELD[iField]);
	}

    if (event.getSource() == rbBemis)      // Bemus-Shutts Field
    {
      if (stType != null)
        stType.delete();
      stType = null;

      pTable.setData(stType);

	  iField = _BEMIS;
	  sField = new String(FIELD[iField]);
	}

    if (event.getSource() == btnSampled)
    {
	  iDate = _SAMPLED;
	  readTimesData();
	}
    if (event.getSource() == btnReceived)
    {
	  iDate = _RECIEVED;
	  readTimesData();
	}
    if (event.getSource() == btnReported)
    {
	  iDate = _REPORTED;
	  readTimesData();
	}
    if (event.getSource() == btnTops)
    {
	  iDate = _TOPS;
	  readFormationData();
	}
	if (event.getSource() == btnSelect)
	{
	  select();
	}

	setButtons();
  }
}
