/*
 * @gridStartTable.java Version 1.1 06/25/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.gui;

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;

import gui.guiTable;

import grid.gridTypeStruct;

/** Class gridStartTable
 *  <p> This Class will display 2 column array Table.
 *
 *  @version 1.1 06/25/2015
 *  @author  John R. Victorine
 */

public class gridStartTable
{
  private int      iRows     = 0;
  private int      iColumns  = 2;
  private String   sColumn[] = {"Total", "Description"};
  private Object   oData[][] = null;
  private int      iData[]   = null;
  private guiTable pTable    = null;

  private gridTypeStruct st  = null;

  /** Constructor gridStartTable()
   *  <p> This is the Constructor for this class.
   */

  public gridStartTable()
  {
    pTable = new guiTable(0, iColumns, sColumn);
    pTable.resizeColumn(0, 10);
  }

  /** Constructor gridStartTable()
   *  <p> This is the Constructor for this class.
   *  @param st = Grid Type Structure
   */

  public gridStartTable(gridTypeStruct st)
  {
	this.st = st;

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    pTable.resizeColumn(0, 10);

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /** Method populateList
   *  <p> This method will load the Data Array for the guiTable.
   */

  void populateList()
  {
	int i = 0;
    int iCount = 0;

    oData = new Object[1][2];
    iData = new int[1];

    oData[0][0] = new String( "" );
    oData[0][1] = new String( "" );
    iData[0]    = 0;

    if (st != null)
    {
      for (i=0; i<st.iRows; i++)
      {
	    if (st.iWells[i] > 3)
	    {
	  	  iCount++;
	    }
	  }

      if (iCount > 0)
      {
        iRows = 0;
        oData = new Object[iCount][2];
        iData = new int[iCount];

        for (i=0; i<st.iRows; i++)
        {
	  	  if (st.iWells[i] > 3)
		  {
            oData[iRows][0] = new String( "" + st.iWells[i] );
            oData[iRows][1] = new String( st.sValue[i] );
            iData[iRows]    = i;

            iRows++;
		  }
		}
      }
	}
  }

  /** Method close()
   *  <p> This method will deallocate the memory for the data to force Java
   *      to clear allocated memory
   */

  public void close()
  {
    sColumn = null;
    oData   = null;
    iData   = null;

    if (pTable != null)
      pTable.close();
    pTable    = null;

    st  = null;
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   *  <p> This method will reload the table
   *  @param st = Grid Type Structure
   */

  public void setData(gridTypeStruct st)
  {
	this.st = st;

    populateList();

    pTable.reload(iRows, oData);

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getSelectedRow()
   *  <p> This method will return the selected row.
   *  @return iSelectedRow - Selected Row.
   */

  public int getSelectedRow() { return (pTable.getSelectedRow()); }

  /** Method getData()
   *  <p> This method will return the selected row data value.
   *  @return sValue - Selected Row data value.
   */

  public String getData()
  {
	String sValue = "";

	if (st!= null)
	{
      if (pTable.getSelectedRow() < st.iRows)
      {
	    sValue = new String( st.sValue[iData[pTable.getSelectedRow()]] );
//	    sValue = new String( st.sValue[pTable.getSelectedRow()] );
	  }
	}

	return (sValue);
  }

  /** Method getTotalRows()
   *  <p> This method will return the total number of rows.
   *  @return iRows - Total Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}
