/*
 * @gridIORead.java Version 1.1 06/25/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.io;

import java.io.*;
import java.awt.*;
import javax.swing.*;

import iqstrat.iqstratHeadersListStruct; // Well Header List Structure
import iqstrat.iqstratHeadersStruct;     // Well Header Data Structure
import iqstrat.io.ReadHeadersXMLFile;    // Read Well Header XML File

import brine.brineListStruct;            // Brine Data List Structure
import brine.io.ReadBrineDataXMLFile;    // Read Brine Data XML File

/** Class gridIORead
 *  <p> This Class will retrieve the Brine Data from the KGS Database.
 *
 *  @version 1.1 06/25/2015
 *  @author  John R. Victorine
 */

public class gridIORead
{
  /* ===================================================================== *
   * ----------------- LOAD WELL HEADER INFORMATION DATA ----------------- *
   * ===================================================================== */

  /** Method readWellData()
   * <p> This method will read the Well Information from the KGS DB xml stream
   * @return st  = The Well Header Data List Structure
   */

  public static iqstratHeadersListStruct readWellData()
  {
	iqstratHeadersListStruct st = null;

    ReadHeadersXMLFile pRead = new ReadHeadersXMLFile(
		                            iqstrat.io.ReadHeadersXMLFile.URL);

    st = pRead.Process( grid.gridStruct.BRINE_WELLS );

    String sError = pRead.GetError();
    if (sError.length() > 0)
      JOptionPane.showMessageDialog( (Component)null,
                                     sError,
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE );
    return (st);
  }

  /* ===================================================================== *
   * -------------------------- LOAD BRINE DATA -------------------------- *
   * ===================================================================== */

  /** Method readBrineData()
   * <p> This method will read the Brine Data from the KGS DB xml stream
   * @param  sPath = The URL Path to retrieve Brine Data
   * @return st    = The Brine Data List Structure
   */

  public static brineListStruct readBrineData( String sPath )
  {
	brineListStruct st = null;

  	ReadBrineDataXMLFile pRead =
	  new ReadBrineDataXMLFile(brine.io.ReadBrineDataXMLFile.URL);
    st = pRead.Process( sPath );

    String sError = pRead.GetError();
    if (sError.length() > 0)
      JOptionPane.showMessageDialog( (Component)null,
                                     sError,
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE );
    return (st);
  }
}

/*
 *  @version 1.1 06/25/2015
 *  @author  John Victorine
 */
