/*
 * @guiSearch.java Version 1.1 03/08/2012
 *
 * Copyright (c) 2010 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gui;

import java.io.*;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/** Class guiSearch
 *  <p> This Class will create the Lithology Control Plot Frame.
 *
 *  @version 1.1 03/08/2012
 *  @author  John R. Victorine
 */

public class guiSearch extends JFrame implements ActionListener
{
  // Input Variables

  private Observable  notifier = null;     // Observable

  // Global Frame Widgets

  private JButton btnContinue     = new JButton();
  private JButton btnCancel       = new JButton();
  private JButton btnFind         = new JButton();

  private JTextField txtDirectory = new JTextField();
  private JTextField txtFilename  = new JTextField();

  /** Constructor guiSearch()
   *  <p> This is the Constructor for this class.
   *  @param notifier = Observable
   */

  public guiSearch(  Observable notifier )
  {
    try
    {
      this.notifier = notifier;

      jbInit();

      addWindowListener( new guiSearch_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    String sPath = kgs.io.kgsIOUser.getHomeDir();

    JPanel pnlButtons   = new JPanel();
    JPanel pnlFiles     = new JPanel();
    JPanel pnlDirectory = new JPanel();
    JPanel pnlFilename  = new JPanel();
    JPanel pnlDirButton = new JPanel();
    JPanel pnlDirText   = new JPanel();

    TitledBorder titledBorderDir = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Enter Directory Path:");

    TitledBorder titledBorderFile = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Enter Filename:");

    titledBorderDir.setTitleFont(new java.awt.Font("Dialog", 1, 11));
    titledBorderFile.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle("Enter Directory Path & Filename");

    // Directory & Filename Panel

    pnlFiles.setLayout(new GridLayout()); //(2,1));

    // -- Directory Panel

    pnlDirectory.setBorder(titledBorderDir);
    pnlDirectory.setLayout(new BorderLayout());

    // .. Directory Text Panel

    pnlDirText.setLayout(new BorderLayout());

    txtDirectory.setText( sPath );

    // .. Directory Search Button Panel

    pnlDirButton.setLayout(new BorderLayout());

    btnFind.setFont(new java.awt.Font("Dialog", 1, 11));
    btnFind.setText("Search");
    btnFind.addActionListener(this);

    // -- Filename Panel

    pnlFilename.setBorder(titledBorderFile);
    pnlFilename.setLayout(new BorderLayout());

    txtFilename.setText( "???" );

    // Buttons Panel

    pnlButtons.setBorder(BorderFactory.createEtchedBorder());

    btnContinue.setFont(new java.awt.Font("Dialog", 1, 11));
    btnContinue.setText("Continue");
    btnContinue.addActionListener(this);

    btnCancel.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCancel.setText("Cancel");
    btnCancel.addActionListener(this);

    this.getContentPane().add(pnlButtons, BorderLayout.SOUTH);
    pnlButtons.add(btnContinue,           null);
    pnlButtons.add(btnCancel,             null);

    this.getContentPane().add(pnlFiles,   BorderLayout.CENTER);
    pnlFiles.add(pnlDirectory,            null);
    pnlDirectory.add(pnlDirButton,        BorderLayout.EAST);
    pnlDirButton.add(btnFind,             BorderLayout.SOUTH);

    pnlDirectory.add(pnlDirText,          BorderLayout.CENTER);
    pnlDirText.add(txtDirectory,          BorderLayout.CENTER);

//    pnlFiles.add(pnlFilename,             null);
    pnlFilename.add(txtFilename,          BorderLayout.CENTER);

    // Display the Frame

    this.setSize(new Dimension(500, 140)); //200));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getDirectory()
   * <p> This method will return the Directory Path
   * @return the Directory Path
   */

  public String getDirectory() { return (txtDirectory.getText()); }

  /** Method getFilename()
   * <p> This method will return the Filename
   * @return the Filename
   */

  public String getFilename()  { return (txtFilename.getText()); }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method find()
   * <p> This method will search for the PC Directory Path to place LAS File
   */

  private void find()
  {
    String         sPath = kgs.io.kgsIOUser.getHomeDir();
    String         sFile = "";
    JFileChooser   fc    = new JFileChooser();
    int            iVal  = -1;

    fc.setFileSelectionMode( JFileChooser.FILES_AND_DIRECTORIES );//.DIRECTORIES_ONLY );
    fc.setCurrentDirectory(new File( sPath ) );
//    fc.setSelectedFile( new File( sFile ) );

    iVal = fc.showOpenDialog( this );
    if (iVal == JFileChooser.APPROVE_OPTION)
    {
      sPath = fc.getSelectedFile().getPath();
      txtDirectory.setText( sPath );

      sFile = fc.getSelectedFile().getName();
      txtFilename.setText( sFile );
    }
  }

  /** Method select()
   * <p> This method will send a message to retrieve the Directory & Filename.
   */

  private void select()
  {
    if (notifier != null)
      notifier.notifyObservers(new String("Get Directory & Filename"));
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    notifier     = null;

    // Global Frame Widgets

    btnContinue  = null;
    btnCancel    = null;
    btnFind      = null;

    txtDirectory = null;
    txtFilename  = null;

    dispose();
  }

  /** METHOD iqstratControlPlotFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class guiSearch_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnFind)     { find(); }
    if (event.getSource() == btnContinue) { select(); }
    if (event.getSource() == btnCancel)   { close(); }
  }
}

/*
 *  @version 1.1 03/08/2012
 *  @author  John Victorine
 */
