/*
 * @pcBrineTable.java Version 1.1 10/28/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;

import gui.guiPCTable;

import pc.pcDataStruct;  // Principal Components Data Structure

/** Class pcBrineTable
 *  <p> This Class will display the Zonation Data.
 *
 *  @version 1.1 10/28/2015
 *  @author  John R. Victorine
 */

public class pcBrineTable
{
  private int          iRows     = 0;
  private int          iColumns  = 0;
  private String       sColumn[] = null;
  private int          iLength[] = null;

  private Object       oData[][] = null;
  private guiPCTable   pTable    = null;

  private pcDataStruct st        = null; // Principal Components Data Structure
  private int          iChange   = 0;

  public static final Color cPOS_1  = new Color(255, 255, 204);
  public static final Color cPOS_2  = new Color(255, 255, 102);
  public static final Color cPOS_3  = new Color(255, 255,  51);
  public static final Color cPOS_4  = new Color(255, 255,   0);
  public static final Color cPOS_5  = new Color(255, 204, 102);
  public static final Color cPOS_6  = new Color(255, 204,  51);
  public static final Color cPOS_7  = new Color(255, 204,   0);
  public static final Color cPOS_8  = new Color(255, 153,  51);
  public static final Color cPOS_9  = new Color(255, 153,   0);
  public static final Color cPOS_10 = new Color(204, 102,   0);
  public static final Color cPOS_11 = new Color(204, 102,  51);
  public static final Color cPOS_12 = new Color(153,  51,   0);
  public static final Color cPOS_13 = new Color(153,  51,  51);
  public static final Color cPOS_14 = new Color(102,  51,   0);
  public static final Color cPOS_15 = new Color(102,  51,  51);
  public static final Color cPOS_16 = new Color( 51,   0,   0);

  public static final int   iPOS    = 16;

  public static final Color cNEG_1  = new Color(204, 255, 204);
  public static final Color cNEG_2  = new Color(153, 255, 153);
  public static final Color cNEG_3  = new Color(102, 255, 102);
  public static final Color cNEG_4  = new Color( 51, 255,  51);
  public static final Color cNEG_5  = new Color(  0, 255,   0);
  public static final Color cNEG_6  = new Color(  0, 255, 153);
  public static final Color cNEG_7  = new Color( 51, 204, 153);
  public static final Color cNEG_8  = new Color(  0, 204, 255);
  public static final Color cNEG_9  = new Color( 51, 153, 255);
  public static final Color cNEG_10 = new Color(  0, 153, 255);
  public static final Color cNEG_11 = new Color(  0, 102, 204);
  public static final Color cNEG_12 = new Color( 51, 102, 204);
  public static final Color cNEG_13 = new Color(102, 102, 204);
  public static final Color cNEG_14 = new Color(102,  51, 204);
  public static final Color cNEG_15 = new Color(102,  51, 102);
  public static final Color cNEG_16 = new Color( 51,   0,  51);

  public static final int   iNEG    = 16;

  public double dataM[]     = null; // Array of Means of the Anions & Cations Columns [BRINE]
  public double dSD[]       = null; // Array of Standard Deviation [Brine]

  public static final int    iColors     = 24;
  public static final double dFactor[][] =
  {
	{ -100.0, -3.0 }, { -3.0, -2.5 }, { -2.5, -2.4 }, { -2.4, -2.3 },
	{ -2.3, -2.2 },   { -2.2, -2.1 }, { -2.1, -2.0 }, { -2.0, -1.9 }, { -1.9, -1.8 },
	{ -1.8, -1.7 },   { -1.7, -1.6 }, { -1.6, -1.5 }, { 1.5,  1.6 },  { 1.6,  1.7 },
	{ 1.7,  1.8 },    { 1.8,  1.9 },  { 1.9,  2.0 },  { 2.0,  2.1 },  { 2.1,  2.2 },
	{ 2.2,  2.3 },    { 2.3,  2.4 },  { 2.4,  2.5 },  { 2.5,  3.0 },  { 3.0,  100.0 },
  };

  public static final Color COLORS[] =
  {
	cNEG_12, cNEG_11, cNEG_10, cNEG_9,
	cNEG_8,  cNEG_7,  cNEG_6,  cNEG_5,  cNEG_4,
	cNEG_4,  cNEG_2,  cNEG_1,  cPOS_1,  cPOS_2,
	cPOS_3,  cPOS_4,  cPOS_5,  cPOS_6,  cPOS_7,
	cPOS_8,  cPOS_9,  cPOS_10, cPOS_11, cPOS_12
  };

  /** Constructor pcBrineTable()
   *  <p> This is the Constructor for this class.
   *  @param st       = principal components data structure
   *  @param iColumns = total number of columns in table
   *  @param sColumn  = array of column headers
   *  @param iLength  = array of column widths
   */

  public pcBrineTable( pcDataStruct st, int iColumns, String sColumn[], int iLength[] )
  {
	this.st       = st;
    this.iColumns = iColumns;
    this.sColumn  = sColumn;
    this.iLength  = iLength;

    populateList();

    if (iRows > 0)
      pTable = new guiPCTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiPCTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);

    for (int i=0; i<st.iColumns; i++)
    {
      setCellColors( true, i+1, iColors,
                     buildCellColors( iColors, st.dataM[i], st.dSD[i], dFactor ),
                     COLORS );
	}
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
    sColumn  = null;
    iLength  = null;
    oData    = null;

    if (pTable != null)
      pTable.close();
    pTable   = null;

    st       = null;
  }

  /** Method resetColumnWidths()
   *  <p> This method will reset the column widths to a specific size.
   */

  private void resetColumnWidths()
  {
    for (int i=0; i<iColumns; i++)
    {
      pTable.resizeColumn(i, iLength[i]);
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method populateList()
   *  <p> This method will populate the Zonation data.
   */

  private void populateList()
  {
    int        i,j;
    int        iCount  = 0;
    double     data    = 0.0;
    String     sLith   = "";

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if (st != null)
    {
      iChange = 1;
      iCount  = 0;
      oData   = new Object[st.iRows][iColumns];

      for (i=0; i<st.iRows; i++)
      {
		oData[iCount][0]  = new String(st.sData[i][0]);
		for (j=0; j<st.iColumns; j++)
		{
		  oData[iCount][j+1] = new Double(st.data[i][j]);
		}
        iCount++;
	  }
    }

    iRows = iCount;
  }

  /** Method setData()
   *  <p> This method will reload the data.
   * @param st = The Zonation Data Structure
   */

  public void setData(pcDataStruct st)
  {
    this.st = st;

    populateList();

    pTable.reload(iRows, oData);

    if (iChange == 0)
      iRows = 0;
  }

  /** Method setCellColors()
   *  <p> This method will set the background of the column based on the
   *      value of the Pay.
   *  @param bv = true color is on; false color is off
   */

  public void setCellColors(boolean bv, int iCol, int iClrs, double dVal[][], Color bkgnd[])
  {
	if (bv)
      pTable.colorColumns( iCol, iClrs, dVal, bkgnd );
    else
      pTable.colorColumns( iCol, 0.0, Color.white );
  }

  /** Method buildCellColors()
   *  <p> This method will build the color array through computation for a lower
   *      to upper with relation to the mean and standard deviation.
   *      Each color range { Mean + L * SD, Mean + U * SD }, where Mean is the
   *      average value of the column, SD is the standard deviation, L is the lower
   *      standard deviation factor, U is the upper standard deviation factor.
   *  @param  iColors = Total number of colors to build
   *  @param  dMean   = The Mean or average value of the column data
   *  @param  dSD     = The standard deviation
   *  @param  dFactor = Array of multipling factors of the standard deviation
   *                    color schema
   *  @return dRange  = Array of values to trigger painting the background of
   *                    table cell.
   */

  public static double[][] buildCellColors( int iColors, double dMean, double dSD,
                                            double dFactor[][] )
  {
	double dRange[][] = null;

	if (iColors > 1)
	{
	  dRange = new double[iColors][2];

	  for (int i=0; i<iColors; i++)
	  {
		dRange[i][0] = dMean + dFactor[i][0] * dSD;  // L - Lower Value
		dRange[i][1] = dMean + dFactor[i][1] * dSD;  // U - Upper Value
	  }
	}

	return (dRange);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTotalRows()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 10/28/2015
 *  @author  John Victorine
 */
