/*
 * @pcCompPanel.java Version 1.1 10/28/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import javax.swing.*;
import javax.swing.event.*;

import pc.gui.pcTable;  // Brine Statistics Table
import plot.plotBufferedWidget;

/** Class pcCompPanel
 *  <p> This Class will create the Table Panel.
 *
 *  @version 1.1 10/28/2015
 *  @author  John R. Victorine
 */

public class pcCompPanel extends JPanel implements ActionListener
{
  public static final String CURVES[][] = brine.brineStandardTools.CURVES;

  // Input Variables

  private String       sLabels[] = null;
  private double       data[][]  = null;
  private int          iColumns  = 0;
  private String       sColumn[] = null;
  private int          iLength[] = null;

  // Global Variables


  private pcTable      pTable    = null;

  // Global Frame Widgets

  /** Constructor pcCompPanel()
   *  <p> This is the Constructor for this class.
   *  @param sLabels  = array of labels used in Column 1
   *  @param data     = double array of the data to be displayed
   *  @param iColumns = total number of columns in table
   *  @param sColumn  = array of column headers
   *  @param iLength  = array of column widths
   */

  public pcCompPanel( String sLabels[], double data[][],
                      int iColumns, String sColumn[], int iLength[] )
  {
    try
    {
  	  this.sLabels  = sLabels;
	  this.data     = data;

      this.iColumns = iColumns;
      this.sColumn  = sColumn;
      this.iLength  = iLength;

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JScrollPane scroll  = new JScrollPane();

    int iHeight = getHeight();

    this.setLayout(new BorderLayout());
    this.setPreferredSize(new Dimension(10, iHeight));

    // Center Panel

    pTable = new pcTable( sLabels, data, iColumns, sColumn, iLength );
    scroll = pTable.getScrollPane();

    // Attach Widgets to Panel

    this.add(scroll,   BorderLayout.CENTER);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
	sLabels  = null;
	data     = null;

    sColumn  = null;
    iLength  = null;

    if (pTable != null)
      pTable.close();
    pTable    = null; // Brine Data Table

    // Global Frame Widgets

  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getWidth()
   *  <p> This method will determine the width of the Frame from the width of
   *      the table.
   */

  public int getWidth()
  {
	int iWidth = 0;

	if (iLength != null)
	{
	  if (iLength.length > 0)
	  {
		for (int i=0; i<iLength.length; i++)
		{
		  iWidth = iWidth + 8 * iLength[i];
		}
	  }
	}

	return (iWidth + 25);
  }

  /** Method getHeight()
   *  <p> This method will determine the height of the Frame.
   */

  public int getHeight()
  {
	int iHeight = 0;

	if (iLength != null)
	{
	  iHeight = iLength.length * 18;
	}

	return (iHeight);
  }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

	image = plot.plotBufferedWidget.componentToImage( this, true );

	return (image);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   *  <p> This method will reload the data.
   *  @param sLabels  = array of labels used in Column 1
   *  @param data     = double array of the data to be displayed
   */

  public void setData( String sLabels[], double data[][] )
  {
	if (pTable != null)
	{
	  pTable.setData( sLabels, data );
	}
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
  }
}


/*
 *  @version 1.1 10/28/2015
 *  @author  John Victorine
 */

