/*
 * @pcEigenPanel.java Version 1.1 10/28/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import pc.gui.pcEigenTable;  // Brine Statistics Table

/** Class pcEigenPanel
 *  <p> This Class will create the Table Panel.
 *
 *  @version 1.1 10/28/2015
 *  @author  John R. Victorine
 */

public class pcEigenPanel extends JPanel implements ActionListener
{
  // Input Variables

  private double       data[]    = null;
  private int          iColumns  = 0;
  private String       sColumn[] = null;
  private int          iLength[] = null;

  // Global Variables

  private pcEigenTable pTable    = null;

  // Global Frame Widgets

  /** Constructor pcEigenPanel()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = Global IQSTRAT Data Structure
   */

  public pcEigenPanel( double data[], int iColumns, String sColumn[], int iLength[] )
  {
    try
    {
      this.data     = data;
      this.iColumns = iColumns;
      this.sColumn  = sColumn;
      this.iLength  = iLength;

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JScrollPane scroll  = new JScrollPane();

    this.setLayout(new BorderLayout());
    this.setPreferredSize(new Dimension(10, 60));

    // Center Panel

    pTable = new pcEigenTable(data, iColumns, sColumn, iLength );
    scroll = pTable.getScrollPane();

    // Attach Widgets to Panel

    this.add(scroll,   BorderLayout.CENTER);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    data    = null; // Brine List Structure

    // Global Variables

    if (pTable != null)
      pTable.close();
    pTable    = null; // Brine Data Table

    iColumns  = 0;
    sColumn   = null;
    iLength   = null;

    // Global Frame Widgets

  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getWidth()
   *  <p> This method will determine the width of the Frame from the width of
   *      the table.
   */

  public int getWidth()
  {
	int iWidth = 0;

	if (iLength != null)
	{
	  if (iLength.length > 0)
	  {
		for (int i=0; i<iLength.length; i++)
		{
		  iWidth = iWidth + 8 * iLength[i];
		}
	  }
	}

	return (iWidth + 25);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   *  <p> This method will reload the data.
   * @param st = The Zonation Data Structure
   */

  public void setData(double data[])
  {
	if (pTable != null)
	{
	  pTable.setData(data);
	}
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
  }
}


/*
 *  @version 1.1 10/28/2015
 *  @author  John Victorine
 */

