/*
 * @pcEigenTable.java Version 1.1 10/28/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;

import gui.guiTable;

import pc.pcDataStruct;  // Principal Components Data Structure

/** Class pcEigenTable
 *  <p> This Class will display the Zonation Data.
 *
 *  @version 1.1 10/28/2015
 *  @author  John R. Victorine
 */

public class pcEigenTable
{
  private int          iRows     = 0;
  private int          iColumns  = 0;
  private String       sColumn[] = null;
  private int          iLength[] = null;

  private double       data[]    = null;

  private Object       oData[][] = null;
  private guiTable     pTable    = null;

  private pcDataStruct st        = null; // Principal Components Data Structure
  private int          iChange   = 0;

  /** Constructor pcEigenTable()
   *  <p> This is the Constructor for this class.
   *  @param data     = principal components data structure
   *  @param iColumns = total number of columns in table
   *  @param sColumn  = array of column headers
   *  @param iLength  = array of column widths
   */

  public pcEigenTable( double data[], int iColumns, String sColumn[], int iLength[] )
  {
	this.data      = data;
    this.iColumns = iColumns;
    this.sColumn  = sColumn;
    this.iLength  = iLength;

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
    sColumn  = null;
    iLength  = null;
    data     = null;
    oData    = null;

    if (pTable != null)
      pTable.close();
    pTable   = null;

    st       = null;
  }

  /** Method resetColumnWidths()
   *  <p> This method will reset the column widths to a specific size.
   */

  private void resetColumnWidths()
  {
    for (int i=0; i<iColumns; i++)
    {
      pTable.resizeColumn(i, iLength[i]);
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method populateList()
   *  <p> This method will populate the Zonation data.
   */

  private void populateList()
  {
    int        i,j;
    int        iCount  = 0;
    double     dSum    = 0.0;
    double     dVal    = 0.0;

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if (data != null)
    {
      iChange = 1;
      iCount  = 0;
      oData   = new Object[2][iColumns];

      oData[iCount][0]  = new String("Eigen");
   	  for (j=1; j<iColumns; j++)
	  {
		dVal = (double) ((int) (data[j-1] * 10000.0) / 10000.0);
	    oData[iCount][j] = new Double( dVal );
	    if (data[j-1] > 0.0)
	      dSum = dSum + data[j-1];
	  }
      iCount++;

      oData[iCount][0]  = new String("%");
   	  for (j=1; j<iColumns; j++)
	  {
		dVal = (double) ((int) (1000.0 * data[j-1] / dSum) / 10.0);
	    oData[iCount][j] = new Double( dVal );
	  }
      iCount++;
    }

    iRows = iCount;
  }

  /** Method setData()
   *  <p> This method will reload the data.
   * @param st = The Zonation Data Structure
   */

  public void setData(double data[])
  {
    this.data = data;

    populateList();

    pTable.reload(iRows, oData);

    if (iChange == 0)
      iRows = 0;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTotalRows()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 10/28/2015
 *  @author  John Victorine
 */
