/*
 * @pcPlotLimitsPanel.java Version 1.1 09/12/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.net.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/** Class pcPlotLimitsPanel
 *  <p> This Class will create the Decline Curve Calculator Panel.
 *
 *  @version 1.1 09/12/2013
 *  @author  John R. Victorine
 */

public class pcPlotLimitsPanel extends JPanel
{
  private static final int XMIN = 0;
  private static final int XMAX = 1;
  private static final int XINC = 2;

  private static final int YMIN = 3;
  private static final int YMAX = 4;
  private static final int YINC = 5;

  private Observable notifier  = null; // Observalbles Object

  private String sTitle1        = "";   // First line of the plot title
  private String sTitle2        = "";   // Second line of the plot title
  private String sTitle3        = "";   // Third line of the plot title

  private double dXmax          = 0.0;  // Maximum X-Axis
  private double dXmin          = 0.0;  // Minimum X-Axis
  private double dXincr         = 0.0;  // Increment along X-Axis

  private double dYmax          = 0.0;  // Maximum Y-Axis
  private double dYmin          = 0.0;  // Minimum Y-Axis
  private double dYincr         = 0.0;  // Increment along X-Axis

  // Global Frame Widgets

  // -- Top Panel

  // .. Titles

  private JTextField txtTitle1 = new JTextField();  // 1st Title
  private JTextField txtTitle2 = new JTextField();  // 2nd Title
  private JTextField txtTitle3 = new JTextField();  // 3rd Title

  // .. X-Axis Text fields

  private JLabel     lblXAxis  = new JLabel();      // Column Title
  private JLabel     lblXmin   = new JLabel();      // Minimum X Label
  private JLabel     lblXmax   = new JLabel();      // Maximum X Label
  private JLabel     lblXincr  = new JLabel();      // Increment X Label

  private JTextField txtXmin   = new JTextField();  // Minimum X-Axis
  private JTextField txtXmax   = new JTextField();  // Maximum X-Axis
  private JTextField txtXincr  = new JTextField();  // Increment X-Axis

  // .. Y-Axis Text fields

  private JLabel     lblYAxis  = new JLabel();      // Column Title
  private JLabel     lblYmin   = new JLabel();      // Minimum Y Label
  private JLabel     lblYmax   = new JLabel();      // Maximum Y Label
  private JLabel     lblYincr  = new JLabel();      // Increment Y Label

  private JTextField txtYmin   = new JTextField();  // Minimum Y-Axis
  private JTextField txtYmax   = new JTextField();  // Maximum Y-Axis
  private JTextField txtYincr  = new JTextField();  // Increment Y-Axis

  /** Constructor pcPlotLimitsPanel()
   *  <p> This is the Constructor for this class.
   *  @param stList = Production Data List Structure
   */

  public pcPlotLimitsPanel( Observable notifier,
	                            String sTitle1, String sTitle2, String sTitle3,
                                double dXmax, double dXmin, double dXincr,
                                double dYmax, double dYmin, double dYincr )
  {
    try
    {
	  this.notifier = notifier;

      this.sTitle1  = new String( sTitle1 );
      this.sTitle2  = new String( sTitle2 );
      this.sTitle3  = new String( sTitle3 );

      this.dXmax    = dXmax;
      this.dXmin    = dXmin;
      this.dXincr   = dXincr;

      this.dYmax    = dYmax;
      this.dYmin    = dYmin;
      this.dYincr   = dYincr;

      jbInit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop            = new JPanel();

    this.setLayout(new BorderLayout());

    // -- Panel Top

    pnlTop = buildPlotControlPanel();

    // Attach Widgets

    this.add(pnlTop,     BorderLayout.NORTH);
  }

  private JPanel buildPlotControlPanel()
  {
	int i, j;

	JPanel panel             = new JPanel();

    JPanel pnlTitles         = new JPanel();
    JPanel pnlTitle1         = new JPanel();
    JPanel pnlTitle2         = new JPanel();
    JPanel pnlTitle3         = new JPanel();

    JPanel pnlAxes           = new JPanel();
    JPanel pnlXAxis          = new JPanel();
    JPanel pnlXAxisCenter    = new JPanel();
    JPanel pnlXMinimum       = new JPanel();
    JPanel pnlXMaximum       = new JPanel();
    JPanel pnlXIncrement     = new JPanel();

    JPanel pnlYAxis          = new JPanel();
    JPanel pnlYAxisCenter    = new JPanel();
    JPanel pnlYMinimum       = new JPanel();
    JPanel pnlYMaximum       = new JPanel();
    JPanel pnlYIncrement     = new JPanel();

    TitledBorder titledBorderTitle1 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "1st Plot Title");
    titledBorderTitle1.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderTitle2 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "2nd Plot Title");
    titledBorderTitle2.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderTitle3 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "3rd Plot Title");
    titledBorderTitle3.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderXAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "X-Axis");
    titledBorderXAxis.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderYAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Y-Axis");
    titledBorderYAxis.setTitleFont(new java.awt.Font("Dialog", 1, 13));

	// Build Production Plot Control Panel

    panel.setLayout(new BorderLayout());

    // .. Titles Panel

    pnlTitles.setLayout(new GridLayout(3,1));

    // .. .. 1st Title Panel

    pnlTitle1.setLayout(new BorderLayout());
    pnlTitle1.setBorder(titledBorderTitle1);

    txtTitle1.setText(sTitle1);
    txtTitle1.setFont(new java.awt.Font("Dialog", 1, 12));
    txtTitle1.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));

    pnlTitle2.setLayout(new BorderLayout());
    pnlTitle2.setBorder(titledBorderTitle2);

    txtTitle2.setText(sTitle2);
    txtTitle2.setFont(new java.awt.Font("Dialog", 1, 12));
    txtTitle2.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));


    pnlTitle3.setLayout(new BorderLayout());
    pnlTitle3.setBorder(titledBorderTitle3);

    txtTitle3.setText(sTitle3);
    txtTitle3.setFont(new java.awt.Font("Dialog", 1, 12));
    txtTitle3.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));

    // -- Axes Panel

    pnlAxes.setLayout(new GridLayout());

    // .. X-Axis Panel

    pnlXAxis.setLayout(new BorderLayout());
    pnlXAxis.setBorder(titledBorderXAxis);

    // .. .. X-Axis Label Panel

    lblXAxis.setText("Sum of Cations (meq/l)");
    lblXAxis.setHorizontalAlignment(SwingConstants.CENTER);
    lblXAxis.setFont(new java.awt.Font("Dialog", 1, 13));

    // .. .. X-Axis Center Panel

    pnlXAxisCenter.setLayout(new GridLayout(3,1));

    // .. - .. X-Axis Minimum Panel

    pnlXMinimum.setLayout(new BorderLayout());

    lblXmin.setText("Minimum:  ");
    lblXmin.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtXmin.setText("" + (int) dXmin);
    txtXmin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXmin.setFont(new java.awt.Font("Dialog", 1, 12));
    txtXmin.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));

    // .. - .. X-Axis Maximum Panel

    pnlXMaximum.setLayout(new BorderLayout());

    lblXmax.setText("Maximum:  ");
    lblXmax.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtXmax.setText("" + (int) dXmax);
    txtXmax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXmax.setFont(new java.awt.Font("Dialog", 1, 12));
    txtXmax.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));

    // .. - .. X-Axis Increment Panel

    pnlXIncrement.setLayout(new BorderLayout());

    lblXincr.setText("Increment:");
    lblXincr.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtXincr.setText("" + (int) dXincr);
    txtXincr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXincr.setFont(new java.awt.Font("Dialog", 1, 12));
    txtXincr.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));

    // .. Y Axis Panel

    pnlYAxis.setLayout(new BorderLayout());
    pnlYAxis.setBorder(titledBorderYAxis);

    // .. .. Y-Axis Label Panel

    lblYAxis.setText("Sum of Anions (meq/l)");
    lblYAxis.setHorizontalAlignment(SwingConstants.CENTER);
    lblYAxis.setFont(new java.awt.Font("Dialog", 1, 13));


    // .. .. Y-Axis Center Panel

    pnlYAxisCenter.setLayout(new GridLayout(3,1));

    // .. - .. Y-Axis Minimum Panel

    pnlYMinimum.setLayout(new BorderLayout());

    lblYmin.setText("Minimum:  ");
    lblYmin.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtYmin.setText("" + (int) dYmin);
    txtYmin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYmin.setFont(new java.awt.Font("Dialog", 1, 12));
    txtYmin.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));

    // .. - .. Y-Axis Maximum Panel

    pnlYMaximum.setLayout(new BorderLayout());

    lblYmax.setText("Maximum:  ");
    lblYmax.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtYmax.setText("" + (int) dYmax);
    txtYmax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYmax.setFont(new java.awt.Font("Dialog", 1, 12));
    txtYmax.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));

    // .. - .. Y-Axis Increment Panel

    pnlYIncrement.setLayout(new BorderLayout());

    lblYincr.setText("Increment:   ");
    lblYincr.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtYincr.setText("" + dYincr);
    txtYincr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYincr.setFont(new java.awt.Font("Dialog", 1, 12));
    txtYincr.addFocusListener(new pcPlotLimitsPanelFocusAdapter(this));

    // Add Widgets to Frame

    // -- Titles

    panel.add(pnlTitles,                  BorderLayout.NORTH);
    pnlTitles.add(pnlTitle1,              null);
    pnlTitle1.add(txtTitle1,              BorderLayout.CENTER);

    pnlTitles.add(pnlTitle2,              null);
    pnlTitle2.add(txtTitle2,              BorderLayout.CENTER);

    pnlTitles.add(pnlTitle3,              null);
    pnlTitle3.add(txtTitle3,              BorderLayout.CENTER);

    // -- Axes

    panel.add(pnlAxes,                    BorderLayout.CENTER);

    // .. X-Axis

    pnlAxes.add(pnlXAxis,                 null);
    pnlXAxis.add(lblXAxis,                BorderLayout.NORTH);

    pnlXAxis.add(pnlXAxisCenter,          BorderLayout.CENTER);
    pnlXAxisCenter.add(pnlXMinimum,       null);
    pnlXMinimum.add(lblXmin,              BorderLayout.WEST);
    pnlXMinimum.add(txtXmin,              BorderLayout.CENTER);

    pnlXAxisCenter.add(pnlXMaximum,       null);
    pnlXMaximum.add(lblXmax,              BorderLayout.WEST);
    pnlXMaximum.add(txtXmax,              BorderLayout.CENTER);

    pnlXAxisCenter.add(pnlXIncrement,     null);
    pnlXIncrement.add(lblXincr,           BorderLayout.WEST);
    pnlXIncrement.add(txtXincr,           BorderLayout.CENTER);

    // .. Y-Axis

    pnlAxes.add(pnlYAxis,                 null);
    pnlYAxis.add(lblYAxis,                BorderLayout.NORTH);

    pnlYAxis.add(pnlYAxisCenter,          BorderLayout.CENTER);
    pnlYAxisCenter.add(pnlYMinimum,       null);
    pnlYMinimum.add(lblYmin,              BorderLayout.WEST);
    pnlYMinimum.add(txtYmin,              BorderLayout.CENTER);

    pnlYAxisCenter.add(pnlYMaximum,       null);
    pnlYMaximum.add(lblYmax,              BorderLayout.WEST);
    pnlYMaximum.add(txtYmax,              BorderLayout.CENTER);

    pnlYAxisCenter.add(pnlYIncrement,     null);
    pnlYIncrement.add(lblYincr,           BorderLayout.WEST);
    pnlYIncrement.add(txtYincr,           BorderLayout.CENTER);

    return (panel);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    notifier   = null; // Observables

    sTitle1    = null; // First line of the plot title
    sTitle2    = null; // Second line of the plot title
    sTitle3    = null; // Third line of the plot title

    // Global Frame Widgets

    // -- Top Panel

    // .. Titles

    txtTitle1 = null; // 1st Title
    txtTitle2 = null; // 2nd Title
    txtTitle3 = null; // 3rd Title

    // .. X-Axis Text fields

    lblXAxis  = null; // Column Title
    lblXmin   = null; // Minimum X Label
    lblXmax   = null; // Maximum X Label
    lblXincr  = null; // Increment X Label

    txtXmin   = null; // Minimum X-Axis
    txtXmax   = null; // Maximum X-Axis
    txtXincr  = null; // Increment X-Axis

    // .. Y-Axis Text fields

    lblYAxis  = null; // Column Title
    lblYmin   = null; // Minimum Y Label
    lblYmax   = null; // Maximum Y Label
    lblYincr  = null; // Increment Y Label

    txtYmin   = null; // Minimum Y-Axis
    txtYmax   = null; // Maximum Y-Axis
    txtYincr  = null; // Increment Y-Axis
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  public String getTitle(int iNo)
  {
	String sTitle = "";

    switch (iNo)
    {
      case 1:
        sTitle = new String( sTitle1 );   // First line of the plot title
        break;

      case 2:
        sTitle = new String( sTitle2 );   // Second line of the plot title
        break;

      case 3:
        sTitle = new String( sTitle3 );   // Third line of the plot title
        break;
	}

    return (sTitle);
  }

  public double getXMax()  { return (dXmax); }  // Maximum X-Axis
  public double getXMin()  { return (dXmin); }  // Minimum X-Axis
  public double getXIncr() { return (dXincr); } // Increment along X-Axis

  public double getYMax()  { return (dYmax); }  // Maximum Y-Axis
  public double getYMin()  { return (dYmin); }  // Minimum Y-Axis
  public double getYIncr() { return (dYincr); } // Increment along Y-Axis

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	int    iField  = -1;
	String sText   = "";
	double dText   = 0.0;
	int    iTemp   = 0;
	double dTemp   = 0.0;
	int    iCycle  = 0;
    int    iCycleL = 0;
    int    iCycleU = 0;
    double diff    = 0.0;
    int    iError  = 0;
    String sError  = "";

    if ((e.getSource() == txtTitle1) ||
        (e.getSource() == txtTitle2) ||
        (e.getSource() == txtTitle3))
    {
	  if (e.getSource() == txtTitle1) { sTitle1 = new String(txtTitle1.getText()); }
	  if (e.getSource() == txtTitle2) { sTitle2 = new String(txtTitle2.getText()); }
	  if (e.getSource() == txtTitle3) { sTitle3 = new String(txtTitle3.getText()); }

      if (notifier != null)
        notifier.notifyObservers(new String("GET Title"));
	}

    if (e.getSource() == txtXmin)
    {
	  iField = XMIN;
	  sText  = txtXmin.getText();
	}

    if (e.getSource() == txtXmax)
    {
	  iField = XMAX;
	  sText  = txtXmax.getText();
	}

    if (e.getSource() == txtXincr)
    {
	  iField = XINC;
	  sText  = txtXincr.getText();
	}

    if (e.getSource() == txtYmin)
    {
	  iField = YMIN;
	  sText  = txtYmin.getText();
	}

    if (e.getSource() == txtYmax)
    {
	  iField = YMAX;
	  sText  = txtYmax.getText();
	}

    if (e.getSource() == txtYincr)
    {
	  iField = YINC;
	  sText  = txtYincr.getText();
	}

	if (iField > -1)
	{
	  if (cmn.cmnString.isNumeric(sText))
	  {
		dText = cmn.cmnString.stringToDouble(sText);
//System.out.println("X: "+dXmin+" "+dXmax+" "+dXincr+" Y: "+dYmin+" "+dYmax+" "+iYCycles);

		switch (iField)
		{
          case XMIN:
            if (dText <= dXmax)
              dXmin  = dText;
            else
            {
			  iError = -1;
			  sError = "Minimum Value must be less than the Maximum";
              txtXmin.setText( "" + (int) dXmin);  // Minimum X-Axis
			}
            break;
          case XMAX:
            if (dText >= dXmin)
              dXmax = dText;
            else
            {
			  iError = -1;
			  sError = "Minimum Value must be less than the Maximum";
              txtXmax.setText( "" + (int) dXmax);  // Maximum X-Axis
		    }
            break;
          case XINC:
            dTemp = (dXmax-dXmin)/dText;
            if (dTemp < 20.0)
              dXincr = dText;
            else
            {
			  iError = -1;
			  sError = "Increment value too small for range of data";
		      txtXincr.setText("" + (int) dXincr); // Increment X-Axis
		    }
            break;

          case YMIN:
            if (dText <= dYmax)
              dYmin = dText;
            else if (dText <= 0.0)
            {
			  iError = -1;
			  sError = "The Minimum value can not be 0.0 or less than 0.0.";
              txtYmin.setText( "" + (int) dYmin);  // Minimum Y-Axis
			}
            else
            {
			  iError = -1;
			  sError = "Minimum Value must be less than the Maximum";
              txtYmin.setText( "" + (int) dYmin);  // Minimum Y-Axis
			}
            break;
          case YMAX:
            if (dText >= dYmin)
              dYmax = dText;
            else
            {
			  iError = -1;
			  sError = "Minimum Value must be less than the Maximum";
              txtXmax.setText( "" + (int) dYmax);  // Maximum Y-Axis
		    }
            break;
          case YINC:
            dTemp = (dXmax-dXmin)/dText;
            if (dTemp < 20.0)
              dYincr = dText;
            else
            {
			  iError = -1;
			  sError = "Increment value too small for range of data";
		      txtXincr.setText("" + (int) dYincr); // Increment Y-Axis
		    }
            break;
		}

        if (iError == -1)
        {
          JOptionPane.showMessageDialog((Component) null,
      	                                sError,
		                                "ERROR",
		                                JOptionPane.ERROR_MESSAGE );
		}
		else
		{
		  switch (iField)
		  {
            case XMIN:
              dXmin = (double) ((int) dText);
              txtXmin.setText( "" + (int) dXmin); // Minimum X-Axis
/*
              iTemp  = (int) (1.0 + (dXmax-dXmin)/20.0);
              dXincr = (double) iTemp;

		      txtXincr.setText("" + (int) iTemp); // Increment X-Axis
*/
		      if (notifier != null)
		        notifier.notifyObservers(new String("GET X"));
              break;
            case XMAX:
              dXmax = (double) ((int) dText);
              txtXmax.setText( "" + (int) dXmax);  // Maximum X-Axis
/*
              iTemp  = (int) (1.0 + (dXmax-dXmin)/20.0);
              dXincr = (double) iTemp;

		      txtXincr.setText("" + (int) iTemp); // Increment X-Axis
*/
		      if (notifier != null)
		        notifier.notifyObservers(new String("GET X"));
              break;
            case XINC:
              dText = (double) ((int) dText);
              dTemp = (dXmax-dXmin)/dText;

              if (dTemp < 20.0)
                dXincr = dText;

		      txtXincr.setText("" + (int) dXincr); // Increment X-Axis

		      if (notifier != null)
		        notifier.notifyObservers(new String("GET X"));
              break;

            case YMIN:
              dYmin    = dText;
   	          dYmax    = cmn.cmnString.stringToDouble(txtYmax.getText());

		      if (notifier != null)
		        notifier.notifyObservers(new String("GET Y"));
              break;
            case YMAX:
              dYmax    = dText;
   	          dYmin    = cmn.cmnString.stringToDouble(txtYmin.getText());

		      if (notifier != null)
		        notifier.notifyObservers(new String("GET Y"));
              break;
            case YINC:
              dText = (double) ((int) dText);
              dTemp = (dYmax-dYmin)/dText;

              if (dTemp < 20.0)
                dYincr = dText;

		      txtYincr.setText("" + (int) dYincr); // Increment Y-Axis

		      if (notifier != null)
		        notifier.notifyObservers(new String("GET Y"));

              break;
		  }
	    }
	  }
	  else
	  {
		sError = new String( sText + " is an invalid numeric entry" );
		JOptionPane.showMessageDialog( (Component) null,
		                               sError,
		                               "ERROR",
		                               JOptionPane.ERROR_MESSAGE );

		switch (iField)
		{
	      case XMIN:
	        txtXmin.setText("" + (int) dXmin);
	        break;

	      case XMAX:
	        txtXmax.setText("" + (int) dXmax);
	        break;

	      case XINC:
	        txtXincr.setText("" + (int) dXincr);
	        break;

	      case YMIN:
	        txtYmin.setText("" + (int) dYmin);
	        break;

	      case YMAX:
	        txtYmax.setText("" + (int) dYmax);
	        break;

	      case YINC:
	        txtXincr.setText("" + (int) dYincr);
	        break;
	    }
	  }
	}
  }
}

/** CLASS pcPlotLimitsPanelFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class pcPlotLimitsPanelFocusAdapter extends java.awt.event.FocusAdapter
{
  pcPlotLimitsPanel adaptee;

  pcPlotLimitsPanelFocusAdapter(pcPlotLimitsPanel adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 09/12/2013
 *  @author  John Victorine
 */
