/*
 * @pcPlotXplotFrame.java Version 1.1 11/20/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import javax.swing.*;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Observer;
import java.util.Observable;

import gui.guiSearchDirectoryFrame; // GUI Search Directory Frame

import plot.plotXYStruct;
import plot.plotSymbolStruct;

import pc.plot.pcPlotXplot;
import pc.gui.pcPlotLimitsPanel;    // Limits Calculator Panel

/** Class pcPlotXplotFrame
 *  <p> This class will display the Cross Plot Frame
 *
 *  @version 1.1 11/20/2015
 *  @author  John R. Victorine
 */

public class pcPlotXplotFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private Observable   pNotifier = null; // Observalbles Object
  private int          iPlot     = -1;
  private plotXYStruct stPlot    = null;

  // Global Variables

  private Observable   notifier  = null; // Observalbles Object
  private pcPlotXplot  panel     = null;

  // -- Search for Directory Path & filename

  private guiSearchDirectoryFrame pSearch = null; // Search for Directory

  private String sPath     = "";
  private String sName     = "";

  // -- Change Limits Frame

  private pcPlotLimitsPanel pLimits = null; // Production Limits Panel
  private JFrame            fLimits = null;

  // -- Plot Limits

  private String sTitle1 = "";   // First line of the plot title
  private String sTitle2 = "";   // Second line of the plot title
  private String sTitle3 = "";   // Third line of the plot title

  private String sXAxis  = "";
  private double dXmax   = 0.0;  // Maximum X-Axis
  private double dXmin   = 0.0;  // Minimum X-Axis
  private double dXincr  = 0.0;  // Increment along X-Axis

  private String sYAxis  = "";
  private double dYmax   = 0.0;  // Maximum Y-Axis
  private double dYmin   = 0.0;  // Minimum Y-Axis
  private double dYincr  = 0.0;  // Increment along Y-Axis

  // Global Widgets

  private JButton btnClose     = null;
  private JButton btnPDF       = null;

  private JButton btnLimits    = new JButton();

  /** Constructor pcPlotXplotFrame()
   *  <p> This is the constructor for this class
   * @param pNotifier = Oberservable
   * @param iPlot    = Plot Type
   * @param stPlot   = XY Plot Data Structure
   */

  public pcPlotXplotFrame( Observable pNotifier, int iPlot, plotXYStruct stPlot )
  {
    try
    {
	  this.pNotifier = pNotifier;
	  this.iPlot     = iPlot;
      this.stPlot    = stPlot;

      getPlotData();

      jbInit();

      addWindowListener( new pcPlotXplotFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel  pnlTop   = new JPanel();
    JPanel  pnlBTN   = new JPanel();

    JButton btnI     = new JButton();
    JButton btnII    = new JButton();

    JScrollPane jScrollPane = new JScrollPane();

    // Create an Observable

    notifier = new pcPlotXplotFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle("Principal Components Scores");

    // Top panel

    pnlTop.setLayout(new BorderLayout());

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(29, 29));
    btnClose.setToolTipText("Close Dialog");
    btnClose.addActionListener(this);

    // -- 1st Separator Button

    btnI.setPreferredSize(new Dimension(4, 35));
    btnI.setText("");

    btnPDF = new JButton(createImageIcon(cmn.cmnStruct.PDF));
    btnPDF.setPreferredSize(new Dimension(29, 29));
    btnPDF.setToolTipText("Save this Panel as a PNG Image");
    btnPDF.addActionListener(this);

    // -- 2nd Separator Button

    btnII.setPreferredSize(new Dimension(4, 35));
    btnII.setText("");

    // Change Limits Button

    btnLimits.setFont(new java.awt.Font("Dialog", 1, 11));
    btnLimits.setText("Modify Limits");
    btnLimits.addActionListener(this);

    // Center panel

    panel = new pcPlotXplot(iPlot, stPlot);

    // Attach Widgets to frame

    this.getContentPane().add(pnlTop,  BorderLayout.NORTH);
    pnlTop.add(pnlBTN,                 BorderLayout.WEST);
    pnlBTN.add(btnClose,               null);
    pnlBTN.add(btnI,                   null);
    pnlBTN.add(btnPDF,                 null);
    pnlBTN.add(btnII,                  null);
    pnlBTN.add(btnLimits,              null);

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
    jScrollPane.getViewport().add(panel, null);

    // Add a Window Listener to close dialog

//    this.addWindowListener(new WindowAdapter()
//    {
//      public void windowClosing(WindowEvent e) { close(); }
//    });

    // Set Dimension of Frame and Display it.

    this.setSize(new Dimension(900, 700));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    this.setLocation( (d.width - this.getSize().width)/2,
                      (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /** Method refresh()
   * <p> This method will refresh the plot if it exists
   */

  public void refresh() { panel.repaint(); }


  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD pcPlotXplotFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class pcPlotXplotFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  if (pNotifier != null)
        pNotifier.notifyObservers( new String("Close Cross Plot Control") );
	  else
	    close();
    }
  }

  public void close()
  {
    pNotifier = null;
    notifier  = null;

    sPath     = null;
    sName     = null;

    sTitle1   = null;
    sTitle2   = null;
    sTitle3   = null;

    sXAxis    = null;
    sYAxis    = null;

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

    if (pLimits != null)
      pLimits.close();
    pLimits   = null;

	if (fLimits != null)
	  fLimits.dispose();
	fLimits = null;

    if (stPlot != null)
      stPlot.delete();
    stPlot = null;

    if (panel != null)
      panel.close();
    panel     = null;

    btnClose     = null;
    btnPDF       = null;

    dispose();
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getPlotData()
   *  <p> This method will retrieve the Plot Data
   */

  private void getPlotData()
  {
    if (stPlot != null)
    {
      sTitle1  = stPlot.sTitle1;     // First line of the plot title
      sTitle2  = stPlot.sTitle2;     // Second line of the plot title
      sTitle3  = stPlot.sTitle3;     // Third line of the plot title

      sXAxis   = new String(stPlot.sXAxis);
      dXmax    = stPlot.dXMaximum;   // Maximum X-Axis
      dXmin    = stPlot.dXMinimum;   // Minimum X-Axis
      dXincr   = stPlot.dXIncrement; // Increment along X-Axis

      sYAxis   = new String(stPlot.sYAxis);
      dYmax    = stPlot.dYMaximum;   // Maximum Y-Axis
      dYmin    = stPlot.dYMinimum;   // Minimum Y-Axis
      dYincr   = stPlot.dYIncrement; // Increment along Y-Axis
	}
  }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   */

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    panel.setLegend(iLegend, stLegend);
  }

  /** Method setData()
   *  <p> This is the Constructor for this class.
   *  @param stPlot - Plot Data Structure.
   */

  public void setData(plotXYStruct stPlot)
  {
    if (panel != null) { panel.setData(stPlot); }
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag) { panel.setFlag(bFlag); }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText) { panel.setTitle1(sText); }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText) { panel.setTitle2(sText); }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
	sTitle3 = new String(sText);
	panel.setTitle3(sText);
  }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setXAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setYAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setVectors()
   *  <p> Set the Vector data
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setVectors(int iVector, double dVector[][], String sVector[])
  {
	panel.setVectors(iVector, dVector, sVector);
  }

  /* ===================================================================== *
   * ---------------------- LIMITS FRAME METHODS ------------------------- *
   * ===================================================================== */

  /** Method displayLimitsFrame()
   * <p> This method will build the Limits Frame so the user can change the
   *     plot limits & titles
   */

  private void displayLimitsFrame()
  {
	if (pLimits != null)
	  pLimits.close();
	pLimits = null;

	if (fLimits != null)
	  fLimits.dispose();
	fLimits = null;

	pLimits = new pcPlotLimitsPanel( notifier,
	                                 sTitle1,  sTitle2,  sTitle3,
                                     dXmax,  dXmin,  dXincr,
                                     dYmax,  dYmin,  dYincr );
	limitsFrame();
  }

  /** Method limitsFrame()
   *  <p> This method will create a Limits Frame to hold the limits Panel
   */

  private void limitsFrame()
  {
	int iX = 0;
	int iY = 0;

	if (pLimits != null)
	{
      fLimits = new JFrame();

      fLimits.setTitle("Limits");
      fLimits.getContentPane().add(pLimits, null);

      // Set the Size and Display the Frame

      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

      fLimits.setSize(425, 275);
      iX = 10;
      iY = d.height - fLimits.getSize().height - 75;
      fLimits.setLocation(iX, 10); //iY);
//      fLimits.setLocation((d.width  - fLimits.getSize().width)/2,
//                           (d.height - fLimits.getSize().height)/2);

      fLimits.setResizable(false);
      fLimits.setVisible(true);

      fLimits.addWindowListener( new limitsFrame_WindowListener() );
	}
  }

  /** METHOD limitsFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class limitsFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  pLimits.close();
      fLimits.dispose();
    }
  }

  /* ======================================================================= *
   * ----------------------------- Save Plots ------------------------------ *
   * ======================================================================= */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   */

  private void searchFrame()
  {
	if (sPath.length() == 0)
      sPath = pc.io.pcIO.getHomeDir();

    sName   = new String( "plot" );
    pSearch = new guiSearchDirectoryFrame(notifier, sPath, "PCplot", ".png");
  }

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Panel
   */

  private void createPDF()
  {
    pc.io.pcIO_Files.createImage( sPath, sName, getImage() );
    pc.io.pcIO_Files.createHTML( sPath, sName );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnPDF)    { searchFrame(); }
    if (event.getSource() == btnLimits) { displayLimitsFrame(); }

    if (event.getSource() == btnClose)
    {
	  if (pNotifier != null)
        pNotifier.notifyObservers( new String("Close Plot Control") );
	  else
	    close();
	}
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("GET Title"))
    {
	  if (pLimits != null)
	  {
		sTitle1 = new String( pLimits.getTitle(1) );
		sTitle2 = new String( pLimits.getTitle(2) );
		sTitle3 = new String( pLimits.getTitle(3) );

//	    setTitles( sTitle1, sTitle2, sTitle3 );
	    setTitle1( sTitle1 );
	    setTitle2( sTitle2 );
	    setTitle3( sTitle3 );
	  }
	}

    if (sArg.equals("GET X"))
    {
	  if (pLimits != null)
	  {
		dXmax  = pLimits.getXMax();
		dXmin  = pLimits.getXMin();
		dXincr = pLimits.getXIncr();

		setXAxis( sXAxis, 0, dXmax, dXmin, dXincr, 1);
	  }
	}

    if (sArg.equals("GET Y"))
    {
	  if (pLimits != null)
	  {
		dYmax    = pLimits.getYMax();
		dYmin    = pLimits.getYMin();
		dYincr   = pLimits.getYIncr();

        setYAxis( sYAxis, 0, dYmax, dYmin, dYincr, 1 );
	  }
	}

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS pcPlotXplotFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class pcPlotXplotFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 11/20/2015
 *  @author  John Victorine
 */
