/*
 * @pcTable.java Version 1.1 10/28/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;

import gui.guiPCTable;

/** Class pcTable
 *  <p> This Class will display the Zonation Data.
 *
 *  @version 1.1 10/28/2015
 *  @author  John R. Victorine
 */

public class pcTable
{
  private int          iRows     = 0;
  private int          iColumns  = 0;
  private String       sColumn[] = null;
  private int          iLength[] = null;

  private Object       oData[][] = null;
  private guiPCTable   pTable    = null;

  private String       sLabels[] = null;
  private double       data[][]  = null;
  private int          iChange   = 0;

  public static final Color cPOS_1  = new Color(255, 255, 204);
  public static final Color cPOS_2  = new Color(255, 255, 102);
  public static final Color cPOS_3  = new Color(255, 255,  51);
  public static final Color cPOS_4  = new Color(255, 255,   0);
  public static final Color cPOS_5  = new Color(255, 204, 102);
  public static final Color cPOS_6  = new Color(255, 204,  51);
  public static final Color cPOS_7  = new Color(255, 204,   0);
  public static final Color cPOS_8  = new Color(255, 153,  51);
  public static final Color cPOS_9  = new Color(255, 153,   0);
  public static final Color cPOS_10 = new Color(204, 102,   0);
  public static final Color cPOS_11 = new Color(204, 102,  51);
  public static final Color cPOS_12 = new Color(153,  51,   0);
  public static final Color cPOS_13 = new Color(153,  51,  51);
  public static final Color cPOS_14 = new Color(102,  51,   0);
  public static final Color cPOS_15 = new Color(102,  51,  51);
  public static final Color cPOS_16 = new Color( 51,   0,   0);

  public static final int   iPOS    = 16;

  public static final Color cNEG_1  = new Color(204, 255, 204);
  public static final Color cNEG_2  = new Color(153, 255, 153);
  public static final Color cNEG_3  = new Color(102, 255, 102);
  public static final Color cNEG_4  = new Color( 51, 255,  51);
  public static final Color cNEG_5  = new Color(  0, 255,   0);
  public static final Color cNEG_6  = new Color(  0, 255, 153);
  public static final Color cNEG_7  = new Color( 51, 204, 153);
  public static final Color cNEG_8  = new Color(  0, 204, 255);
  public static final Color cNEG_9  = new Color( 51, 153, 255);
  public static final Color cNEG_10 = new Color(  0, 153, 255);
  public static final Color cNEG_11 = new Color(  0, 102, 204);
  public static final Color cNEG_12 = new Color( 51, 102, 204);
  public static final Color cNEG_13 = new Color(102, 102, 204);
  public static final Color cNEG_14 = new Color(102,  51, 204);
  public static final Color cNEG_15 = new Color(102,  51, 102);
  public static final Color cNEG_16 = new Color( 51,   0,  51);

  public static final int   iNEG    = 16;

  public static final Color cNULL   = new Color(190, 190, 190);

  public static final int    iColors     = 11;
  public static final double dFactor[][] =
  {
	{ -0.4, -0.29 }, { -0.55, -0.4 },  { -0.65, -0.55 }, { -0.8, -0.65 }, { -0.99, -0.8 },
	{  0.3,  0.4 },  {  0.4,   0.55 }, {  0.55,  0.65 }, {  0.65, 0.8 },  {  0.8,  0.999 },
	{  .999,  2.0 }
  };

  public static final Color COLORS[] =
  {
	cNEG_8,  cNEG_9,  cNEG_10,  cNEG_11,  cNEG_12,
	cPOS_5,  cPOS_6,  cPOS_7,  cPOS_8,  cPOS_10,
	cNULL
  };
/*
  public static final Color COLORS[] =
  {
	cNEG_1,  cNEG_4,  cNEG_7,  cNEG_8,  cNEG_10,
	cPOS_1,  cPOS_4,  cPOS_7,  cPOS_9,  cPOS_11,
	cNULL
  };
*/
  /** Constructor pcTable()
   *  <p> This is the Constructor for this class.
   *  @param sLabels  = array of labels used in Column 1
   *  @param data     = double array of the data to be displayed
   *  @param iColumns = total number of columns in table
   *  @param sColumn  = array of column headers
   *  @param iLength  = array of column widths
   */

  public pcTable( String sLabels[], double data[][],
                  int iColumns, String sColumn[], int iLength[] )
  {
	this.sLabels  = sLabels;
	this.data     = data;
    this.iColumns = iColumns;
    this.sColumn  = sColumn;
    this.iLength  = iLength;

    populateList();

    if (iRows > 0)
      pTable = new guiPCTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiPCTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);

    for (int i=1; i<iColumns; i++)
    {
      setCellColors( true, i, iColors, dFactor, COLORS );
	}
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
	sLabels  = null;
	data     = null;

    sColumn  = null;
    iLength  = null;
    oData    = null;

    if (pTable != null)
      pTable.close();
    pTable   = null;

  }

  /** Method resetColumnWidths()
   *  <p> This method will reset the column widths to a specific size.
   */

  private void resetColumnWidths()
  {
    for (int i=0; i<iColumns; i++)
    {
      pTable.resizeColumn(i, iLength[i]);
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method populateList()
   *  <p> This method will populate the Zonation data.
   */

  private void populateList()
  {
    int        i,j;
    int        iCount  = 0;
    double     dval    = 0.0;

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if ((sLabels != null) && (data != null))
    {
      iChange = 1;
      iCount  = 0;
      oData   = new Object[sLabels.length][iColumns];

      for (i=0; i<sLabels.length; i++)
      {
		oData[iCount][0]  = new String(sLabels[i]);
		for (j=0; j<sLabels.length; j++)
		{
		  dval = (double) ((int) (1000.0 * data[i][j]) / 1000.0);

		  oData[iCount][j+1] = new Double(dval);
		}
        iCount++;
	  }
    }

    iRows = iCount;
  }

  /** Method setData()
   *  <p> This method will reload the data.
   *  @param sLabels  = array of labels used in Column 1
   *  @param data     = double array of the data to be displayed
   */

  public void setData( String sLabels[], double data[][] )
  {
	this.sLabels  = sLabels;
	this.data     = data;

    populateList();

    pTable.reload(iRows, oData);

    if (iChange == 0)
      iRows = 0;
  }

  /** Method setCellColors()
   *  <p> This method will set the background of the column based on the
   *      value of the Pay.
   *  @param bv = true color is on; false color is off
   */

  public void setCellColors(boolean bv, int iCol, int iClrs, double dVal[][], Color bkgnd[])
  {
	if (bv)
      pTable.colorColumns( iCol, iClrs, dVal, bkgnd );
    else
      pTable.colorColumns( iCol, 0.0, Color.white );
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTotalRows()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 10/28/2015
 *  @author  John Victorine
 */
