/*
 * @pcDataUtility.java Version 1.1 10/15/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc;

import pc.pcDataStruct;

/** Class pcDataUtility
 *  <p> This Class will provide basic utility for the pc data
 *
 *  @version 1.1 10/15/2015
 *  @author  John R. Victorine
 */

public class pcDataUtility
{
  /** Method copy()
   * <p> This method will copy data from one structure to another
   * @param  stOld = The original Data Structure
   * @return stNew = The new Data Structure
   */

  public static pcDataStruct copy(pcDataStruct stOld)
  {
    pcDataStruct stNew = null;

    if (stOld != null)
    {
	  stNew          = new pcDataStruct();

      stNew.iRows    = stOld.iRows;     // Total number of Wells in this dataset
      stNew.iColumns = stOld.iColumns;  // Total number of Anions & Cations Columns in this dataset

      stNew.iBrine   = new int[stOld.iColumns][2];

      stNew.sData    = new String[stOld.iRows][3];
      stNew.data     = new double[stOld.iRows][stOld.iColumns];

      stNew.dAdjust  = new double[stOld.iRows][stOld.iColumns];

      stNew.dataM    = new double[stOld.iColumns];
      stNew.dSD      = new double[stOld.iColumns];

      stNew.dAnions  = new double[stOld.iRows];
      stNew.dCations = new double[stOld.iRows];

      for (int i=0; i<stNew.iRows; i++)
      {
        for (int j=0; j<3; j++)
        {
          stNew.sData[i][j]  = new String( stOld.sData[i][j] );
		}

        stNew.dAnions[i]  = stOld.dAnions[i];
        stNew.dCations[i] = stOld.dCations[i];

        for (int j=0; j<stNew.iColumns; j++)
        {
          stNew.data[i][j]    = stOld.data[i][j];
          stNew.dAdjust[i][j] = stOld.dAdjust[i][j];
		}
	  }

      for (int i=0; i<stNew.iColumns; i++)
      {
        stNew.dataM[i] = stOld.dataM[i];
        stNew.dSD[i]   = stOld.dSD[i];

        for (int j=0; j<2; j++)
        {
		  stNew.iBrine[i][j] = stOld.iBrine[i][j];
		}
	  }
    }

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will transfer data from one structure to another
   *     and delete the old structure
   * @param  stOld = The original Data Structure
   * @return stNew = The new Data Structure
   */

  public static pcDataStruct transfer(pcDataStruct stOld)
  {
    pcDataStruct stNew = null;

    stNew = copy( stOld );

    if (stOld != null)
      stOld.delete();
    stOld = null;

    return (stNew);
  }
}

/*
 *  @version 1.1 10/15/2015
 *  @author  John Victorine
 */
