/*
 * @pcPlotFilterUtility.java Version 1.1 11/05/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc;

import pc.pcPlotFilterStruct;
import pc.pcPlotFilterListStruct;

/** Class pcPlotFilterUtility
 *  <p> This class will hold the Constants for the data plot classes
 *
 *  @version 1.1 11/05/2015
 *  @author  John R. Victorine
 */

public class pcPlotFilterUtility
{
  // -------------------------------------------------------
  // Filter possible symobls & colors
  // -------------------------------------------------------

  // Symbols Types

  public static final int OVAL      = plot.plotSymbolStruct.OVAL;
  public static final int RECTANGLE = plot.plotSymbolStruct.RECTANGLE;
  public static final int TRIANGLE  = plot.plotSymbolStruct.TRIANGLE;
  public static final int DIAMOND   = plot.plotSymbolStruct.DIAMOND;
  public static final int DELTA     = plot.plotSymbolStruct.DELTA;
  public static final int MAXIMUM   = 5;

  public static final int    _TOTAL = 13;
  public static final double _RATIO[][] = {
    { 0.0,  0.89, 255, 255, 102 },   // < 0.89          2
    { 0.89, 0.91, 255, 255,   0 },   // 0.89 - 0.91     4
    { 0.91, 0.93, 255, 204,  51 },   // 0.91 - 0.93     6
    { 0.93, 0.95, 255, 153,  51 },   // 0.93 - 0.95     8
    { 0.95, 0.97, 204, 102,   0 },   // 0.95 - 0.97    10
    { 0.97, 0.99, 153,  51,   0 },   // 0.97 - 0.99    12
    { 0.99, 1.01,   0,   0,   0 },   // 0.99 - 1.01
    { 1.01, 1.03,  51, 102, 204 },   // 1.01 - 1.03    12
    { 1.03, 1.05,   0, 153, 255 },   // 1.03 - 1.05    10
    { 1.05, 1.07,   0, 204, 255 },   // 1.05 - 1.07     8
    { 1.07, 1.09,  51, 204, 153 },   // 1.07 - 1.09     7
    { 1.09, 1.11,   0, 255, 153 },   // 1.09 - 1.11     6
    { 1.11, 5.0,   51, 255,  51 },   // > 1.11          4
  };

  public static final String RATIO[] = {
    "< 0.89",
    "0.89 - 0.91",
    "0.91 - 0.93",
    "0.93 - 0.95",
    "0.95 - 0.97",
    "0.97 - 0.99",
    "0.99 - 1.01",
    "1.01 - 1.03",
    "1.03 - 1.05",
    "1.05 - 1.07",
    "1.07 - 1.09",
    "1.09 - 1.11",
    "> 1.11"
  };

/*
  public static final int _MMHOS_M  = 16;
  public static final int MMHOS_M[][] =
  { {255, 255, 204},
    {255, 255, 102},     2
    {255, 255,  51},
    {255, 255,   0},     4
    {255, 204, 102},
    {255, 204,  51},     6
    {255, 204,   0},
    {255, 153,  51},     8
    {255, 153,   0},
    {204, 102,   0},    10
    {204, 102,  51},
    {153,  51,   0},    12
    {153,  51,  51},
    {102,  51,   0},    14
    {102,  51,  51},
    { 51,   0,   0} };  16

  public static final int _PU  = 16;
  public static final int PU[][] =
  { {204, 255, 204},
    {153, 255, 153},     2
    {102, 255, 102},
    { 51, 255,  51},     4
    {  0, 255,   0},
    {  0, 255, 153},     6
    { 51, 204, 153},
    {  0, 204, 255},     8
    { 51, 153, 255},
    {  0, 153, 255},    10
    {  0, 102, 204},
    { 51, 102, 204},    12
    {102, 102, 204},
    {102,  51, 204},    14
    {102,  51, 102},
    { 51,   0,  51}};   16
*/

  /** Method getFilter()
   *  <p> This method will retrieve a default filter list
   */

  public static pcPlotFilterListStruct getFilter()
  {
	int    MAX      = 0;
	double DATA[][] = null;
	String NAME[]   = null;

	pcPlotFilterStruct     stFilter = null;
	pcPlotFilterListStruct st = new pcPlotFilterListStruct();

    MAX  = _TOTAL;
    DATA = _RATIO;
    NAME = RATIO;

    if ((DATA != null) && (NAME != null))
    {
	  for (int i=0; i<MAX; i++)
	  {
		stFilter           = new pcPlotFilterStruct();
		stFilter.sKEY      = new String( cmn.cmnString.UniqueName() + "_" + i );
		stFilter.iBy       = pc.pcPlotFilterStruct._BY_DATA;
        stFilter.dataStart = DATA[i][0]; // Starting Data
        stFilter.dataEnd   = DATA[i][1];   // Ending Data

        // Symbol and Color

        stFilter.iEmpty  = 1;      // Open or Filled Symbol
        stFilter.iSymbol = DIAMOND;
        stFilter.iRed    = (int) DATA[i][2]; // Red Color in RGB Setting
        stFilter.iGreen  = (int) DATA[i][3]; // Green Color in RGB Setting
        stFilter.iBlue   = (int) DATA[i][4]; // Blue Color in RGB Setting
        stFilter.symbol  = new String(NAME[i]);  // Label for the selected symbol.

	    st = add( stFilter, st );
	  }
	}

	return (st);
  }

  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static pcPlotFilterListStruct copyList( pcPlotFilterListStruct stOld )
  {
    pcPlotFilterListStruct stNew = null;

    if (stOld != null)
    {
      stNew            = new pcPlotFilterListStruct();
      stNew.iCount     = stOld.iCount;
      stNew.stItem     = new pcPlotFilterStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy( stOld.stItem[i] );
      }
    }

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy the data structure
   * @param  stOld = The old data structure
   * @return stNew = The new data structure
   */

  public static pcPlotFilterStruct copy( pcPlotFilterStruct stOld )
  {
    pcPlotFilterStruct stNew = new pcPlotFilterStruct();

    if (stOld != null)
    {
      stNew.iBy        = stOld.iBy;                 // Filter By Indicator

      stNew.sKEY       = new String(stOld.sKEY);    // Primary Key

      stNew.depthStart = stOld.depthStart;          // Starting Depth
      stNew.depthEnd   = stOld.depthEnd;            // Ending Depth

      stNew.dataStart  = stOld.dataStart;           // Starting Data Value
      stNew.dataEnd    = stOld.dataEnd;             // Ending Data Value

      // Symbol and Color

      stNew.iEmpty     = stOld.iEmpty;              // Open or Filled Symbol
      stNew.iSymbol    = stOld.iSymbol;             // Symbol Identifier.
      stNew.iRed       = stOld.iRed;                // Red Color in RGB Setting
      stNew.iGreen     = stOld.iGreen;              // Green Color in RGB Setting
      stNew.iBlue      = stOld.iBlue;               // Blue Color in RGB Setting
      stNew.symbol     = new String(stOld.symbol);  // Label for the selected symbol.
    }

    return (stNew);
  }

  /** Method add()
   * <p> This method will add the data to the list
   * @param  stBlock = the New Data Structure
   * @param  st      = the List Structure
   * @return st      = the Modified List structure
   */

  public static pcPlotFilterListStruct add(
	  pcPlotFilterStruct     stBlock,
	  pcPlotFilterListStruct st )
  {
	int i = 0;
	int iCount = 0;
    pcPlotFilterListStruct stTemp = new pcPlotFilterListStruct();

    stTemp.iCount = 1;
    stTemp.stItem = new pcPlotFilterStruct[1];

    if (st != null)
    {
      stTemp.stItem = new pcPlotFilterStruct[st.iCount+1];

      for (i=0; i<st.iCount; i++)
      {
        stTemp.stItem[iCount] = copy( st.stItem[i] );
        iCount++;
      }

      st.delete();
      st = null;
    }

    stTemp.stItem[iCount] = copy( stBlock );
    iCount++;

    st        = new pcPlotFilterListStruct();
    st.stItem = new pcPlotFilterStruct[iCount];
    st.iCount = iCount;

    for (i=0; i<iCount; i++)
    {
      st.stItem[i] = copy( stTemp.stItem[i] );
    }

    stTemp.delete();
    stTemp = null;

    return (st);
  }

  /** Method modify()
   * <p> This method will modify data within the list
   * @param  stBlock = the New Data Structure
   * @param  st      = the List Structure
   * @return st      = the Modified List structure
   */

  public static pcPlotFilterListStruct modify(
	  pcPlotFilterStruct     stBlock,
	  pcPlotFilterListStruct st )
  {
	int i = 0;
	int iCount = 0;
    pcPlotFilterListStruct stTemp = new pcPlotFilterListStruct();

    if (st != null)
    {
      stTemp.stItem = new pcPlotFilterStruct[st.iCount];

      for (i=0; i<st.iCount; i++)
      {
		if (!st.stItem[i].sKEY.equals(stBlock.sKEY))
          stTemp.stItem[iCount] = copy( st.stItem[i] );
        else
          stTemp.stItem[iCount] = copy( stBlock );

        iCount++;
      }

      st.delete();
      st = null;

      st        = new pcPlotFilterListStruct();
      st.stItem = new pcPlotFilterStruct[iCount];
      st.iCount = iCount;

      for (i=0; i<iCount; i++)
      {
        st.stItem[i] = copy( stTemp.stItem[i] );
      }

      stTemp.delete();
      stTemp = null;
    }

    return (st);
  }

  /** Method remove()
   * <p> This method will remove the data from the list
   *     This method assumes that the data structure that
   *     is to be removed exists in the list.
   * @param  stBlock = the New Data Structure
   * @param  st      = the List Structure
   * @return st      = the Modified List structure
   */

  public static pcPlotFilterListStruct remove(
	  String                   sKEY,
	  pcPlotFilterListStruct st )
  {
	int i = 0;
	int iCount = 0;
    pcPlotFilterListStruct stTemp = null;

    if (st != null)
    {
	  if (st.iCount > 1)
	  {
	    stTemp        = new pcPlotFilterListStruct();
        stTemp.stItem = new pcPlotFilterStruct[st.iCount-1];

        for (i=0; i<st.iCount; i++)
        {
  		  if (!sKEY.equals(st.stItem[i].sKEY))
  		  {
            stTemp.stItem[iCount] = copy( st.stItem[i] );
            iCount++;
	      }
        }

        st.delete();
        st = null;

        st        = new pcPlotFilterListStruct();
        st.stItem = new pcPlotFilterStruct[iCount];
        st.iCount = iCount;

        for (i=0; i<iCount; i++)
        {
          st.stItem[i] = copy( stTemp.stItem[i] );
        }

        stTemp.delete();
        stTemp = null;
      }
      else
      {
	    st.delete();
	    st = null;
	  }
    }

    return (st);
  }

  /** Method print()
   *  <p> This method print the contents of List Structure
   *  @param  st      = the List Structure
   */

  public static void print(pcPlotFilterListStruct st)
  {
    if (st != null)
    {
	  for (int i=0; i<st.iCount; i++)
	  {
		System.out.println(
			st.stItem[i].iBy +
		    "KEY=" + st.stItem[i].sKEY          + " " +
		    "Depth: " + st.stItem[i].depthStart + "-" +
		                st.stItem[i].depthEnd   + " " +
		    "Data: "  + st.stItem[i].dataStart  + "-" +
		                st.stItem[i].dataEnd    + " " +
		    "Symbol=" +
		      plot.plotSymbolStruct.SHAPE_EMPTY[st.stItem[i].iSymbol] + " " +
		    "RGB="    + st.stItem[i].iRed   + " " +
		                st.stItem[i].iGreen + " " +
		                st.stItem[i].iBlue  + " " +
		    "Legend=" + st.stItem[i].symbol );
      }
    }
  }
}

/*
 *  @version 1.1 11/05/2015
 *  @author  John Victorine
 */
