/*
 * @pcPlotUtility.java Version 1.1 11/20/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc;

import plot.plotXYStruct;         // Plot XY Data Structure
import plot.plotSymbolStruct;     // Symbol Data Structure

import pc.pcDataStruct;           // Principal Components Data Structure
import pc.pcPlotFilterListStruct; // Plot Filter List Structure

/** Class pcPlotUtility.java
 *  <p> This Class will provide basic utility for the plot data structure
 *
 *  @version 1.1 11/20/2015
 *  @author  John R. Victorine
 */

public class pcPlotUtility
{
  public static final int _RED             = pc.pcPlotConstants._RED;

  public static final int _BROWN           = pc.pcPlotConstants._BROWN;
  public static final int _SIENNA          = pc.pcPlotConstants._SIENNA;
  public static final int _BURLYWOOD       = pc.pcPlotConstants._BURLYWOOD;
  public static final int _TAN             = pc.pcPlotConstants._TAN;

  public static final int _SALMON          = pc.pcPlotConstants._SALMON;
  public static final int _ORANGE_RED_1    = pc.pcPlotConstants._ORANGE_RED_1;
  public static final int _DARK_ORANGE     = pc.pcPlotConstants._DARK_ORANGE;
  public static final int _ORANGE          = pc.pcPlotConstants._ORANGE;

  public static final int _MIDDLE_YELLOW   = pc.pcPlotConstants._MIDDLE_YELLOW;
  public static final int _DARKER_YELLOW   = pc.pcPlotConstants._DARKER_YELLOW;
  public static final int _YELLOW          = pc.pcPlotConstants._YELLOW;

  public static final int _GREEN_YELLOW    = pc.pcPlotConstants._GREEN_YELLOW;
  public static final int _YELLOW_GREEN    = pc.pcPlotConstants._YELLOW_GREEN;
  public static final int _PALE_GREEN      = pc.pcPlotConstants._PALE_GREEN;
  public static final int _LIGHT_GREEN     = pc.pcPlotConstants._LIGHT_GREEN;
  public static final int _GREEN           = pc.pcPlotConstants._GREEN;
  public static final int _LAWN_GREEN      = pc.pcPlotConstants._LAWN_GREEN;
  public static final int _FOREST_GREEN    = pc.pcPlotConstants._FOREST_GREEN;
  public static final int _OLIVE_DRAB      = pc.pcPlotConstants._OLIVE_DRAB;

  public static final int _TURQUOISE       = pc.pcPlotConstants._TURQUOISE;
  public static final int _AQUAMARINE_1    = pc.pcPlotConstants._AQUAMARINE_1;
  public static final int _AQUAMARINE_3    = pc.pcPlotConstants._AQUAMARINE_3;
  public static final int _AQUAMARINE_4    = pc.pcPlotConstants._AQUAMARINE_4;
  public static final int _MIDDLE_CYAN     = pc.pcPlotConstants._MIDDLE_CYAN;
  public static final int _CYAN            = pc.pcPlotConstants._CYAN;

  public static final int _SKY_BLUE        = pc.pcPlotConstants._SKY_BLUE;
  public static final int _DEEP_SKY_BLUE_1 = pc.pcPlotConstants._DEEP_SKY_BLUE_1;
  public static final int _DEEP_SKY_BLUE_2 = pc.pcPlotConstants._DEEP_SKY_BLUE_2;
  public static final int _MIDDLE_BLUE     = pc.pcPlotConstants._MIDDLE_BLUE;
  public static final int _ROYAL_BLUE      = pc.pcPlotConstants._ROYAL_BLUE;
  public static final int _BLUE            = pc.pcPlotConstants._BLUE;

  public static final int _LIGHT_PINK      = pc.pcPlotConstants._LIGHT_PINK;
  public static final int _HOT_PINK        = pc.pcPlotConstants._HOT_PINK;
  public static final int _DEEP_PINK       = pc.pcPlotConstants._DEEP_PINK;

  public static final int _LIGHT_MAGENTA   = pc.pcPlotConstants._LIGHT_MAGENTA;
  public static final int _MAGENTA         = pc.pcPlotConstants._MAGENTA;
  public static final int _VIOLET          = pc.pcPlotConstants._VIOLET;
  public static final int _DARK_VIOLET     = pc.pcPlotConstants._DARK_VIOLET;

  public static final int _GREY_90         = pc.pcPlotConstants._GREY_90;
  public static final int _GREY            = pc.pcPlotConstants._GREY;
  public static final int _GREY_50         = pc.pcPlotConstants._GREY_50;
  public static final int _BLACK           = pc.pcPlotConstants._BLACK;

  public static final int _COLOR_TOTAL     = pc.pcPlotConstants.TOTAL;

  public static final int _CLR_TOTAL = 21;
  public static final int _CLR[] = {
	_RED,   _ORANGE,      _TAN, _FOREST_GREEN, _MIDDLE_CYAN, _BLUE, _DARK_VIOLET,
	_BROWN, _DARK_ORANGE, _ORANGE_RED_1, _LAWN_GREEN, _TURQUOISE, _MIDDLE_BLUE, _MAGENTA,
	_SIENNA, _SALMON, _YELLOW_GREEN, _OLIVE_DRAB, _AQUAMARINE_3, _SKY_BLUE, _VIOLET
  };

  /* ======================================================================= *
   * ---------------- SET ANIONS/CATIONS RATIO DATA METHODS ---------------- *
   * ======================================================================= */

  /** Method setPlotStruct()
   * <p> This method will set the Plot Structure
   *  @param  st       = principle component data structure
   *  @param  sTitle1  = 1st Plot Title String
   *  @param  sTitle2  = 2nd Plot Title String
   *  @param  sTitle3  = 3rd Plot Title String
   *  @param  sXAxis   = X-Axis Title
   *  @param  dXMin    = Minimum X-Coordinate
   *  @param  dXMax    = Maximum X-Coordinate
   *  @param  dXIncr   = X-Axis Increment
   *  @param  sYAxis   = Y-Axis Title
   *  @param  dYMin    = Minimum Y-Coordinate
   *  @param  dYMax    = Maximum Y-Coordinate
   *  @param  dYIncr   = Y-Axis Increment
   *  @return stPlotXY = XY Plot Data Structure
   */

  public static plotXYStruct setPlotStruct( pcDataStruct st,
                                            String sTitle1,
                                            String sTitle2,
                                            String sTitle3,
                                            String sXAxis,
                                            double dXMin, double dXMax, double dXIncr,
                                            String sYAxis,
                                            double dYMin, double dYMax, double dYIncr )
  {
	int i=0;
    plotXYStruct           stPlotXY = null;
    pcPlotFilterListStruct stLegend = null;

//    if (stPlotXY != null)
//      stPlotXY.delete();
//    stPlotXY = null;

	stPlotXY             = new plotXYStruct();

    stPlotXY.sTitle1     = new String( sTitle1 );
    stPlotXY.sTitle2     = new String( sTitle2 );
    stPlotXY.sTitle3     = new String( sTitle3 );

    stPlotXY.iXLog       = 0;      // X-Axis Semilog Indicator
    stPlotXY.iXCycles    = 1;      // Number of Cycles
    stPlotXY.dXMaximum   = dXMax;  // Maximum X-Axis
    stPlotXY.dXMinimum   = dXMin;  // Minimum X-Axis
    stPlotXY.dXIncrement = dXIncr; // Increment along the X-Axis
    stPlotXY.sXMnemonic  = new String( "" );
    stPlotXY.sXAxis      = new String( sXAxis );

    stPlotXY.iYLog       = 0;      // Y-Axis Semilog Indicator
    stPlotXY.iYCycles    = 1;      // Number of Cycles
    stPlotXY.dYMaximum   = dYMax;  // Maximum Y-Axis
    stPlotXY.dYMinimum   = dYMin;  // Minimum Y-Axis
    stPlotXY.dYIncrement = dYIncr; // Increment along the Y-Axis
    stPlotXY.sYMnemonic  = new String( "" );
    stPlotXY.sYAxis      = new String( sYAxis );

    stLegend = pc.pcPlotFilterUtility.getFilter();

    // Add Legend to Plot Data Structure

    if (stLegend != null)
    {
	  if (stPlotXY != null)
	  {
  		if (stLegend.iCount > 0)
  		{
          stPlotXY.bLegend   = true;
          stPlotXY.sLegTitle = new String( "Anions/Cations:" );
		  stPlotXY.stLegend  = new plotSymbolStruct[stLegend.iCount];
		  stPlotXY.iLegend   = stLegend.iCount;

		  for (i=0; i<stLegend.iCount; i++)
		  {
            stPlotXY.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[i].iSymbol,
                    stLegend.stItem[i].iRed,
                    stLegend.stItem[i].iGreen,
                    stLegend.stItem[i].iBlue,
                    stLegend.stItem[i].symbol );
	      }
		}
	  }
    }

    stPlotXY = addData( st, stPlotXY, stLegend );

	return (stPlotXY);
  }

  /** Method addData()
   * <p> This method will add the plot data
   *  @param  st       = principle component data structure
   *  @param  stPlotXY = XY Plot Data Structure
   *  @param  stLegend = Filter Plot List Structure
   *  @return stPlotXY = XY Plot Data Structure
   */

  public static plotXYStruct addData( pcDataStruct           st,
                                      plotXYStruct           stPlotXY,
                                      pcPlotFilterListStruct stLegend )
  {
    int    i        = 0;
    int    iRed     = 0;
    int    iGreen   = 0;
    int    iBlue    = 0;
	int    iCount   = 0;
    int    iFilter  = -1;
    double data     = 0.0;

    // Clear the Plot Data Structure

    if (stPlotXY != null)
    {
      stPlotXY.dX = null; // Array of Horizontal Axis Data
      stPlotXY.dY = null; // Array of Vertical Axis Data
      stPlotXY.dZ = null; // Array of Other Data
      stPlotXY.sZ = null; // Array of Description

      if (stPlotXY.stSymbols != null)
      {
	    for(i=0; i<stPlotXY.iRows; i++)
	    {
		  if (stPlotXY.stSymbols[i] != null)
		  {
		    stPlotXY.stSymbols[i].delete();
	    	stPlotXY.stSymbols[i] = null;
	      }
	    }
        stPlotXY.stSymbols = null;
      }
	}

    if (st != null)
    {
      for (i=0; i<st.iRows; i++)
      {
		data = st.dAnions[i] / st.dCations[i];

		if (isDepth( data, stLegend ))
		  iCount++;
	  }

      stPlotXY.iRows     = iCount;             // Number of Rows
      stPlotXY.depths    = new double[iCount]; // Array of Depths
      stPlotXY.dX        = new double[iCount]; // Array of Horizontal Axis Data
      stPlotXY.dY        = new double[iCount]; // Array of Vertical Axis Data
      stPlotXY.dZ        = new double[iCount]; // Array of Other Data
      stPlotXY.sZ        = new String[iCount]; // Array of Description
      stPlotXY.stSymbols = new plotSymbolStruct[iCount]; // Array of Symbol

      iCount = 0;

      for (i=0; i<st.iRows; i++)
      {
		data = st.dAnions[i] / st.dCations[i];

		if (isDepth( data, stLegend ))
        {
	      if (iCount < stPlotXY.iRows)
	      {
			stPlotXY.depths[iCount] = 0.0;
            stPlotXY.dZ[iCount]     = 0.0;
            stPlotXY.dY[iCount]     = st.dAnions[i];
            stPlotXY.dX[iCount]     = st.dCations[i];
            stPlotXY.sZ[iCount]     = new String("");

            iFilter = getFilter( data, stLegend );

            if (iFilter == 999)
            {
              stPlotXY.stSymbols[iCount] =
                plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    plot.plotSymbolStruct.OVAL,
                    iRed,
                    iGreen,
                    iBlue,
                    "NONE");
            }
            else
            {
   			  stPlotXY.sZ[iCount] = new String(
			    stLegend.stItem[iFilter].symbol );
		      stPlotXY.stSymbols[iCount] =
		        plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[iFilter].iSymbol,
                    stLegend.stItem[iFilter].iRed,
                    stLegend.stItem[iFilter].iGreen,
                    stLegend.stItem[iFilter].iBlue,
                    stLegend.stItem[iFilter].symbol);
		    }

            iCount++;
		  }
		}
	  }
	}

	return (stPlotXY);
  }

  /** Method getFilter()
   *  <p> This method will check if the data is part of the filter
   *  @param  data     = the data value
   *  @param  stLegend = Filter Plot List Structure
   *  @return iValue   = Legend
   */

  public static int getFilter( double data, pcPlotFilterListStruct stLegend )
  {
	int iValue = -1;

    if (stLegend != null)
    {
      for (int i=0; i<stLegend.iCount; i++)
      {
	    if ((data >= stLegend.stItem[i].dataStart) &&
	        (data < stLegend.stItem[i].dataEnd))
	    {
  	      iValue = i;
		}
	  }
	}

	return (iValue);
  }

  /** Method isDepth()
   *  <p> This method will check if the data is part of the filter
   *  @param  data     = the data value
   *  @param  stLegend = Filter Plot List Structure
   *  @return bValue   = true, if the data within filter, false otherwise
   */

  public static boolean isDepth( double data, pcPlotFilterListStruct stLegend )
  {
	boolean bValue = false;

    if (stLegend != null)
    {
      for (int i=0; i<stLegend.iCount; i++)
      {
		if ((data >= stLegend.stItem[i].dataStart) &&
		    (data < stLegend.stItem[i].dataEnd))
		{
  	      bValue = true;
		}
	  }
	}

	return (bValue);
  }

  /* ======================================================================= *
   * ---------------- SET PRINCIPAL COMPONENTS DATA METHODS ---------------- *
   * ======================================================================= */

  /** Method setPlotStruct()
   * <p> This method will set the Plot Structure
   *  @param  iRows    = Total number of Rows
   *  @param  iColumns = Total number of Columns
   *  @param  data     = The first 2 rows of Principle components
   *                     - The second column is X
   *                     - The first column is Y
   *  @return stPlotPc = The Plot XY Data Structure
   */

  public static plotXYStruct setPlotStruct(int iRows, int iColumns, double data[][])
  {
	int i=0;
	double                 dLimit[] = null;
	plotXYStruct           stPlotPc = null;
	pcPlotFilterListStruct stL      = null;

    String sTitle1  = "Kansas Geological Survey";
    String sTitle2  = "Principal Components Crossplot";
    String sTitle3  = "";

    String sXAxis   = "PC1 Scores";
    double dXMin    = -10.0;
    double dXMax    =  10.0;
    double dXIncr   =   2.0;

    String sYAxis   = "PC2 Scores";
    double dYMin    = -10.0;
    double dYMax    =  10.0;
    double dYIncr   =   2.0;

    dLimit = pc.pcUtility.getLimits( iRows, iColumns, data );
    dXMin  = dLimit[0];
    dXMax  = dLimit[1];
    dXIncr = dLimit[2];
    dYMin  = dLimit[3];
    dYMax  = dLimit[4];
    dYIncr = dLimit[5];

	stPlotPc             = new plotXYStruct();

    stPlotPc.sTitle1     = new String( sTitle1 );
    stPlotPc.sTitle2     = new String( sTitle2 );
    stPlotPc.sTitle3     = new String( sTitle3 );

    stPlotPc.iXLog       = 0;      // X-Axis Semilog Indicator
    stPlotPc.iXCycles    = 1;      // Number of Cycles
    stPlotPc.dXMaximum   = dXMax;  // Maximum X-Axis
    stPlotPc.dXMinimum   = dXMin;  // Minimum X-Axis
    stPlotPc.dXIncrement = dXIncr; // Increment along the X-Axis
    stPlotPc.sXMnemonic  = new String( "" );
    stPlotPc.sXAxis      = new String( sXAxis );

    stPlotPc.iYLog       = 0;      // Y-Axis Semilog Indicator
    stPlotPc.iYCycles    = 1;      // Number of Cycles
    stPlotPc.dYMaximum   = dYMax;  // Maximum Y-Axis
    stPlotPc.dYMinimum   = dYMin;  // Minimum Y-Axis
    stPlotPc.dYIncrement = dYIncr; // Increment along the Y-Axis
    stPlotPc.sYMnemonic  = new String( "" );
    stPlotPc.sYAxis      = new String( sYAxis );

//    stLegend = pc.pcPlotFilterUtility.getFilter();

    // Add Legend to Plot Data Structure

    if (stL != null)
    {
	  if (stPlotPc != null)
	  {
  		if (stL.iCount > 0)
  		{
          stPlotPc.bLegend   = true;
          stPlotPc.sLegTitle = new String( "NONE:" );
		  stPlotPc.stLegend  = new plotSymbolStruct[stL.iCount];
		  stPlotPc.iLegend   = stL.iCount;

		  for (i=0; i<stL.iCount; i++)
		  {
            stPlotPc.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stL.stItem[i].iSymbol,
                    stL.stItem[i].iRed,
                    stL.stItem[i].iGreen,
                    stL.stItem[i].iBlue,
                    stL.stItem[i].symbol );
	      }
		}
	  }
    }

    stPlotPc = addData( iRows, iColumns, data, stPlotPc );

    return (stPlotPc);
  }

  /** Method addData()
   * <p> This method will add the plot data
   *  @param  iRows    = Total number of Rows
   *  @param  iColumns = Total number of Columns
   *  @param  data     = The first 2 rows of Principle components
   *                     - The second column is X
   *                     - The first column is Y
   *  @param  stPlotPc = The Plot XY Data Structure
   *  @return stPlotPc = The Plot XY Data Structure
   */

  public static plotXYStruct addData( int iRows, int iColumns, double data[][],
                                      plotXYStruct stPlotPc )
  {
    int    i        = 0;
    int    iRed     = 150;
    int    iGreen   = 150;
    int    iBlue    = 150;
	int    iCount   = 0;
    int    iFilter  = -1;
//    double data     = 0.0;
	pcPlotFilterListStruct stL = null;

    // Clear the Plot Data Structure

    if (stPlotPc != null)
    {
      stPlotPc.dX = null; // Array of Horizontal Axis Data
      stPlotPc.dY = null; // Array of Vertical Axis Data
      stPlotPc.dZ = null; // Array of Other Data
      stPlotPc.sZ = null; // Array of Description

      if (stPlotPc.stSymbols != null)
      {
	    for(i=0; i<stPlotPc.iRows; i++)
	    {
		  if (stPlotPc.stSymbols[i] != null)
		  {
		    stPlotPc.stSymbols[i].delete();
	    	stPlotPc.stSymbols[i] = null;
	      }
	    }
        stPlotPc.stSymbols = null;
      }
	}

    if (stPlotPc != null)
    {
      stPlotPc.iRows     = iRows;             // Number of Rows
      stPlotPc.depths    = new double[iRows]; // Array of Depths
      stPlotPc.dX        = new double[iRows]; // Array of Horizontal Axis Data
      stPlotPc.dY        = new double[iRows]; // Array of Vertical Axis Data
      stPlotPc.dZ        = new double[iRows]; // Array of Other Data
      stPlotPc.sZ        = new String[iRows]; // Array of Description
      stPlotPc.stSymbols = new plotSymbolStruct[iRows]; // Array of Symbol

      iCount = 0;

      for (i=0; i<iRows; i++)
      {
//		data = st.dAnions[i] / st.dCations[i];

//		if (isDepth( data ))
        {
	      if (iCount < stPlotPc.iRows)
	      {
			stPlotPc.depths[iCount] = 0.0;
            stPlotPc.dZ[iCount]     = 0.0;
            stPlotPc.dY[iCount]     = data[i][0];
            stPlotPc.dX[iCount]     = data[i][1];
            stPlotPc.sZ[iCount]     = new String("");

            iFilter = 999; //getFilter( data );

            if (iFilter == 999)
            {
              stPlotPc.stSymbols[iCount] =
                plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    plot.plotSymbolStruct.DIAMOND,
                    iRed,
                    iGreen,
                    iBlue,
                    "NONE");
            }
            else
            {
   			  stPlotPc.sZ[iCount] = new String(
			    stL.stItem[iFilter].symbol );
		      stPlotPc.stSymbols[iCount] =
		        plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stL.stItem[iFilter].iSymbol,
                    stL.stItem[iFilter].iRed,
                    stL.stItem[iFilter].iGreen,
                    stL.stItem[iFilter].iBlue,
                    stL.stItem[iFilter].symbol);
		    }

            iCount++;
		  }
		}
	  }
	}

	return (stPlotPc);
  }
}


/*
 *  @version 1.1 11/20/2015
 *  @author  John Victorine
 */
