/*
 * @pcUtility.java Version 1.1 10/15/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc;

import Jama.*;                 // Import Jama classes

import brine.brineListStruct;  // Brine Data List Structure

import pc.pcDataStruct;

/** Class pcUtility
 *  <p> This Class will provide basic utility for the data
 *
 *  @version 1.1 10/15/2015
 *  @author  John R. Victorine
 */

public class pcUtility
{
  public static final int _SPGR   = brine.brineStandardTools._SPGR;   // Specific Gravity
  public static final int _SIGMA  = brine.brineStandardTools._SIGMA;  // Specific Conductivity
  public static final int _PH     = brine.brineStandardTools._PH;     // PH
  public static final int _DEG    = brine.brineStandardTools._DEG;    // Degree F
  public static final int _OHM    = brine.brineStandardTools._OHM;    // Resistivity (Rw)
  public static final int _OHM75  = brine.brineStandardTools._OHM75;  // Resistivity at 75 deg
  public static final int _OHME   = brine.brineStandardTools._OHME;   // Estimated Rw

  public static final int _ALKNTY = brine.brineStandardTools._ALKNTY; // Alkalinity
  public static final int _NTU    = brine.brineStandardTools._NTU;    // Turbidity

  // .. Alkali Metal
  public static final int _LI     = brine.brineStandardTools._LI;     // Lithium            1+
  public static final int _NA     = brine.brineStandardTools._NA;     // Sodium             1+
  public static final int _K      = brine.brineStandardTools._K;      // Potassium          1+
  public static final int _RB     = brine.brineStandardTools._RB;     // Rubidium           1+
  public static final int _CS     = brine.brineStandardTools._CS;     // Cesium             1+
  public static final int _NA_K   = brine.brineStandardTools._NA_K;   // Sodium/Potasium    1+

  // ..  Alkali Earth Metal
  public static final int _BE     = brine.brineStandardTools._BE;     // Beryllium          2+
  public static final int _MG     = brine.brineStandardTools._MG;     // Magnesium          2+
  public static final int _CA     = brine.brineStandardTools._CA;     // Calcium            2+
  public static final int _SR     = brine.brineStandardTools._SR;     // Strontium          2+
  public static final int _BA     = brine.brineStandardTools._BA;     // Barium             2+

  // ..  VIB
  public static final int _CR_II  = brine.brineStandardTools._CR_II;  // Chromium (II)   chromous  2+
  public static final int _CR_III = brine.brineStandardTools._CR_III; // Chromium (III)  chromic   3+

  // ..  VIIB
  public static final int _MN_II  = brine.brineStandardTools._MN_II;  // Manganese (II)  manganous 2+
  public static final int _MN_III = brine.brineStandardTools._MN_III; // Manganese (III) manganic  3+

  // ..  VIIIB
  public static final int _FE_II  = brine.brineStandardTools._FE_II;  // Iron (II)    ferrous      2+
  public static final int _FE_III = brine.brineStandardTools._FE_III; // Iron (III)   ferric		 3+
  public static final int _CO_II  = brine.brineStandardTools._CO_II;  // Cobalt (II)  cobaltous    2+
  public static final int _CO_III = brine.brineStandardTools._CO_III; // Cobalt (III) cobaltic     3+
  public static final int _NI_II  = brine.brineStandardTools._NI_II;  // Nickel (II)  nickelous    2+
  public static final int _NI_III = brine.brineStandardTools._NI_III; // Nickel (III) nickelic     3+

  // ..  IB
  public static final int _CU_I   = brine.brineStandardTools._CU_I;   // Copper (I)   cuprous      1+
  public static final int _CU_II  = brine.brineStandardTools._CU_II;  // Copper (II)  cupric       2+
  public static final int _AG     = brine.brineStandardTools._AG;     // Silver                    1+
  public static final int _AU     = brine.brineStandardTools._AU;     // Gold (I)     aurous       1+
  public static final int _AU_III = brine.brineStandardTools._AU_III; // Gold (III)   auric        3+

    // ..  IIB
  public static final int _ZN     = brine.brineStandardTools._ZN;     // Zinc                      2+
  public static final int _CD     = brine.brineStandardTools._CD;     // Cadmium                   2+
  public static final int _HG     = brine.brineStandardTools._HG;     // Mercury (I)  mercurous Hg2 2+
  public static final int _HG_II  = brine.brineStandardTools._HG_II;  // Mercury (II) mercuric     2+

  // ..  IIIA
  public static final int _AL     = brine.brineStandardTools._AL;     // Aluminium                 3+

  //.. VA
  public static final int _SB_III = brine.brineStandardTools._SB_III; // Antimony (III)  antimonous  3+
  public static final int _SB_V   = brine.brineStandardTools._SB_V;   // Antimony (V)    antimonic   5+
  public static final int _BI_III = brine.brineStandardTools._BI_III; // Bismuthous(III) bismuthous  3+
  public static final int _BI_V   = brine.brineStandardTools._BI_V;   // Bismuth(V)      bismuthic   5+

  // ..  IVA
  public static final int _SN_II  = brine.brineStandardTools._SN_II;  // Tin (II)  stannous          2+
  public static final int _SN_IV  = brine.brineStandardTools._SN_IV;  // Tin (IV)  stannic           4+
  public static final int _PB_II  = brine.brineStandardTools._PB_II;  // Lead (II) plumbous          2+
  public static final int _PB_IV  = brine.brineStandardTools._PB_IV;  // Lead (IV) plumbic           4+

  // .. Other
  public static final int _NH4    = brine.brineStandardTools._NH4;    // Ammonium                    1+

  // Common Anions

  public static final int _F      = brine.brineStandardTools._F;      // Flouride        1-
  public static final int _CL     = brine.brineStandardTools._CL;     // Chloride        1-
  public static final int _BR     = brine.brineStandardTools._BR;     // Bromide         1-
  public static final int _I      = brine.brineStandardTools._I;      // Iodide          1-

  public static final int _OH     = brine.brineStandardTools._OH;     // Hydroxide       1-

  public static final int _BO3    = brine.brineStandardTools._BO3;    // Borate
  public static final int _CO3    = brine.brineStandardTools._CO3;    // Carbonate       2-
  public static final int _HCO3   = brine.brineStandardTools._HCO3;   // Bicarbonate     1-

  public static final int _CLO    = brine.brineStandardTools._CLO;    // Hypochlorite    1-
  public static final int _CLO2   = brine.brineStandardTools._CLO2;   // Chlorite        1-
  public static final int _CLO3   = brine.brineStandardTools._CLO3;   // Chlorate        1-
  public static final int _CLO4   = brine.brineStandardTools._CLO4;   // Per chlorate    1-

  public static final int _CN     = brine.brineStandardTools._CN;     // Cyanide         1-
  public static final int _NCO    = brine.brineStandardTools._NCO;    // Cyanate         1-
  public static final int _OCN    = brine.brineStandardTools._OCN;    // Isocyanate      1-
  public static final int _SCN    = brine.brineStandardTools._SCN;    // Thiocyanite     1-

  public static final int _N      = brine.brineStandardTools._N;      // Nitride         3-
  public static final int _N3     = brine.brineStandardTools._N3;     // Azide           1-
  public static final int _NO2    = brine.brineStandardTools._NO2;    // Nitrite         1-
  public static final int _NO3    = brine.brineStandardTools._NO3;    // Nitrate         1-

  public static final int _CRO4   = brine.brineStandardTools._CRO4;   // Chromium (II)   Chromate    2-
  public static final int _CR2O7  = brine.brineStandardTools._CR2O7;  // Chromium (III)  Dichormate  2-

  public static final int _MNO4   = brine.brineStandardTools._MNO4;   // Permanganate    1-

  public static final int _P      = brine.brineStandardTools._P;      // Phosphide       3-
  public static final int _PO3    = brine.brineStandardTools._PO3;    // Phosphite       3-
  public static final int _PO4    = brine.brineStandardTools._PO4;    // Phosphate       3-
  public static final int _HPO4   = brine.brineStandardTools._HPO4;   // Monohydrogen Phosphate 2-
  public static final int _H2PO4  = brine.brineStandardTools._H2PO4;  // Dihydrogen Phosphate   1-

  public static final int _AS     = brine.brineStandardTools._AS;     // Arsinide        3-
  public static final int _SE     = brine.brineStandardTools._SE;     // Selenide        2-

  public static final int _S      = brine.brineStandardTools._S;      // Sulfide            2-
  public static final int _HS     = brine.brineStandardTools._HS;     // Hydrogen Sulfide   1-
  public static final int _SO3    = brine.brineStandardTools._SO3;    // Sulphite           2-
  public static final int _HSO3   = brine.brineStandardTools._HSO3;   // Hydrogen Sulphite  1-
  public static final int _S2O3   = brine.brineStandardTools._S2O3;   // Thiosulphate       2-
  public static final int _SO4    = brine.brineStandardTools._SO4;    // Sulfate            2-
  public static final int _HSO4   = brine.brineStandardTools._HSO4;   // Bisulfate          1-

  public static final int _SOLID  = brine.brineStandardTools._SOLID;  // Total Solids
  public static final int _TDS    = brine.brineStandardTools._TDS;    // Total Solids Computed

  public static final int _NAME   = brine.brineStandardTools._NAME;   // Well Name
  public static final int _API    = brine.brineStandardTools._API;    // API-Number
  public static final int _SRC    = brine.brineStandardTools._SRC;    // Source

  public static final String CURVES[][] = brine.brineStandardTools.CURVES;

  /** Method computeMean()
   *  <p> This method will compute the mean of each row of data
   *  @param  iRows    = Total number of rows of data set.
   *  @param  iColumns = Total number of columns of data set.
   *  @param  d        = the data set
   *  @return dMean    = Mean Value of each column of data set.
   */

  public static double[] computeMean( int iRows, int iColumns, double d[][] )
  {
	double dMean[] = null;

	if ((d != null) && (iRows > 0) && (iColumns > 0))
	{
	  dMean = new double[iColumns];

  	  for (int j=0; j<iColumns; j++)
	  {
        dMean[j] = 0.0;

  	    for (int i=0; i<iRows; i++)
		{
		  dMean[j] = dMean[j] + d[i][j];
		}

		dMean[j] = dMean[j] / iRows;
	  }
	}

	return (dMean);
  }

  /** Method adjustByMean()
   *  <p> Adjust the raw data with respect to the computed mean
   *  @param  iRows    = Total number of rows of data set.
   *  @param  iColumns = Total number of columns of data set.
   *  @param  d        = the data set
   *  @param  M        = Mean Value of each column of data set.
   *  @return dAdata   = The data adjusted with respect to the Mean
   */

  public static double[][] adjustByMean( int iRows, int iColumns, double d[][], double M[] )
  {
	double dAdata[][] = null;

	if ((d != null) && (iRows > 0) && (iColumns > 0) && (M != null))
	{
	  dAdata = new double[iRows][iColumns];

      for (int j=0; j<iColumns; j++)
	  {
	    for (int i=0; i<iRows; i++)
		{
		  dAdata[i][j] = d[i][j] - M[j];
		}
	  }
	}

	return (dAdata);
  }

  /** Method computeSD()
   *  <p> This method will compute the standard deviation of each row of data
   *  @param  iRows    = Total number of rows of data set.
   *  @param  iColumns = Total number of columns of data set.
   *  @param  d        = the data set
   *  @return dMean    = Mean Value of each column of data set.
   */

  public static double[] computeSD( int iRows, int iColumns, double d[][] )
  {
	double dSD[] = null;

	if ((d != null) && (iRows > 0) && (iColumns > 0))
	{
	  dSD = new double[iColumns];

  	  for (int j=0; j<iColumns; j++)
	  {
        dSD[j] = 0.0;

  	    for (int i=0; i<iRows; i++)
		{
		  dSD[j] = dSD[j] + d[i][j] * d[i][j];
		}

		dSD[j] = Math.sqrt( dSD[j] / (iRows-1) );
	  }
	}

	return (dSD);
  }

  /** Method covariance()
   *  <p> This method will build the covariance Matrix of Adjusted Data
   */

  public static Matrix covariance( int iRows, int iColumns, double d[][] )
  {
	Matrix M    = null;
	double dSum = 0.0;
//	double dNorm = 0.0;
	double dVal = 0.0;
	int    N    = iRows-1;
	double s[]  = null;
	double s2   = 0.0;

	if ((d != null) && (iRows > 0) && (iColumns > 1))
	{
      s = new double[iColumns];

	  for (int i=0; i<iColumns; i++)
	  {
		s2 = 0.0;
  	    for (int k=0; k<iRows; k++)
		{
		  s2 = s2 + d[k][i] * d[k][i];
		}
		s2 = s2 / N;
		s[i] = Math.sqrt(s2);
	  }

	  M = new Matrix(iColumns, iColumns);

	  for (int i=0; i<iColumns; i++)
	  {
		for (int j=0; j<iColumns; j++)
		{
		  dSum = 0.0;

		  for (int k=0; k<iRows; k++)
		  {
			dSum = dSum + d[k][i] * d[k][j] / (s[i] * s[j]);
		  }

		  dVal = dSum / N;
		  M.set(i,j,dVal);
		}
	  }
	}

	return (M);
  }

  /** Method getData()
   *  <p> Compute the Raw data from the Eigenvector and adjusted Data and Mean
   *  @param  iRows    = Total number of rows of data set.
   *  @param  iColumns = Total number of columns of data set.
   *  @param  d        = the adjusted data set
   *  @param  Et       = Eigenvectors
   *  @return out      = The data returned.
   */

  public static double[][] getData( int iRows, int iColumns, double d[][], Matrix Et )
  {
	int i,j,k,n;
	double dMatrix[][] = null;
	double out[][]     = null;
	double dSum        = 0.0;

	if ((d != null) && (iRows > 0) && (iColumns > 0) && (Et != null))
	{
//	  Et.print(8,4);
	  dMatrix = Et.getArray();

	  out = new double[iRows][iColumns];

      for (k=0; k<iRows; k++)
	  {
	    for (i=0; i<iColumns; i++)
	    {
		  out[k][i] = 0.0;
		  for (j=0; j<iColumns; j++)
		  {
			out[k][i] = out[k][i] + dMatrix[i][j] * d[k][j];
		  }
		}
	  }
	}

	return (out);
  }

  /** Method setMatrix()
   *  <p> This method will build the Matrix of from Data Array
   */

  public static Matrix setMatrix( int iRows, int iColumns, double d[][] )
  {
	Matrix M    = null;

	if ((d != null) && (iRows > 0) && (iColumns > 1))
	{
	  M = new Matrix(iRows, iColumns);

	  for (int i=0; i<iRows; i++)
	  {
		for (int j=0; j<iColumns; j++)
		{
		  M.set( i, j, d[i][j] );
		}
	  }
	}

	return (M);
  }

  /* ======================================================================= *
   * --------------------------- LOAD BRINE DATA --------------------------- *
   * ======================================================================= */

  /** Method getColumns()
   *  <p> This method will count the number anions and cations that can be used
   *  @param  st     = The brine List Structure
   *  @return iCount = The total number of Cations & Anions
   */

  public static int getColumns( brineListStruct st )
  {
	int iCount = 0;

	if (st != null)
	{
	  for (int i=_LI; i<_SOLID; i++)
	  {

//		if ((i != _MN_II) &&
//		    (i != _FE_II) &&
//		    (i != _FE_III) &&
//		    (i != _BA) &&
//		    (i != _ZN))

//		if ((i == _NA) ||
//		    (i == _CL))
        if ((i != _FE_III) &&
		    (i != _BA) &&
		    (i != _ZN))
		{
		  if (st.checkData(i))
		  {
		    iCount++;
		  }
		}
	  }

	  if (st.checkData(_PH))
	    iCount++;
	}

	return (iCount);
  }

  /** Method getOrder()
   *  <p> This method will order the number anions and cations that can be used
   *  @param  st     = The brine List Structure
   *  @return iBrine = The order of the Brine Data columns
   */

  public static int[][] getOrder( brineListStruct st )
  {
	int iBrine[][] = null;
	int iColumns   = 0;
	int iCount     = 0;

	if (st != null)
	{
	  iColumns = getColumns( st );
	  iBrine   = new int[iColumns][2];

	  for (int i=_LI; i<_SOLID; i++)
	  {

//		if ((i != _MN_II) &&
//		    (i != _FE_II) &&
//		    (i != _FE_III) &&
//		    (i != _BA) &&
//		    (i != _ZN))

//		if ((i == _NA) ||
//		    (i == _CL))
        if ((i != _FE_III) &&
		    (i != _BA) &&
		    (i != _ZN))
		{
	      if (st.checkData(i))
	      {
		    if (iCount<iColumns)
		    {
              iBrine[iCount][0] = iCount;
              iBrine[iCount][1] = i;
              iCount++;
			}
		  }
		}
	  }

	  if (st.checkData(_PH))
      {
	    if (iCount<iColumns)
	    {
          iBrine[iCount][0] = iCount;
          iBrine[iCount][1] = _PH;
          iCount++;
		}
	  }
	}

	return (iBrine);
  }

  /** Method getBrineData()
   *  <p> This method will retrieve the brine data as arrays
   *  @param  st     = brine List Data Structure
   *  @return stData = Principal Components Data Structure
   */

  public static pcDataStruct getBrineData( brineListStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    int          iColumns = 0;
    double       dValue   = 0.0;

    if (st != null)
    {
	  stData          = new pcDataStruct();

      stData.iRows    = st.iCount;
	  stData.iColumns = getColumns( st );
      stData.iBrine   = getOrder( st );

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      for (i=0; i<st.iCount; i++)
      {
		stData.sData[i][0] = new String( st.stItem[i].sAPI );
		stData.sData[i][1] = new String( st.stItem[i].sKID );
		stData.sData[i][2] = new String( st.stItem[i].sKEY );

        stData.dCations[i] = 0.0;
        stData.dAnions[i]  = 0.0;

		for (j=0; j<stData.iColumns; j++)
		{
		  stData.data[i][j] = st.getData( stData.iBrine[j][1], i );

		  if ((stData.iBrine[j][1] >= _LI) && (stData.iBrine[j][1] <= _NH4))
		    stData.dCations[i] = stData.dCations[i] + stData.data[i][j];
		  else if ((stData.iBrine[j][1] >= _F) && (stData.iBrine[j][1] <= _HSO4))
		    stData.dAnions[i]  = stData.dAnions[i]  + stData.data[i][j];
		}
	  }

	  stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method getBrineData_meq()
   *  <p> This method will retrieve the brine data as arrays
   *  @param  st     = brine List Data Structure
   *  @return stData = Principal Components Data Structure
   */

  public static pcDataStruct getBrineData_meq( brineListStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    int          iColumns = 0;
    double       dValue   = 0.0;
    int          ion[]    = null;
    double       data[]   = null;
    double       dmeq[]   = null;

    if (st != null)
    {
	  stData          = new pcDataStruct();

      stData.iRows    = st.iCount;
	  stData.iColumns = getColumns( st );
      stData.iBrine   = getOrder( st );

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      ion             = new int[stData.iColumns];

      for (i=0; i<stData.iColumns-1; i++)
      {
		ion[i] = stData.iBrine[i][1];
	  }

      for (i=0; i<st.iCount; i++)
      {
		stData.sData[i][0] = new String( st.stItem[i].sAPI );
		stData.sData[i][1] = new String( st.stItem[i].sKID );
		stData.sData[i][2] = new String( st.stItem[i].sKEY );

        data = brine.math.brineMath.get_mg_per_liter(-1, i, ion, st );
        dmeq = brine.math.brineMath.mg_to_meq_per_liter(-1, ion, data );

        stData.dCations[i] = 0.0;
        stData.dAnions[i]  = 0.0;

		for (j=0; j<stData.iColumns-1; j++)
		{
		  stData.data[i][j] = dmeq[j];

		  if ((stData.iBrine[j][1] >= _LI) && (stData.iBrine[j][1] <= _NH4))
		    stData.dCations[i] = stData.dCations[i] + stData.data[i][j];
		  else if ((stData.iBrine[j][1] >= _F) && (stData.iBrine[j][1] <= _HSO4))
		    stData.dAnions[i]  = stData.dAnions[i]  + stData.data[i][j];
		}

        stData.data[i][stData.iColumns-1] = st.getData( _PH, i );

		stData.dCations[i] = (double)((int) (100.0 * stData.dCations[i] ) / 100.0);
		stData.dAnions[i]  = (double)((int) (100.0 * stData.dAnions[i] ) / 100.0);
	  }

	  stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method applyGroup()
   *  <p> This method will separate the brine data by type for printing.
   *  @param  st     = Original brine List Data Structure
   *  @param  stPC   = Principal Components Data Structure
   *  @return stNorm = Normalized brine List Data Structure
   */

  public static brineListStruct applyGroup( brineListStruct st, pcDataStruct stPC )
  {
	brineListStruct stNorm = null;

	if ((st != null) && (stPC != null))
	{
      for (int i=0; i<stPC.iRows; i++)
      {
	    for (int k=0; k<st.iCount; k++)
		{
		  if (st.stItem[k].sKEY.equals(stPC.sData[i][2]))
		  {
            stNorm = brine.brineUtility.add(st.stItem[k], stNorm);
		  }
	    }
	  }

      stNorm = brine.brineUtility.copyBrineHeader( st, stNorm );
	}

	return (stNorm);
  }

  /** Method applyNormilization()
   *  <p> This method will modify the original data set and apply the normilization
   *      data to the original data.
   *  @param  st     = Original brine List Data Structure
   *  @param  stPC   = Principal Components Data Structure
   *  @return stNorm = Normalized brine List Data Structure
   */

  public static brineListStruct applyNormilization( brineListStruct st, pcDataStruct stPC )
  {
	brineListStruct stNorm = null;
	double          dmg    = 0.0;

	if ((st != null) && (stPC != null))
	{
      stNorm = brine.brineUtility.copyList( st );

      if (stNorm != null)
      {
        for (int i=0; i<stPC.iRows; i++)
        {
		  for (int j=0; j<stPC.iColumns; j++)
		  {
			for (int k=0; k<stNorm.iCount; k++)
			{
		      if (stNorm.stItem[k].sKEY.equals(stPC.sData[i][2]))
		      {
//System.out.print(stNorm.stItem[k].sAPI + " " + stNorm.stItem[k].sName);
                if ((stPC.iBrine[j][1] >= _LI) && (stPC.iBrine[j][1] <= _HSO4))
                {
				  dmg = brine.math.brineMath.meq_to_mg_per_liter( stPC.iBrine[j][1],
				                                                  stPC.data[i][j]);
				  stNorm.setData( stPC.iBrine[j][1], k, dmg );
//System.out.println(CURVES[stPC.iBrine[j][1]][1]+" "+st.getData(stPC.iBrine[j][1], k)+" "+stNorm.getData(stPC.iBrine[j][1], k));
				}
				else
				{
				  stNorm.setData( stPC.iBrine[j][1], k, stPC.data[i][j] );
//System.out.println(CURVES[stPC.iBrine[j][1]][1]+" "+st.getData(stPC.iBrine[j][1], k)+" "+stNorm.getData(stPC.iBrine[j][1], k));
				}
			  }
			}
		  }
		}
	  }
	}

	return (stNorm);
  }

  /** Method getLimits()
   *  <p> This method compute the limits of the data set for plotting.
   *  @param  st     = Principal Components Data Structure
   *  @return dLimit = Array of Limits for the plot
   */

  public static double[] getLimits( pcDataStruct st  )
  {
	//                  Xmin Xmax Xinc Ymin Ymax Yinc
	double dLimit[] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };

	if (st != null)
	{
      for (int i=0; i<st.iRows; i++)
      {
  	    if (i==0)
  	    {
		  dLimit[0] = st.dCations[i];
		  dLimit[1] = st.dCations[i];
		  dLimit[3] = st.dAnions[i];
		  dLimit[4] = st.dAnions[i];
		}

		if (dLimit[0] > st.dCations[i])
		  dLimit[0] = st.dCations[i];

		if (dLimit[1] < st.dCations[i])
		  dLimit[1] = st.dCations[i];

		if (dLimit[3] > st.dAnions[i])
		  dLimit[3] = st.dAnions[i];

		if (dLimit[4] < st.dAnions[i])
		  dLimit[4] = st.dAnions[i];

		for (int j=0; j<6; j++)
		{
		  dLimit[j] = (double) ( (int) (dLimit[j]/100.0) * 100.0 );
		}

		dLimit[2] = (dLimit[1] - dLimit[0]) / 10.0;
		dLimit[5] = (dLimit[4] - dLimit[3]) / 10.0;

		dLimit[2] = (double) ((int) (dLimit[2]/100.0) * 100.0);
		dLimit[5] = (double) ((int) (dLimit[5]/100.0) * 100.0);

		if (dLimit[2] == 0.0) { dLimit[2] = 100.0;}
		if (dLimit[5] == 0.0) { dLimit[5] = 100.0;}
	  }
	}

	return (dLimit);
  }

  /** Method getLimits()
   *  <p> This method compute the limits of the data set for plotting.
   *  @param  iRows    = Total number of Rows
   *  @param  iColumns = Total number of Columns
   *  @param  data     = The first 2 rows of Principle components
   *                     - The second column is X
   *                     - The first column is Y
   *  @return dLimit   = Array of Limits for the plot
   */

  public static double[] getLimits( int iRows, int iColumns, double data[][] )
  {
	//                  Xmin Xmax Xinc Ymin Ymax Yinc
	double dLimit[] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };

	if (data != null)
	{
      for (int i=0; i<iRows; i++)
      {
  	    if (i==0)
  	    {
		  dLimit[0] = data[i][1];
		  dLimit[1] = data[i][1];
		  dLimit[3] = data[i][0];
		  dLimit[4] = data[i][0];
		}

		if (dLimit[0] > data[i][1])
		  dLimit[0] = data[i][1]-1;

		if (dLimit[1] < data[i][1])
		  dLimit[1] = data[i][1]+1;

		if (dLimit[3] > data[i][0])
		  dLimit[3] = data[i][0]-1;

		if (dLimit[4] < data[i][0])
		  dLimit[4] = data[i][0]+1;

		for (int j=0; j<6; j++)
		{
//		  dLimit[j] = (double) ( (int) (dLimit[j]*100.0) / 100.0 );
		  dLimit[j] = (double) ( (int) dLimit[j] );
		}

		dLimit[2] = 1.0; //(dLimit[1] - dLimit[0]) / 10.0;
		dLimit[5] = 1.0;//(dLimit[4] - dLimit[3]) / 10.0;

//		dLimit[2] = (double) ((int) (dLimit[2]/10.0) * 10.0);
//		dLimit[5] = (double) ((int) (dLimit[5]/10.0) * 10.0);

//		if (dLimit[2] == 0.0) { dLimit[2] = 10.0;}
//		if (dLimit[5] == 0.0) { dLimit[5] = 10.0;}
	  }
	}

	return (dLimit);
  }

  /** Method isData()
   *  <p> This method will check data for columns with only 0.
   *  @param  st     = Principal Components Data Structure
   *  @return iCol   = Columns with problems
   */

  public static int[] isData( pcDataStruct st )
  {
	int i,j;
    int iCol[]   = null;

    if (st != null)
    {
      iCol = new int[st.iColumns];
      for (i=0; i<st.iColumns; i++)
        iCol[i] = -1;

      for (i=0; i<st.iRows; i++)
      {
  	    for (j=0; j<st.iColumns; j++)
		{
		  if (st.data[i][j] > 0.0)
            iCol[j] = 1;
		}
	  }
	}

	return (iCol);
  }

  /** Method count_good()
   *  <p> This method will count the brine data within the Anions/Cations Ratio of 1.0
   *      +/- dVal.
   *  @param  dVal   = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param  st     = Principal Components Data Structure
   *  @return iCount = The Count of Good Data
   */

  public static int count_good( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if ((dRatio >= dMin) && (dRatio <= dMax))
		  iCount++;
	  }
	}

	return (iCount);
  }

  /** Method isData_good()
   *  <p> This method will check data for columns with only 0.
   *  @param  dVal   = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param  st     = Principal Components Data Structure
   *  @return iCol   = Columns with problems
   */

  public static int[] isData_good( double dVal, pcDataStruct st )
  {
	int i,j;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCol[]   = null;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      iCol = new int[st.iColumns];
      for (i=0; i<st.iColumns; i++)
        iCol[i] = -1;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if ((dRatio >= dMin) && (dRatio <= dMax))
		{
		  for (j=0; j<st.iColumns; j++)
		  {
			if (st.data[i][j] > 0.0)
              iCol[j] = 1;
		  }
		}
	  }
	}

	return (iCol);
  }

  /** Method getBrineData_good()
   *  <p> This method will retrieve the brine data within the Anions/Cations Ratio of 1.0
   *      +/- dVal.
   *  @param  dVal   = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param  st     = Principal Components Data Structure
   *  @return stData = The Good Principal Components Data Structure
   */

  public static pcDataStruct getBrineData_good( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if ((dRatio >= dMin) && (dRatio <= dMax))
		  iCount++;
	  }

	  stData          = new pcDataStruct();
      stData.iColumns = st.iColumns;
      stData.iRows    = iCount;

      stData.iBrine   = new int[stData.iColumns][2];

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      for (i=0; i<stData.iColumns; i++)
      {
		stData.iBrine[i] = st.iBrine[i];
	  }

      iCount = 0;
      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if ((dRatio >= dMin) && (dRatio <= dMax))
		{
		  if (iCount < st.iRows)
		  {
			for (j=0; j<3; j++)
              stData.sData[iCount][j]  = new String( st.sData[i][j] );

            stData.dCations[iCount] = st.dCations[i];
            stData.dAnions[iCount]  = st.dAnions[i];

		    for (j=0; j<stData.iColumns; j++)
		    {
              stData.data[iCount][j]  = st.data[i][j];
			}
			iCount++;
		  }
		}
	  }

      stData.iRows   = iCount;
      stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method count_rogue()
   *  <p> This method will count the brine data outside the Anions/Cations Ratio of 1.0
   *      +/- dVal.
   *  @param dVal    = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param st      = Principal Components Data Structure
   *  @return stData = The Rogue Principal Components Data Structure
   */

  public static int count_rogue( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if ((dRatio < dMin) || (dRatio > dMax))
		  iCount++;
	  }
	}

	return (iCount);
  }

  /** Method isData_rogue()
   *  <p> This method will check data for columns with only 0.
   *  @param  dVal   = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param  st     = Principal Components Data Structure
   *  @return iCol   = Columns with problems
   */

  public static int[] isData_rogue( double dVal, pcDataStruct st )
  {
	int i,j;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCol[]   = null;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      iCol = new int[st.iColumns];
      for (i=0; i<st.iColumns; i++)
        iCol[i] = -1;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if ((dRatio < dMin) || (dRatio > dMax))
		{
		  for (j=0; j<st.iColumns; j++)
		  {
			if (st.data[i][j] > 0.0)
              iCol[j] = 1;
		  }
		}
	  }
	}

	return (iCol);
  }

  /** Method getBrineData_rogue()
   *  <p> This method will retrieve the brine data outside the Anions/Cations Ratio of 1.0
   *      +/- dVal.
   *  @param dVal    = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param st      = Principal Components Data Structure
   *  @return stData = The Rogue Principal Components Data Structure
   */

  public static pcDataStruct getBrineData_rogue( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if ((dRatio < dMin) || (dRatio > dMax))
		  iCount++;
	  }

	  stData          = new pcDataStruct();
      stData.iColumns = st.iColumns;
      stData.iRows    = iCount;

      stData.iBrine   = new int[stData.iColumns][2];

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      for (i=0; i<stData.iColumns; i++)
      {
		stData.iBrine[i] = st.iBrine[i];
	  }

      iCount = 0;
      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if ((dRatio < dMin) || (dRatio > dMax))
		{
		  if (iCount < st.iRows)
		  {
			for (j=0; j<3; j++)
              stData.sData[iCount][j]  = new String( st.sData[i][j] );

            stData.dCations[iCount] = st.dCations[i];
            stData.dAnions[iCount]  = st.dAnions[i];

		    for (j=0; j<stData.iColumns; j++)
		    {
              stData.data[iCount][j]  = st.data[i][j];
			}
			iCount++;
		  }
		}
	  }

      stData.iRows   = iCount;
      stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method count_rogueBelow()
   *  <p> This method will count the brine data outside the Anions/Cations Ratio of 1.0
   *      +/- dVal.
   *  @param dVal    = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param st      = Principal Components Data Structure
   *  @return iCount = The Count of Rogue Data below the Anions/Cations Ratio of 1.0
   */

  public static int count_rogueBelow( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio < dMin)
		  iCount++;
	  }
	}

	return (iCount);
  }

  /** Method isData_rogueBelow()
   *  <p> This method will check data for columns with only 0.
   *  @param  dVal   = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param  st     = Principal Components Data Structure
   *  @return iCol   = Columns with problems
   */

  public static int[] isData_rogueBelow( double dVal, pcDataStruct st )
  {
	int i,j;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCol[]   = null;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      iCol = new int[st.iColumns];
      for (i=0; i<st.iColumns; i++)
        iCol[i] = -1;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio < dMin)
		{
		  for (j=0; j<st.iColumns; j++)
		  {
			if (st.data[i][j] > 0.0)
              iCol[j] = 1;
		  }
		}
	  }
	}

	return (iCol);
  }

  /** Method getBrineData_rogueBelow()
   *  <p> This method will retrieve the brine data below the Anions/Cations Ratio of 1.0
   *      +/- dVal.
   *  @param dVal    = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param st      = Principal Components Data Structure
   *  @return stData = The Rogue Principal Components Data Structure
   */

  public static pcDataStruct getBrineData_rogueBelow( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio < dMin)
		  iCount++;
	  }

	  stData          = new pcDataStruct();
      stData.iColumns = st.iColumns;
      stData.iRows    = iCount;

      stData.iBrine   = new int[stData.iColumns][2];

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      for (i=0; i<stData.iColumns; i++)
      {
		stData.iBrine[i] = st.iBrine[i];
	  }

      iCount = 0;
      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio < dMin)
		{
		  if (iCount < st.iRows)
		  {
			for (j=0; j<3; j++)
              stData.sData[iCount][j]  = new String( st.sData[i][j] );

            stData.dCations[iCount] = st.dCations[i];
            stData.dAnions[iCount]  = st.dAnions[i];

		    for (j=0; j<stData.iColumns; j++)
		    {
              stData.data[iCount][j]  = st.data[i][j];
			}
			iCount++;
		  }
		}
	  }

      stData.iRows   = iCount;
      stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method count_rogueAbove()
   *  <p> This method will count the brine data outside the Anions/Cations Ratio of 1.0
   *      +/- dVal.
   *  @param dVal    = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param st      = Principal Components Data Structure
   *  @return iCount = The Count of Rogue Data above the Anions/Cations Ratio of 1.0
   */

  public static int count_rogueAbove( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio > dMax)
		  iCount++;
	  }
	}

	return (iCount);
  }

  /** Method isData_rogueAbove()
   *  <p> This method will check data for columns with only 0.
   *  @param  dVal   = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param  st     = Principal Components Data Structure
   *  @return iCol   = Columns with problems
   */

  public static int[] isData_rogueAbove( double dVal, pcDataStruct st )
  {
	int i,j;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCol[]   = null;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      iCol = new int[st.iColumns];
      for (i=0; i<st.iColumns; i++)
        iCol[i] = -1;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio > dMax)
		{
		  for (j=0; j<st.iColumns; j++)
		  {
			if (st.data[i][j] > 0.0)
              iCol[j] = 1;
		  }
		}
	  }
	}

	return (iCol);
  }

  /** Method getBrineData_rogueAbove()
   *  <p> This method will retrieve the brine data above the Anions/Cations Ratio of 1.0
   *      +/- dVal.
   *  @param dVal    = The decimal value limit, i.e. for +/-2%, dVal = .02.
   *  @param st      = Principal Components Data Structure
   *  @return stData = The Rogue Principal Components Data Structure
   */

  public static pcDataStruct getBrineData_rogueAbove( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
	  dMin = 1.0 - dVal;
	  dMax = 1.0 + dVal;

      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio > dMax)
		  iCount++;
	  }

	  stData          = new pcDataStruct();
      stData.iColumns = st.iColumns;
      stData.iRows    = iCount;

      stData.iBrine   = new int[stData.iColumns][2];

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      for (i=0; i<stData.iColumns; i++)
      {
		stData.iBrine[i] = st.iBrine[i];
	  }

      iCount = 0;
      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio > dMax)
		{
		  if (iCount < st.iRows)
		  {
			for (j=0; j<3; j++)
              stData.sData[iCount][j]  = new String( st.sData[i][j] );

            stData.dCations[iCount] = st.dCations[i];
            stData.dAnions[iCount]  = st.dAnions[i];

		    for (j=0; j<stData.iColumns; j++)
		    {
              stData.data[iCount][j]  = st.data[i][j];
			}
			iCount++;
		  }
		}
	  }

      stData.iRows   = iCount;
      stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method mergeBrineData()
   *  <p> This method merge the two data sets into one, with the Good first and
   *      Rogue second.
   *  @param stG     = "Good" Principal Components Data Structure
   *  @param stR     = "Rogue" Principal Components Data Structure
   *  @return stData = The Combination Principal Components Data Structure
   */

  public static pcDataStruct mergeBrineData( pcDataStruct stG, pcDataStruct stR )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    double       dMin     = 0.0;
    double       dMax     = 0.0;
    int          iCount   = 0;
    int          iTotal   = 0;

    if ((stG != null) && (stR != null))
    {
	  iTotal = stG.iRows + stR.iRows;

	  stData          = new pcDataStruct();
      stData.iColumns = stG.iColumns;
      stData.iRows    = iTotal;

      stData.iBrine   = new int[stData.iColumns][2];

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      for (i=0; i<stData.iColumns; i++)
      {
		stData.iBrine[i] = stG.iBrine[i];
	  }

      iCount = 0;
      for (i=0; i<stG.iRows; i++)
      {
  	    if (iCount < iTotal)
		{
		  for (j=0; j<3; j++)
            stData.sData[iCount][j]  = new String( stG.sData[i][j] );

          stData.dCations[iCount] = stG.dCations[i];
          stData.dAnions[iCount]  = stG.dAnions[i];

		  for (j=0; j<stData.iColumns; j++)
		  {
            stData.data[iCount][j]  = stG.data[i][j];
	      }
	  	  iCount++;
	    }
	  }

      for (i=0; i<stR.iRows; i++)
      {
  	    if (iCount < iTotal)
		{
		  for (j=0; j<3; j++)
            stData.sData[iCount][j]  = new String( stR.sData[i][j] );

          stData.dCations[iCount] = stR.dCations[i];
          stData.dAnions[iCount]  = stR.dAnions[i];

		  for (j=0; j<stData.iColumns; j++)
		  {
            stData.data[iCount][j]  = stR.data[i][j];
	      }
	  	  iCount++;
	    }
	  }

      stData.iRows   = iCount;
      stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method getBrineData_GT()
   *  <p> This method will retrieve the brine data Greater than a Anions/Cations Ratio.
   *  @param dVal = The ration of comparison
   *  @param st   = Principal Components Data Structure
   */

  public static pcDataStruct getBrineData_GT( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio > dVal)
		  iCount++;
	  }

	  stData          = new pcDataStruct();
      stData.iColumns = st.iColumns;
      stData.iRows    = iCount;

      stData.iBrine   = new int[stData.iColumns][2];

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      for (i=0; i<stData.iColumns; i++)
      {
		stData.iBrine[i] = st.iBrine[i];
	  }

      iCount = 0;
      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio > dVal)
		{
		  if (iCount < st.iRows)
		  {
		    for (j=0; j<3; j++)
              stData.sData[iCount][j]  = new String( st.sData[i][j] );

            stData.dCations[iCount] = st.dCations[i];
            stData.dAnions[iCount]  = st.dAnions[i];

		    for (j=0; j<stData.iColumns; j++)
		    {
              stData.data[iCount][j]  = st.data[i][j];
			}
			iCount++;
		  }
		}
	  }

      stData.iRows   = iCount;
      stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method getBrineData_LT()
   *  <p> This method will retrieve the brine data Greater than a Anions/Cations Ratio.
   *  @param dVal = The ration of comparison
   *  @param st   = Principal Components Data Structure
   */

  public static pcDataStruct getBrineData_LT( double dVal, pcDataStruct st )
  {
	int i,j;
	pcDataStruct stData   = null;
    double       dRatio   = 0.0;
    int          iCount   = 0;

    if (st != null)
    {
      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio < dVal)
		  iCount++;
	  }

	  stData          = new pcDataStruct();
      stData.iColumns = st.iColumns;
      stData.iRows    = iCount;

      stData.iBrine   = new int[stData.iColumns][2];

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

      for (i=0; i<stData.iColumns; i++)
      {
		stData.iBrine[i] = st.iBrine[i];
	  }

      iCount = 0;
      for (i=0; i<st.iRows; i++)
      {
	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
		if (dRatio < dVal)
		{
		  if (iCount < st.iRows)
		  {
		    for (j=0; j<3; j++)
              stData.sData[iCount][j]  = new String( st.sData[i][j] );

            stData.dCations[iCount] = st.dCations[i];
            stData.dAnions[iCount]  = st.dAnions[i];

		    for (j=0; j<stData.iColumns; j++)
		    {
              stData.data[iCount][j]  = st.data[i][j];
			}
			iCount++;
		  }
		}
	  }

      stData.iRows   = iCount;
      stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /** Method compute()
   *  <p> This method will compute the resultant raw data from the eigenvector
   *  @param  pcd      = Principal Components Data Structure
   *  @param  dataM    = Array of Adjusted Data X-XBar
   *  @param  evectors = Matrix of Eigenvectors
   *  @return stData   = Principal Components Data Structure
   */

   public static pcDataStruct compute( pcDataStruct pcd, double dataM[][], Matrix evectors )
   {
	 pcDataStruct stData = null;
	 int          iVal   = 0;

     if (pcd != null)
     {
	   stData          = new pcDataStruct();
	   stData.iRows    = pcd.iRows;
	   stData.iColumns = pcd.iColumns;

       stData.iBrine   = new int[stData.iColumns][2];

       stData.sData    = new String[stData.iRows][3];
       stData.data     = new double[stData.iRows][stData.iColumns];

       stData.dataM    = new double[stData.iColumns];

       stData.dAnions  = new double[stData.iRows];
       stData.dCations = new double[stData.iRows];

	   Matrix mdata    = setMatrix( pcd.iRows, pcd.iColumns, dataM );
	   Matrix mdataT   = mdata.transpose();

	   Matrix trans    = evectors.transpose();

 	   Matrix md       = evectors.times(mdataT);
 	   Matrix md2      = trans.times(md);

       Matrix md2T     = md2.transpose();
       stData.data     = md2T.getArray();

       for (int i=0; i<stData.iColumns; i++)
       {
	 	 stData.iBrine[i] = pcd.iBrine[i];
	   }

	   iVal = 0;
	   for (int i=0; i<pcd.iRows; i++)
	   {
		 for (int j=0; j<3; j++)
           stData.sData[i][j]  = new String( pcd.sData[i][j] );

         stData.dCations[i] = 0.0;
         stData.dAnions[i]  = 0.0;

	  	 for (int j=0; j<pcd.iColumns; j++)
		 {
		   iVal              = (int) (100.0 * (stData.data[i][j] + pcd.dataM[j]));
	       stData.data[i][j] = (double) iVal / 100.0;

		  if ((stData.iBrine[j][1] >= _LI) && (stData.iBrine[j][1] <= _NH4))
		    stData.dCations[i] = stData.dCations[i] + stData.data[i][j];
		  else if ((stData.iBrine[j][1] >= _F) && (stData.iBrine[j][1] <= _HSO4))
		    stData.dAnions[i]  = stData.dAnions[i]  + stData.data[i][j];
      	 }

		 stData.dCations[i] = (double)((int) (100.0 * stData.dCations[i] ) / 100.0);
		 stData.dAnions[i]  = (double)((int) (100.0 * stData.dAnions[i] ) / 100.0);
	   }

       stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
       stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
       stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	 }

	 return (stData);
   }

  /** Method compute()
   *  <p> This method will compute the resultant raw data from the eigenvector
   *  @param pcd      = Principal Components Data Structure
   *  @param dMean    = array of mean data values for each column
   *  @param dataM    = Array of Adjusted Data X-XBar
   *  @param evectors = Matrix of Eigenvectors
   *  @return stData  = Principal Components Data Structure
   */

   public static pcDataStruct compute( pcDataStruct pcd, double dMean[],
                                       double dataM[][], Matrix evectors )
   {
	 pcDataStruct stData = null;
	 int          iVal   = 0;

     if (pcd != null)
     {
	   stData          = new pcDataStruct();
	   stData.iRows    = pcd.iRows;
	   stData.iColumns = pcd.iColumns;

       stData.iBrine   = new int[stData.iColumns][2];

       stData.sData    = new String[stData.iRows][3];
       stData.data     = new double[stData.iRows][stData.iColumns];

       stData.dAdjust  = new double[stData.iRows][stData.iColumns];

       stData.dataM    = new double[stData.iColumns];

       stData.dAnions  = new double[stData.iRows];
       stData.dCations = new double[stData.iRows];

	   Matrix mdata    = setMatrix( pcd.iRows, pcd.iColumns, dataM );
	   Matrix mdataT   = mdata.transpose();

	   Matrix trans    = evectors.transpose();

 	   Matrix md       = evectors.times(mdataT);
 	   Matrix md2      = trans.times(md);

       Matrix md2T     = md2.transpose();
       stData.data     = md2T.getArray();

       for (int i=0; i<stData.iColumns; i++)
       {
	 	 stData.iBrine[i] = pcd.iBrine[i];
	   }

	   iVal = 0;
	   for (int i=0; i<pcd.iRows; i++)
	   {
		 for (int j=0; j<3; j++)
           stData.sData[i][j]  = new String( pcd.sData[i][j] );

         stData.dCations[i] = 0.0;
         stData.dAnions[i]  = 0.0;

	  	 for (int j=0; j<pcd.iColumns; j++)
		 {
		   if (dMean != null)
		     iVal = (int) (100.0 * (stData.data[i][j] + dMean[j]));
		   else
		     iVal = (int) (100.0 * (stData.data[i][j] + pcd.dataM[j]));
	       stData.data[i][j] = (double) iVal / 100.0;

		   if ((stData.iBrine[j][1] >= _LI) && (stData.iBrine[j][1] <= _NH4))
		     stData.dCations[i] = stData.dCations[i] + stData.data[i][j];
		   else if ((stData.iBrine[j][1] >= _F) && (stData.iBrine[j][1] <= _HSO4))
		     stData.dAnions[i]  = stData.dAnions[i]  + stData.data[i][j];
      	 }

		 stData.dCations[i] = (double)((int) (100.0 * stData.dCations[i] ) / 100.0);
		 stData.dAnions[i]  = (double)((int) (100.0 * stData.dAnions[i] ) / 100.0);
	   }

       stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
       stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
       stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	 }

	 return (stData);
   }

  /* ======================================================================= *
   * ----------------- LOAD BRINE DATA BY LIST OF MNEMONICS ---------------- *
   * ======================================================================= */

  /** Method getBrineData_meq()
   *  <p> This method will retrieve the brine data as arrays
   *  @param  iBrine = List of Brine Mnemonics for processing
   *  @param  st     = brine List Data Structure
   *  @return stData = Principal Components Data Structure
   */

  public static pcDataStruct getBrineData_meq( int iBrine[], brineListStruct st )
  {
	int i,j,k;
	pcDataStruct stData   = null;
    int          iColumns = 0;
    double       dValue   = 0.0;
    int          ion[]    = null;
    double       data[]   = null;
    double       dmeq[]   = null;

    int          iCount   = 0;
    int          iTotal   = 0;

    if (st != null)
    {
	  stData          = new pcDataStruct();

      stData.iRows    = st.iCount;
	  stData.iColumns = iBrine.length;

      stData.iBrine   = new int[stData.iColumns][2];

      stData.sData    = new String[stData.iRows][3];
      stData.data     = new double[stData.iRows][stData.iColumns];

      stData.dataM    = new double[stData.iColumns];

      stData.dAnions  = new double[stData.iRows];
      stData.dCations = new double[stData.iRows];

	  for (i=0; i<iBrine.length; i++)
	  {
        stData.iBrine[i][0] = i;
        stData.iBrine[i][1] = iBrine[i];
	  }

      for (i=0; i<iBrine.length; i++)
      {
		if ((iBrine[i] >= _LI) && (iBrine[i] < _SOLID))
		{
		  iCount++;
		}
	  }

	  iTotal = iCount;
	  iCount = 0;
      ion    = new int[iTotal];

      for (i=0; i<iBrine.length; i++)
      {
		if ((iBrine[i] >= _LI) && (iBrine[i] < _SOLID))
		{
		  if (iCount < iTotal)
		  {
  		    ion[iCount] = iBrine[i];
		    iCount++;
		  }
		}
	  }

      for (i=0; i<st.iCount; i++)
      {
		stData.sData[i][0] = new String( st.stItem[i].sAPI );
		stData.sData[i][1] = new String( st.stItem[i].sKID );
		stData.sData[i][2] = new String( st.stItem[i].sKEY );

        data = brine.math.brineMath.get_mg_per_liter(-1, i, ion, st );
        dmeq = brine.math.brineMath.mg_to_meq_per_liter(-1, ion, data );

        stData.dCations[i] = 0.0;
        stData.dAnions[i]  = 0.0;

        for (j=0; j<stData.iColumns; j++)
        {
		  for (k=0; k<ion.length; k++)
		  {
			if (ion[k] == stData.iBrine[j][1])
			{
			  stData.data[i][j] = dmeq[k];

  		      if ((stData.iBrine[j][1] >= _LI) && (stData.iBrine[j][1] <= _NH4))
		        stData.dCations[i] = stData.dCations[i] + stData.data[i][j];
		      else if ((stData.iBrine[j][1] >= _F) && (stData.iBrine[j][1] <= _HSO4))
		        stData.dAnions[i]  = stData.dAnions[i]  + stData.data[i][j];
			}
		  }

		  if (stData.iBrine[j][1] < _LI)
		  {
            stData.data[i][j] = st.getData( _PH, i );
		  }
		}

		stData.dCations[i] = (double)((int) (100.0 * stData.dCations[i] ) / 100.0);
		stData.dAnions[i]  = (double)((int) (100.0 * stData.dAnions[i] ) / 100.0);
	  }

	  stData.dataM   = computeMean( stData.iRows, stData.iColumns, stData.data );
      stData.dAdjust = adjustByMean( stData.iRows, stData.iColumns, stData.data,  stData.dataM );
      stData.dSD     = computeSD( stData.iRows, stData.iColumns, stData.dAdjust );
	}

	return (stData);
  }

  /* ======================================================================= *
   * --------------------------------- PRINT ------------------------------- *
   * ======================================================================= */

  /** Method print()
   *  <p> Print the Brine data
   *  @param st = the Principle Component Data Structure
   */

  public static void print( pcDataStruct st )
  {
	int i,j;
    double dRatio = 0.0;

	if (st != null)
	{
	  System.out.print("API-Number  ");
  	  for (j=0; j<st.iColumns; j++)
	  {
	    System.out.print(CURVES[st.iBrine[j][1]][1]+"  ");
	  }
	  System.out.print( "Cations, Anions, Ratio" );

	  System.out.println("");

      for (i=0; i<st.iRows; i++)
      {
		System.out.print(st.sData[i][0]+"  "+st.sData[i][1]+"  "+st.sData[i][2]);
  	    for (j=0; j<st.iColumns; j++)
	    {
	  	  System.out.print(st.data[i][j]+"  ");
	    }

	    dRatio = (double) ((int) ((100.0 * st.dAnions[i]) / st.dCations[i]) / 100.0);
	    System.out.println(" "+ st.dCations[i]+" "+st.dAnions[i]+"  "+dRatio);
	  }
	}
  }
}

/*
 *  @version 1.1 10/15/2015
 *  @author  John Victorine
 */
