/*
 * @plotBufferedWidget.java 11/04/2015
 *
 */

package plot;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.*;

/** Class plotBufferedWidget
 *  <p> Here is a great method which will capture the content
 *      of any Component visible or not.
 *      These methods were found on the internet an answer to
 *      a question for creating a table to a buffered image.
 */

public class plotBufferedWidget
{
  /** Method componentToImage()
   *  <p> Convert a Java Component to a Buffered image.
   *  @param  component = Component to convert to image
   *  @param  visible   = Component is visible
   *  @return img       = plot buffered image
   */

  public static BufferedImage componentToImage( Component component, boolean visible )
  {
	BufferedImage    img = null;
	Graphics2D       g2d = null;
	CellRendererPane crp = null;

    if (visible)
    {
      img = new BufferedImage( component.getWidth(),
                               component.getHeight(),
                               BufferedImage.TRANSLUCENT);
      g2d = (Graphics2D) img.getGraphics();
      g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON );
      component.paintAll(g2d);
    }
    else
    {
      component.setSize( component.getPreferredSize() );
      layoutComponent( component );

      img = new BufferedImage( component.getWidth(),
                               component.getHeight(),
                               BufferedImage.TRANSLUCENT );
      crp = new CellRendererPane();
      crp.add(component);
      crp.paintComponent( img.createGraphics(), component, crp, component.getBounds() );
    }

    return (img);
  }

  /** Method layoutComponent()
   *  @param  c = Java Component
   */

  private static void layoutComponent(Component c)
  {
    synchronized (c.getTreeLock())
    {
      c.doLayout();
      if (c instanceof Container)
      {
        for (Component child : ((Container) c).getComponents())
        {
          layoutComponent(child);
        }
      }
    }
  }
}


